#ifndef __samples_h__
#define __samples_h__

#include <system.h>

//===========================================================================================
// assert is used to check library error returns
#include <assert.h>

#ifdef __linux__
#include <conio.h>
//===========================================================================================

	#ifdef __PPC__
		#define _PCTyp( ) printf(" ###  PowerPC - LINUX Test  ###\n");

		#if !defined(EXPORT_RESULTS)
			#define EXPORT_RESULTS	1
		#endif

		#undef DISPLAY_RESULTS
		#define DISPLAY_RESULTS	0

	#else
		#define _PCTyp( ) printf(" ### DesktopPC - LINUX Text ###\n");

		#if !defined(DISPLAY_RESULTS)
			#define DISPLAY_RESULTS	1
		#endif
	#endif	// __PPC__


	#define _DataTimeInfo( SampleName )						\
		printf("\n ------------------------------\n");		\
    printf(" ### MATRIX-VISION GmbH (c) ###\n");		\
    _PCTyp( );											\
		printf(" ### " __DATE__ " / " __TIME__  " ###\n");	\
		printf(" ------------------------------\n");		\
    printf(" TEST: "#SampleName "\n");					\
    printf(" ------------------------------\n");

#else	// if win32
	#define _DataTimeInfo( SampleName )
#endif	// __linux__

#if !defined(USE_STOPWATCH)
	#define USE_STOPWATCH	1
#endif

#define _GetImgInfo( imgName )														\
		printf(" ==============================\n");								\
		printf("  Image Name: %s \n",imgName);										\
		IPL_LONG width=0,height=0,samples=0,bits=0;									\
		IPL_FlInquire(FL_DONT_CARE, IPL_WIDTH,				&width,		imgName);	\
		IPL_FlInquire(FL_DONT_CARE, IPL_HEIGHT,				&height,	imgName);	\
		IPL_FlInquire(FL_DONT_CARE,	IPL_SAMPLES_PER_PIXEL,	&samples,	imgName);	\
		IPL_FlInquire(FL_DONT_CARE,	IPL_BITS_PER_SAMPLE,	&bits,		imgName);	\
		printf("   X     /   Y    S   Bit\n");										\
		printf(" %4ld    / %4ld  %2ld  %3ld\n",width,height,samples,bits);			\
		printf(" ==============================\n");

//===========================================================================================
#if USE_STOPWATCH
	#ifdef __linux__
		#include <stopwatch.h>
		#undef _StartWatch
		#undef _StopWatch

		#ifndef GET_VAL
			#define _StartWatch()		do { 																	\
											struct timeval start_tv; StartTimer ( &start_tv )
			#define _StopWatch(mesg)		ElapsedMessage ( &start_tv, mesg, 30 ) ;							\
										} while(0)
		#else	//// GET_VAL
			//extern double run_time;

			#define _StartWatch()		do { 																\
											struct timeval start_tv, diff_tv; /*char fformat[50]; */StartTimer ( &start_tv )
			#define _StopWatch(mesg)		ElapsedTime ( &start_tv, &diff_tv, 30 );						\
											printf( "= %-*s : %13ld usec\n", ( 30 & 0xff ), mesg,			\
															diff_tv.tv_sec * 1000000 + diff_tv.tv_usec );	\
										} while(0)

											//sprintf( fformat, "%2ld.%06ld", diff_tv.tv_sec, diff_tv.tv_usec );
											//run_time = atof( fformat );
											//printf( "= %-*s : %f sec\n", ( 30 & 0xff ), mesg, run_time,);
		#endif	// GET_VAL
	#else // if win32
		#if 1
			#define _StartWatch()		do { 													    \
											 LARGE_INTEGER start_t, stop_t, diff, perffreq;                    \
											 double timeins;				\
											 QueryPerformanceFrequency (&perffreq);			\
											QueryPerformanceCounter (&start_t);
			#define _StopWatch(mesg)		QueryPerformanceCounter (&stop_t);		\
																	diff.QuadPart = stop_t.QuadPart - start_t.QuadPart;                  \
																	timeins = ((double)(diff.QuadPart) / (double)(perffreq.QuadPart)); \
																				printf ( "= %-*s : %4.6f sec\n", 30, mesg, timeins) ;	\
										} while(0)
		#else
			#define _StartWatch()		do { 													    \
											unsigned long start_msec, used_msec;                    \
											start_msec = GetTickCount();
			#define _StopWatch(mesg)		used_msec = GetTickCount()-start_msec;                  \
																				printf ( "= %-*s : %f sec\n", 30, mesg, ((float)used_msec/(float)1000 )) ;	\
										} while(0)
		#endif // 1
	#endif	//__linux__
#else
	#define _StartWatch()	printf("start dummy\n");
	#define _StopWatch(mesg) printf("stop dummy %s\n", mesg );
#endif	// USE_STOPWATCH


//===========================================================================================
#if EXPORT_RESULTS
	#define __ProcessResult(fct)	\
			assert(IPL_FlExport(DstImage##fct,IPL_DONT_CARE, IPL_BMP, #fct".bmp"));

#elif DISPLAY_RESULTS
	#define __ProcessResult(fct)		\
		IPL_HWND IPLWND##fct;			\
		assert(DISP_OpenWindow(DstImage##fct, &IPLWND##fct, IPLWND##fct, (IPL_VOID *)" "#fct ));

#else
	#define __ProcessResult(fct)

#endif	// DISPLAY_RESULTS


//===========================================================================================
#define _intarg( var )		do { if( argc-- > 1 )		var = atoi( *argv++ ); } while(0)
#define _strarg( var )		do { if( argc-- > 1 )		var = *argv++; } while(0)
#define _hexarg( var )		do { if( argc-- > 1 )		var = strtol( *argv++, NULL, 0 );} while(0)
#define _skiparg()		   	argv++

//===========================================================================================
/// Print the error message of the \b SDK call.
#define PRINTSDKERRORMSG( _call )																	\
	{																								\
		fprintf( stderr, "*** %s (%s: %d): \"%s\"\n", __FUNCTION__, __FILE__, __LINE__, #_call );	\
		fprintf( stderr, "***\t\"%s\" (code: %ld)\n",												\
						IPL_GetErrorString(IPL_GetErrorCode2()), IPL_GetErrorCode2());				\
	}

//===========================================================================================
/// Check the \b SDK call and \c abort if an error occurred.
#ifndef CHECKSDKCALL
	#define CHECKSDKCALL( call )			\
		{									\
			if( IPL_FALSE == ( call ))		\
			{								\
				PRINTSDKERRORMSG( call );	\
				abort( );					\
			}								\
		}
#endif	// CHECKSDKCALL

//===========================================================================================
/// Check the \b SDK call. If an error occurred \c close the opened \c device and \c abort.
#ifndef CHECKDEVSDKCALL
	#define CHECKDEVSDKCALL( _call, _dev )		\
		{										\
			if( IPL_FALSE == ( _call ))			\
			{									\
				PRINTSDKERRORMSG( _call );		\
				if( _dev != IPL_DONT_CARE )		\
				{								\
					DEV_Close( _dev );			\
					_dev = IPL_DONT_CARE;		\
				}								\
				abort( );						\
			}									\
		}
#endif	// CHECKDEVSDKCALL

//===========================================================================================

#endif	// __samples_h__


