#ifndef MVIMPACT_OCR_IMPL_H_
#define MVIMPACT_OCR_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace ocr {


inline OcrCharacter::OcrCharacter(const Image & pattern, int ascii)
	: m_pattern(pattern), m_ascii(ascii)
{
}

inline Image OcrCharacter::getPattern() const
{
	return m_pattern;
}
	
inline OcrCharacter& OcrCharacter::setPattern(const Image & pattern)
{
	m_pattern = pattern;
	return *this;
}


inline int OcrCharacter::getAscii() const
{
	return m_ascii;
}

	
inline OcrCharacter& OcrCharacter::setAscii(int ascii)
{
	m_ascii = ascii;
	return *this;
}

	

inline OcrResult::OcrResult()
{ }

inline OcrResult::OcrResult(int ascii, double quality)
  : m_ascii(ascii), m_quality(quality),m_angle(0) 
{
}

inline OcrResult::OcrResult(int ascii, double quality, double angle)
        : m_ascii(ascii), m_quality(quality),m_angle(angle) 
{
}


inline int OcrResult::getAscii() const
{
	return m_ascii;
}
	
inline OcrResult& OcrResult::setAscii(int ascii)
{
	m_ascii = ascii;
	return *this;
}
	
inline double OcrResult::getQuality() const
{
	return m_quality;
}
	
inline OcrResult& OcrResult::setQuality(double quality)
{
	m_quality = quality;
	return *this;
}

inline double OcrResult::getAngle() const
{
	return m_angle;
}
	
inline OcrResult& OcrResult::setAngle(double angle)
{
        m_angle = angle;
	return *this;
}


inline OcrResult::operator mvIMPACT_C::OCR_RESULT() const 
{
	mvIMPACT_C::OCR_RESULT result;

	result.ascii = getAscii();
	result.quality = (mvIMPACT_C::IPL_FLOAT)getQuality();
	result.angle = (mvIMPACT_C::IPL_FLOAT)getAngle();

	return result;
}


inline OcrResult::OcrResult(const mvIMPACT_C::OCR_RESULT& r)
        : m_ascii(r.ascii), m_quality(r.quality),m_angle(r.angle)
{ }


//-----------------------------------------------------------------------------
inline void SegmentList::sort
(
	const OcrFontDatabase&  db,
	const Image&        S
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::OCR_SortSegment(
			db.getHandle(),
			S.getHandle(),
			getData(),
			getSize()))
		throwImpactError();
}

//-----------------------------------------------------------------------------
inline void SegmentList::split
(
	const OcrFontDatabase&  db,
	const Image&        S
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_LONG count = (mvIMPACT_C::IPL_LONG)getSize();
	if (!mvIMPACT_C::OCR_SplitSegment(
			db.getHandle(),
			S.getHandle(),
			getCapacity(),
			getData(),
			&count))
		throwImpactError();

	// Store new size.
	setSize (count);
}

//-----------------------------------------------------------------------------
inline void SegmentList::merge
(
	const OcrFontDatabase&  db,
	const Image&        S
)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_LONG count = (mvIMPACT_C::IPL_LONG)getSize();
	if (!mvIMPACT_C::OCR_MergeSegment(
			db.getHandle(),
			S.getHandle(),
			getCapacity(),
			getData(),
			&count))
		throwImpactError();

	// Store new size.
	setSize (count);
}


inline SegmentList::SegmentList (int capacity )
	{
		if (capacity<=0) capacity=100;
		m_pData     = new mvIMPACT_C::IPL_RECT[capacity];
		m_iCapacity = capacity;
		m_iSize     = 0;
	}

inline SegmentList::~SegmentList ()
{
	// a destructor must never throw
	try
	{
		dispose();
	}
	catch(...)
	{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
	}
}

inline SegmentList::SegmentList (const SegmentList& s)
{
	copy(s);
}

inline SegmentList& SegmentList::operator= (const SegmentList& rhs)
{
	if(this != &rhs) 
	{
		dispose();
		copy(rhs);
	}
	
	return *this;
}



inline int SegmentList::getSize () const
	{
		return m_iSize;
	}

inline SegmentList& SegmentList::setSize (int size)
	{
		if(size < 0) { throwImpactException("Size < 0"); }
		m_iSize = size;
		if (m_iSize > m_iCapacity) m_iSize = m_iCapacity;
		return *this;
	}

inline int SegmentList::getCapacity () const
	{
		return m_iCapacity;
	}

inline SegmentList& SegmentList::setCapacity (int capacity)
	{
		if(capacity < 0) { throwImpactException("Capacity < 0"); }
		if (m_iCapacity == capacity) return *this;

		// Shrinking is no problem.
		if (capacity < m_iCapacity)
		{
			m_iCapacity = capacity;
			if (m_iSize > m_iCapacity)
			{
				m_iSize = m_iCapacity;
			}
		}
		else
		// Enlarging requires to copy the data to a new buffer.
		{
			mvIMPACT_C::IPL_RECT* pTmp = new mvIMPACT_C::IPL_RECT[capacity];
			::memcpy (pTmp, m_pData, sizeof(mvIMPACT_C::IPL_RECT)*m_iCapacity);
			delete [] m_pData;

			m_pData     = pTmp;
			m_iCapacity = capacity;
		}
		return *this;
	}

inline Rectangle SegmentList::getSegment 
	(
		int index
	) const
	{
		if (index < 0 || index >= m_iSize)
		{
			throwImpactException("Index out of range");
		}

		return m_pData[index];
	}

inline SegmentList& SegmentList::setSegment 
	(
		int index, 
		const Rectangle& r
	)
	{
		if (index >= m_iCapacity)
		{
			throwImpactException("Index out of range");
		}

		m_pData[index] = r;
		return *this;
	}

inline mvIMPACT_C::IPL_RECT* SegmentList::getData() const
	{
		return m_pData;
	}


inline void SegmentList::dispose()
{
		delete[] m_pData;
}
	
inline void SegmentList::copy(const SegmentList& src)
{
	m_pData     = new mvIMPACT_C::IPL_RECT[src.m_iCapacity];
	m_iCapacity = src.m_iCapacity;
	m_iSize     = src.m_iSize;
	::memcpy (m_pData, src.m_pData, sizeof(mvIMPACT_C::IPL_RECT)*m_iSize);
}


inline 	OcrFontDatabase::OcrFontDatabase (
		const std::string& ocrDbFilename,
		bool bMustExist ) 
		: handle_(-1), counter_(new int(1))
	{
		assert(unassigned == getHandle());
		if (!mvIMPACT_C::OCR_Open(
				&handle_, const_cast<char*>(ocrDbFilename.c_str()), bMustExist ? 0l : 1l))
		throwImpactError();
		assert(unassigned != getHandle());

		// Default classification method to correlation.
		setClassificationType (ocCorrelation);
	}

inline OcrFontDatabase::OcrFontDatabase()
		: handle_(unassigned), counter_(new int(1))		 
	{ 
	}

inline OcrFontDatabase::~OcrFontDatabase()
	{
		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}

inline mvIMPACT_C::OCR_BUFHANDLE OcrFontDatabase::getHandle  () const { return handle_; }

inline int OcrFontDatabase::getUseCount() const { return *counter_; }

inline bool OcrFontDatabase::getIsUnique   () const { return 1 == *counter_; }

inline OcrFontDatabase& OcrFontDatabase::operator=(const OcrFontDatabase& r)
	{ 
		if (&r != this)
		{
			dispose();
			copy(r);
		}
		return *this;
	}

inline OcrFontDatabase::OcrFontDatabase(const OcrFontDatabase& r) 
{
		copy(r);
}

inline void OcrFontDatabase::addCharacter 
	(
		const Image& S, 
		const Rectangle& segment,
		int              ascii
	)
	{
		mvIMPACT_C::IPL_RECT iplSegment = segment;
		if (!mvIMPACT_C::OCR_Teach (
				getHandle(), 
				S.getHandle(),
				&iplSegment, 
				ascii))
			throwImpactError();
	}

inline void OcrFontDatabase::delCharacter 
	(
		int index
	)
	{
		if (!mvIMPACT_C::OCR_DelPattern (getHandle(), index))
			throwImpactError();
	}

inline OcrCharacter OcrFontDatabase::getCharacterInfo
	(
		int index
	) const
	{
		mvIMPACT_C::OCR_PATTERN info;

		if (!mvIMPACT_C::OCR_GetPattern (getHandle(), index, &info ))
			throwImpactError();
		
		Image pattern((int)info.w,(int)info.h);

		if (!mvIMPACT_C::IPL_PutBufData( pattern.getHandle(), 0, 0, 0, 0, 0, info.w, info.h, info.data))
			throwImpactError();
		
		OcrCharacter ocrChar;
        ocrChar.setPattern(pattern);
        ocrChar.setAscii(info.ascii);

		return ocrChar;
	}


inline void OcrFontDatabase::segment
	(
		const Image&        S,
		SegmentList&            segmentList
	)
	{
		mvIMPACT_C::IPL_LONG count;
		if (!mvIMPACT_C::OCR_Segment(
				getHandle(), 
				S.getHandle(), 
				segmentList.getCapacity(), 
				segmentList.getData(),
				&count))
			throwImpactError();

		// Store new size.
		segmentList.setSize(count);
	}

inline OcrResult OcrFontDatabase::readChar
	(
		const Image&        S,
		const Rectangle&        segment
	) const
	{
		mvIMPACT_C::OCR_RESULT result;

		mvIMPACT_C::IPL_RECT iplSegment = segment;
		mvIMPACT_C::IPL_LONG count = 1;
		if (!mvIMPACT_C::OCR_ReadChar(
				getHandle(),
				S.getHandle(),
				&iplSegment,
				&count,
				&result))
			throwImpactError();

		return result;
	}

inline OcrResult OcrFontDatabase::compareChar
	(
		const Image&        S,
		const Rectangle&        segment,
		int index
	) const
	{
		mvIMPACT_C::OCR_RESULT result;
		mvIMPACT_C::IPL_RECT iplSegment = segment;
		if (!mvIMPACT_C::OCR_CompareChar(
				getHandle(),
				S.getHandle(),
				&iplSegment,
				(mvIMPACT_C::IPL_LONG)index,
				&result))
			throwImpactError();

		return result;
	}


inline std::vector<OcrResult> OcrFontDatabase::readChar
	(
		const Image&        S,
		const Rectangle&        segment,
		int                     numberOfResults
	) const
	{
		if (numberOfResults < 1)
		{
			throwImpactException("Number of results < 1");
		}

		DynArray<mvIMPACT_C::OCR_RESULT, OcrResult> tmpResult(numberOfResults);

		mvIMPACT_C::IPL_LONG count = numberOfResults;
		mvIMPACT_C::IPL_RECT iplSegment = segment;
		if (!mvIMPACT_C::OCR_ReadChar(
				getHandle(),
				S.getHandle(),
				&iplSegment,
				&count,
				tmpResult.getPointer()))
			throwImpactError();
		std::vector<OcrResult> result = tmpResult.asVector();

		//remove the empty elements from the vector
		for (int elements = numberOfResults; elements>count; elements--) {
			result.pop_back();
		}

		return result;
	}

inline std::string OcrFontDatabase::readString
	(
		const Image&        S,
		const SegmentList&  segmentList
	) const
{
	DynArray<mvIMPACT_C::IPL_CHAR> iplResult(segmentList.getSize()*2 + 1);
	
	if(!mvIMPACT_C::OCR_ReadString(
			getHandle(),
			S.getHandle(),
			segmentList.getData(),
			segmentList.getSize(),
			iplResult,
			iplResult.getSize()))
	{ throwImpactError(); }
	
	return std::string(iplResult);
}


inline std::vector<double> OcrFontDatabase::verifyChar
	(
		const Image&        S,
		const Rectangle&        segment,
		const std::vector<int>& vAscii
	) const
	{
		if (vAscii.size() == 0)
		{
			throwImpactException("No character specified");
		}

		int size = (int)vAscii.size();

		// Create temporary OCR_RESULT buffer and fill in ascii values.
		DynArray<mvIMPACT_C::OCR_RESULT> pResult(size);
		// one cannot use an auto_ptr here, because it is an array, which must
		// be deleted with delete[] and not with delete (!).
		// Therefore, the delete[] must be called manually before either throwing
		// an exception or returning from the function
		int i;
		for (i=0; i<size; ++i) pResult.getPointer()[i].ascii = vAscii[i];

		mvIMPACT_C::IPL_RECT iplSegment = segment;
		if (!mvIMPACT_C::OCR_VerifyChar(
				getHandle(),
				S.getHandle(),
				&iplSegment,
				size,
				pResult.getPointer()))
		{
			throwImpactError();
		}

		std::vector<double> vResult;
		vResult.resize(size);

		// Copy the quality values.
		for (i=0; i<size; ++i) { vResult[i] = pResult.getPointer()[i].quality; }

		return vResult;
	}

inline int OcrFontDatabase::getThreshold() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRSEG_THRESHOLD, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setThreshold(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRSEG_THRESHOLD, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline bool OcrFontDatabase::getBlackOnWhite() const 
	{
		mvIMPACT_C::IPL_BOOL iplResult;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRSEG_CHAR_COLOR, (void*)&iplResult))
		{ throwImpactError(); }
		
		return 0 == iplResult;
	}

inline OcrFontDatabase& OcrFontDatabase::setBlackOnWhite
	(
		bool value
	)
	{
		mvIMPACT_C::IPL_BOOL iplValue = !value;
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRSEG_CHAR_COLOR, (void*)& iplValue))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMinSegWidth() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRSEG_MINWIDTH, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMinSegWidth(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRSEG_MINWIDTH, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMaxSegWidth() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRSEG_MAXWIDTH, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMaxSegWidth(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRSEG_MAXWIDTH, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMinSegHeight() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRSEG_MINHEIGHT, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMinSegHeight(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRSEG_MINHEIGHT, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMaxSegHeight() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRSEG_MAXHEIGHT, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMaxSegHeight(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRSEG_MAXHEIGHT, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMinCharWidth() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRCHAR_MINWIDTH, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMinCharWidth(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRCHAR_MINWIDTH, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMaxCharWidth() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRCHAR_MAXWIDTH, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMaxCharWidth(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRCHAR_MAXWIDTH, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMinCharHeight() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRCHAR_MINHEIGHT, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMinCharHeight(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRCHAR_MINHEIGHT, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getMaxCharHeight() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRCHAR_MAXHEIGHT, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setMaxCharHeight(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRCHAR_MAXHEIGHT, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline bool OcrFontDatabase::getSortFlag() const 
	{
		mvIMPACT_C::IPL_BOOL iplResult;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCRSEG_SORTFLAG, (void*)&iplResult))
		{ throwImpactError(); }

		return (0 != iplResult);
	}

inline OcrFontDatabase& OcrFontDatabase::setSortFlag(bool value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCRSEG_SORTFLAG, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getCharacterCount() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCR_PATTERNNUMBER, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrClassification OcrFontDatabase::getClassificationType() const 
	{
		OcrClassification v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCR_CLASSIFICATIONTYPE, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setClassificationType(OcrClassification value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCR_CLASSIFICATIONTYPE, (void*)& value))
		{ throwImpactError(); }
		return *this;
	}

inline bool OcrFontDatabase::getNormalizeFlag() const 
	{
		mvIMPACT_C::IPL_BOOL iplResult;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCR_NORMALIZEFLAG, (void*)&iplResult))
		{ throwImpactError(); }

		return (0 != iplResult);
	}

inline OcrFontDatabase& OcrFontDatabase::setNormalizeFlag(bool value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCR_NORMALIZEFLAG, (void*)&value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getNormalizeWidth() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCR_NORMWIDTH, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setNormalizeWidth(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCR_NORMWIDTH, (void*)&value))
		{ throwImpactError(); }
		return *this;
	}

inline int OcrFontDatabase::getNormalizeHeight() const 
	{
		int v;
		if(!mvIMPACT_C::OCR_Inquire(getHandle(), mvIMPACT_C::OCR_NORMHEIGHT, (void*)&v))
		{ throwImpactError(); }
		return v;
	}

inline OcrFontDatabase& OcrFontDatabase::setNormalizeHeight(int value) 
	{
		if(!mvIMPACT_C::OCR_Control(getHandle(), mvIMPACT_C::OCR_NORMHEIGHT, (void*)&value))
		{ throwImpactError(); }
		return *this;
	}

inline OcrFontDatabase::OcrFontDatabase(
		mvIMPACT_C::OCR_BUFHANDLE bufhandle) 
		: handle_(-1), counter_(new int(1))
	{
		assert(unassigned == getHandle());
		handle_  = bufhandle;
		assert(unassigned != getHandle());
	}


inline void OcrFontDatabase::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			if (unassigned != getHandle())
			{
				assert(unassigned != getHandle());
				if (!mvIMPACT_C::OCR_Close(getHandle())) throwImpactError();
				handle_ = unassigned;
				assert(unassigned == getHandle());
			}
		}
	}


inline void OcrFontDatabase::copy(const OcrFontDatabase& src)
{
	handle_ = src.handle_;
	counter_ = src.counter_;
	
	(*counter_)++;	
}


inline std::string getVersionOCR()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::OCR_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}


	} // namespace ocr
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_OCR_IMPL_H_
