#! /usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page affinetrans_page Affinetrans example

\section affinetrans_section_purpose Purpose
This example shows how to do an geometric transformation on an image.

\section affinetrans_section_intro What it does
The example shows an image. If a key is pressed the image is rotated and is displayed again.

\section affinetrans_section_description How it works
First the image is loaded and displayed. Then the sample waits for user input.

\code
## load any supported file format
image = loadImage("screw.bmp")

## create a window containing the loaded image:
window = Window(image)

## wait for user to press space
runMessageLoop()
\endcode

Then the matrix is defined. Assuming that the elements have the following ordering

\verbatim
    m11 m12 m13
    m21 m22 m23
    m31 m32 m33
\endverbatim

the new pixels are calculated using the following equations

\verbatim
x' = m11*x + m12*y + m13
y' = m21*x + m22*y + m23
\endverbatim

when calling base.matrix.

\code
## now populate a matrix that shifts the coordinate system by (150, -150) pixels
## and then rotates the image by 30 degrees around the top-left corner
sin30 = sin( radians(30) )
cos30 = cos( radians(30) )

r = (
    cos30, -sin30,  150.0,
    sin30,  cos30, -150.0,
      0.0,    0.0,    1.0 )

rot = MatrixDouble(3, 3, r)

## perform affine transformation
dest = matrix(image, rot)
\endcode

After that the new image, that was stored in variable \c dest is displayed and the sample
waits until the user presses a key again.

\code
## create a window containing the transformed image:
window2 = Window(dest)

## keep sample alive until console window is closed
## or space bar is pressed
runMessageLoop()

del window2
del window
\endcode

\section affinetrans_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from math import sin, cos, radians

## load any supported file format
image = loadImage("screw.bmp")

## create a window containing the loaded image:
window = Window(image)

## wait for user to press space
runMessageLoop()

## now populate a matrix that shifts the coordinate system by (150, -150) pixels
## and then rotates the image by 30 degrees around the top-left corner
sin30 = sin( radians(30) )
cos30 = cos( radians(30) )

r = (
    cos30, -sin30,  150.0,
    sin30,  cos30, -150.0,
      0.0,    0.0,    1.0 )

rot = MatrixDouble(3, 3, r)

## perform affine transformation
dest = matrix(image, rot)

## create a window containing the transformed image:
window2 = Window(dest)

## keep sample alive until console window is closed
## or space bar is pressed
runMessageLoop()

del window2
del window

#</matrixdoc>
