#! /usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page aoi_page AOI example (Python)

\section aoi_section_purpose Purpose
The AOI example shows, how different types of objects of interests can be used.
Additionally the usage of some shortcut notation in Python is shown.

\section aoi_section_intro What it does
This sample shows how to create areas of interest and lines of interest and
shows how to change some properties. Additionally the use of tuples instead
of constructors, which is allowed in some cases is shown. Furthermore
the usage of properties instead of "getter" and "setter" methods is presented.

\section aoi_section_description How it works
First of all an image is loaded into memory and a window is created to show
the image.

\code
#load image
image = loadImage("screw.bmp")

# create a window containing the loaded image:
window = Window(image)
\endcode

Then an AreaOfInterest object is created, which is associated to \c window.
The corresponding area of interest is then shown in this window.

\code
aoi1 = AreaOfInterest(window, Rectangle(10, 10, 110, 110))
\endcode

Afterwards some properties for the created AreaOfInterest object are set.

\code
aoi1.setColor(RGBColor(0, 255, 0))
aoi1.text = "Rectangle 1"
aoi1.setTextColor(RGBColor(255, 0, 0))
\endcode

Then another AreaOfInterest object is created, similar to the first one.
\code
aoi2 = AreaOfInterest(window, (210, 210, 310, 310))
aoi2.color = (255, 0, 0)
aoi2.text = "Rectangle 2"
aoi2.textColor = (255, 0, 0)
\endcode

Note that, here, the constructors of base.Rectangle and
base.RGBColor have been replaced by tuples. This shortcuts
can be used whenever one of these classes has to be provided as an argument for
a function. The classes which can be replaced by such tuples are listed
at \todo \ref PYTHON_SPECIAL.

Afterwards, a LineOfInterest object is created.

\code
loi1 = LineOfInterest(window, Line(Point(150, 50), Point(150, 150)))
loi1.color = RGBColor(0, 0, 255)
loi1.text = "Line 1"
loi1.textColor = RGBColor(255, 0, 0)
\endcode

Here, the "setter" functions have been replaced by assignments to properties.
For details about properties see \todo \ref PYTHON_SPECIAL

Then another LineOfInterest object is created.
\code
loi2 = LineOfInterest(window, Line((350, 250), (350, 350)))
loi2.color = (255, 255, 255)
loi2.text = "Line 2"
loi2.textColor = (255, 0, 0)
\endcode

Note that a nested tuple construct like \c ((350, 250), (350, 350)) is
not allowed instead of a Line object. The short form only applies for
explicit calls of functions or constructors.

Finally, the message loop of the windowing system is called. Now you can
interactively change the position and size (or length respectively) of the
generated objects of interest. Finally, press space to return from
the message loop function.

\code
# wait for event (space key in main window)
runMessageLoop()
\endcode

Finally the window object is explicitly deleted and with it the window disappears from
screen. At the end of the script this needs not to be done, because all objects
are automatically destroyed by the Python interpreter on program exit.

\code
del window
\endcode

\section aoi_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

#load image
image = loadImage("screw.bmp")

# create a window containing the loaded image:
window = Window(image)

# create an area of interest:
aoi1 = AreaOfInterest(window, Rectangle(10, 10, 110, 110))
aoi1.setColor(RGBColor(0, 255, 0))
aoi1.text = "Rectangle 1"
aoi1.setTextColor(RGBColor(255, 0, 0))

#create second area of interest
aoi2 = AreaOfInterest(window, (210, 210, 310, 310))
aoi2.setColor((255, 0, 0))
aoi2.text = "Rectangle 2"
aoi2.setTextColor((255, 0, 0))

#create a line of interest
loi1 = LineOfInterest(window, Line(Point(150, 50), Point(150, 150)))
loi1.color = RGBColor(0, 0, 255)
loi1.text = "Line 1"
loi1.textColor = RGBColor(255, 0, 0)

#create a line of interest
loi2 = LineOfInterest(window, Line((350, 250), (350, 350)))
loi2.color = (255, 255, 255)
loi2.text = "Line 2"
loi2.textColor = (255, 0, 0)

# wait for event (space key in main window)
runMessageLoop()

# close and destroy Window:
del window

#</matrixdoc>
