#! /usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Blob
\endxmlonly

\page blob_page Blob example (Python)

\section blob_section_purpose Purpose
This example shows how to calculate some simple features of the blobs in an image.

\section blob_section_intro What it does
The example calculates for each blob in the input image the coordinates of the extreme box and
the area.

\section blob_section_description How it works
First of all an image is loaded and binarized using a simple fixed threshold. Then a window is
displayed to show the binarized image.

\code
#load image
image = loadImage("money.tif")
image = compareGreater(image, 80)

#display image
window = Window(image)

#wait until space was pressed
runMessageLoop("Hit the space bar to perform blob analysis.\n")
\endcode

A new BlobFeatureList object is allocated to hold the list of features to be calculated. Then
the two features "area" and "extreme box" are added to the list. The add* member functions of
BlobFeatureList return IDs. These values are used to identify the features later when the results
are to be extracted (see below). The IDs are actually integer values.

\code
#set the features
features = BlobFeatureList()
areaID = features.addArea()
boxID = features.addExtremeBox()

\endcode

Then the analysis is started and the number of blobs in the image is retrieved. Note if there were
more than one image band, slice or frame, then for each such dimension a separate blob analysis is
performed by blob.analyze. The functions which are used to retrieve the results then provide
the values for the band, slice and frame they want the results from. The dimension arguments default to 0
in the functions to avoid tedious typing for the most common case of gray-valued images.

\code
#run blob analysis
result = analyze(image, features)

#get the number of blobs
numberOfBlobs = result.getNumberOfBlobs(0, 0, 0)
\endcode

Finally, the results are displayed for each blob. The called variant of analyze labels the input image, thus
it has to be thresholded again. Another possibility would be to set a special color palette to show the labelled
blobs. After all the message loop is run and finally, the window is destroyed.

\code
#display results
for i in range(numberOfBlobs):
    area = result.getSingleResult(areaID, i)
    box = result.getSingleResult(boxID, i)
    areaValue = area[0]
    left, top, right, bottom = box
    print "Object: " , i, "\t Area: ", areaValue, "\t Bounds: ", left, top, right, bottom

compareGreater(image, image, 0)

runMessageLoop()

del window

\endcode

\section blob_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.blob import *

#load image
image = loadImage("money.tif")
image = compareGreater(image, 80)

#display image
window = Window(image)

#wait until space was pressed
runMessageLoop("Hit the space bar to perform blob analysis.\n")

#set the features
features = BlobFeatureList()
areaID = features.addArea()
boxID = features.addExtremeBox()

#run blob analysis
result = analyze(image, features)

#get the number of blobs
numberOfBlobs = result.getNumberOfBlobs(0, 0, 0)

#display results
for i in range(numberOfBlobs):
    area = result.getSingleResult(areaID, i)
    box = result.getSingleResult(boxID, i)
    areaValue = area[0]
    left, top, right, bottom = box
    print "Object: " , i, "\t Area: ", areaValue, "\t Bounds: ", left, top, right, bottom

compareGreater(image, image, 0)

runMessageLoop()

del window

##</matrixdoc>
