#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Calibration
\endxmlonly

\page calib_page Calibration example (Python)

\section calib_section_purpose Purpose
This sample shows how to use class CameraCalibration to calculate calibration information
and calibrate an image

\section calib_section_intro What it does
The sample uses a calibration grid to calculate the calibration information and afterwards
calculate a calibrated version of the image for the grid image.

\section calib_section_description How it works
First the calibration image is loaded from disk and displayed in a Window. Then a new
CameratCalibration instance is generated and the user is requested to start the calibration
algorithm by pressing the space key.

\code
## Load the grid image.
image = loadImage("grid04.bmp")

## Display the grid image.
window = Window(image)

## Allocate a calibration buffer. All defaults are set.
calibration = CameraCalibration()

runMessageLoop("Press space to teach calibration data\n")
\endcode

Then the teach process is started. Afterwards some statistics are displayed and
the program waits for space to be pressed.

\code
## Start the teach process.
calibration.teachByGrid(image)

## Read some results.
icnt = calibration.iterationCount
print "iteration count: %d" % icnt

stat = calibration.statistics

print "meanXDifference: %8.4f meanStddevXDifference: %8.4f" % (
    stat.meanXDifference, stat.stddevXDifference
    )
print "meanYDifference: %8.4f meanStddevYDifference: %8.4f" % (
    stat.meanYDifference, stat.stddevYDifference
    )
print "meanEuclideanDistance: %8.4f stddevEuclideanDistance: %8.4f" % (
    stat.meanEuclideanDistance, stat.stddevEuclideanDistance
    )

runMessageLoop("Press space to transform calibration grid\n")

\endcode

Finally the grid image is transformed using the calculated calibration data. Then
the window is updated for the transformed image to be displayed. Afterwards the
program waits for a key to be pressed and destroys the window.

\code
## Transform/calibrate the image.
calibration.transformImage(image, image)

## Update the display.
window.redraw()

runMessageLoop("Press space to quit program")
del window
\endcode

\section calib_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.calibration import *

## Load the grid image.
image = loadImage("grid04.bmp")

## Display the grid image.
window = Window(image)

## Allocate a calibration buffer. All defaults are set.
calibration = CameraCalibration()

runMessageLoop("Press space to teach calibration data\n")

## Start the teach process.
calibration.teachByGrid(image)

## Read some results.
icnt = calibration.iterationCount
print "iteration count: %d" % icnt

stat = calibration.statistics

print "meanXDifference: %8.4f meanStddevXDifference: %8.4f" % (
    stat.meanXDifference, stat.stddevXDifference
    )
print "meanYDifference: %8.4f meanStddevYDifference: %8.4f" % (
    stat.meanYDifference, stat.stddevYDifference
    )
print "meanEuclideanDistance: %8.4f stddevEuclideanDistance: %8.4f" % (
    stat.meanEuclideanDistance, stat.stddevEuclideanDistance
    )

runMessageLoop("Press space to transform calibration grid\n")

## Transform/calibrate the image.
calibration.transformImage(image, image)

## Update the display.
window.redraw()

runMessageLoop("Press space to quit program")

del window

#</matrixdoc>
