#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Data Matrix
\endxmlonly

\page datamatrix_page Datamatrix example (Python)

\section datamatrix_section_purpose Purpose
This example shows how to read a datamatrix code in an image.

\section datamatrix_section_intro What it does
This example shows how to locate and decode a datamatrix code in an image and print the
result as well as some statistics.

\section datamatrix_section_description How it works

First the input image is loaded and displayed.

\code
#load and display image
image = loadImage("dmtrx_6.bmp")
window = Window(image)
\endcode

Then a new DatamatrixConfiguration object is created and some of the settings are
changed.

\code
#setup datamatrix configuration
config = DatamatrixConfiguration()
config.spacing = 20
config.contrast = 50
\endcode

Afterwards the decoding process is started.

\code
#decode the image
results = decodeDM(image, config)
\endcode

Then some general statistics about the decoding process are displayed:

\code
#get candidates and decoding time
candidates = results.numberOfCandidates
print "Candidates:\t", results.numberOfCandidates
print "Total decoding time:\t", results.totalDecodingTime
\endcode

Then for each found candidate some statistics are displayed. Afterwards
each code type which produces sensible results is displayed for the current
code.

\code
#step through the candidates and display results
for candCounter in range(candidates):
    print "\tCandidate\t", candCounter
    print "\t\tDecoding time:\t", results.getDecodingTime(candCounter), " ms"
    codes = results.getNumberOfCodes(candCounter)

    #step through all codes
    for codeCounter in range(codes):

        #get information about code
        codeType = results.getCodeType(candCounter, codeCounter)
        code = results.getCode(candCounter, codeCounter)
        dimensions = results.getDatamatrixDims(candCounter, codeCounter)
        errors = results.getNumberOfErrors(candCounter, codeCounter)
        score = results.getReadabilityScore(candCounter, codeCounter)

        #print information
        print "\t\tCode:\t\t", codeCounter
        print "\t\tCodetype:\t", codeType
        print "\t\tDimensions:\t", dimensions.width, " x ", dimensions.height
        print "\t\tFlipped:\t%s" % ("false, code seen from front","true, code seen from back")[results.getFlipped(candCounter,codeCounter)] 
        print "\t\tErrors:\t\t", errors
        print "\t\tScore:\t\t", score
        print "\t\tData:\t\t", code
\endcode

Finally the sample waits for space to be pressed and destroys the window.

\code
#keep sample alive
runMessageLoop()

del window
\endcode

\section datamatrix_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.datamatrix import *

#load and display image
image = loadImage("dmtrx_6.bmp")
window = Window(image)

#setup datamatrix configuration
config = DatamatrixConfiguration()
config.spacing = 20
config.contrast = 50

#decode the image
results = decodeDM(image, config)

#get candidates and decoding time
candidates = results.numberOfCandidates
print "Candidates:\t", results.numberOfCandidates
print "Total decoding time:\t", results.totalDecodingTime

#step through the candidates and display results
for candCounter in range(candidates):
    print "\tCandidate\t", candCounter
    print "\t\tDecoding time:\t", results.getDecodingTime(candCounter), " ms"
    codes = results.getNumberOfCodes(candCounter)

    #step through all codes
    for codeCounter in range(codes):

        #get information about code
        codeType = results.getCodeType(candCounter, codeCounter)
        code = results.getCode(candCounter, codeCounter)
        dimensions = results.getDatamatrixDims(candCounter, codeCounter)
        errors = results.getNumberOfErrors(candCounter, codeCounter)
        score = results.getReadabilityScore(candCounter, codeCounter)

        #print information
        print "\t\tCode:\t\t", codeCounter
        print "\t\tCodetype:\t", codeType
        print "\t\tDimensions:\t", dimensions.width, " x ", dimensions.height
        print "\t\tFlipped:\t%s" % ("false, code seen from front","true, code seen from back")[results.getFlipped(candCounter,codeCounter)] 
        print "\t\tErrors:\t\t", errors
        print "\t\tScore:\t\t", score
        print "\t\tData:\t\t", code

#keep sample alive
runMessageLoop()

del window

##</matrixdoc>
