#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page keyboard_page Direct keyboard input example (Python)

\section keyboard_section_purpose Purpose
This sample shows how direct keyboard input from the terminal (console, shell) window is handled.

\section keyboard_section_intro What it does
This sample shows how blocking and non-blocking functions for getting keyboard inputs work.
\note This sample does not work with IDLE, but only in the terminal !

\section keyboard_section_description How it works

First the use of non-blocking keyboard input is shown. The keyHit function is
run in a while loop where actually nothing is done, but where the programmer
can run his processing loop which can then be aborted by the user by pressing some
key.

\code
## Running non-blocking keyHit in a loop
print "Hit some key, please"
print

while not keyHit():
   ## working hard here
    pass
\endcode

Afterwards the key which was pressed is read via the function base.getChar and
printed. This function returns the first unprocessed key in the keyboard buffer. If there
is no such key, it blocks until there is one available. In this case the first alternative applies.

\code
print "Thanks for hitting a key! "

## we have a key here now, that should be eaten
c = getChar()
print "Thanks for hitting: " + c
print
\endcode

Then getChar is called again, but blocks, because the user has not pressed any further
key until now. Afterwards the key is printed. The keyboard echo is turned off by default.

\code
## Running blocking getChar without echo
print "Hit some more key, please"
print

c = getChar()
print "Thanks for hitting: " + c
print
\endcode

Now the same procedure is done with the only difference that the keyboard echo is turned on.

\code
## Running blocking getChar with echo
print "Hit some more key, please"
print

c = getChar(True)
print " <-- that's the getChar echo\n"
print "Thanks for hitting: " + c
print
\endcode

Finally the user is requested to press a key to quit the program.

\code
print "press any key to quit"
c = getChar()
\endcode

\section keyboard_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

import sys

sys.stderr.write("Warning: This example does not run under IDLE !\n")

## Running non-blocking keyHit in a loop
print "Hit some key, please"
print

while not keyHit():
   ## working hard here
    pass

print "Thanks for hitting a key! "

## we have a key here now, that should be eaten
c = getChar()
print "Thanks for hitting: " + c
print

## Running blocking getChar without echo
print "Hit some more key, please"
print

c = getChar()
print "Thanks for hitting: " + c
print

## Running blocking getChar with echo
print "Hit some more key, please"
print

c = getChar(True)
print " <-- that's the getChar echo\n"
print "Thanks for hitting: " + c
print

print "press any key to quit"
c = getChar()

##</matrixdoc>
