#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Focus
\endxmlonly

\page multifocus_page Multifocus example (Python)

\section multifocus_section_purpose Purpose
This sample shows how to calculate an in-focus-image from three images which are only partially in-focus.

\section multifocus_section_intro What it does
Three images which are only partially in-focus are displayed and then an image is calculated which consists
of the merged in-focus areas of the three images.

\section multifocus_section_description How it works

First an empty list is created and then the three input images are loaded from disk and appended to the list.

\code
#create a list with the three out-focus images
images = [loadImage("screw%d.bmp" % n) for n in (1, 2, 3)]
\endcode

Then the in-focus image is calculated by a single function call.

\code
#calculate in-focus image from list
newImage = computeMultiFocusImage(images)
\endcode

Afterwards the bit shift for all newly created windows is set to 2 because the images in this
example consist of 10 bit gray values.

\code
setDefaultShift(2)
\endcode

Afterwards for each of the images a window is opened.

\code
#display the out-focus images
window1 = Window(images[0])
window2 = Window(images[1])
window3 = Window(images[2])

\endcode

Then the calculated in-focus image is displayed.

\code
#display in-focus image
window4 = Window(newImage)

\endcode

Finally the program waits for the user pressing space in one of the windows and destroys them afterwards.

\code
#keep demo alive
runMessageLoop()

del window1
del window2
del window3
del window4
\endcode

\section multifocus_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.focus import *

#create a list with the three out-focus images
images = [loadImage("screw%d.bmp" % n) for n in (1, 2, 3)]

#calculate in-focus image from list
newImage = computeMultiFocusImage(images)

setDefaultShift(2)

#display the out-focus images
window1 = Window(images[0])
window2 = Window(images[1])
window3 = Window(images[2])

#display in-focus image
window4 = Window(newImage)

#keep demo alive
runMessageLoop()

del window1
del window2
del window3
del window4

#</matrixdoc>
