#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT OCR
\endxmlonly

\page ocr_page Simple Ocr example (Python)

\section ocr_section_purpose Purpose
This sample shows a rather simple ocr problem after a special preprocessing step.

\section ocr_section_intro What it does
First the source image is unwinded, because the characters are arranged on an arc. Then
the ocr is done on the resulting image.

\section ocr_section_description How it works

First the input image is loaded and a window is opened to display it.

\code
## Load sample file
image = loadImage("unwind.bmp")

window = Window(image)
\endcode

Then the interesting part of the image is unwinded.

\code
imageUnwind = unwind (
            image,          ## source image
            400,            ## XExtent
            400,            ## YExtent
            312.,           ## x-center of polar transformation
            234.,           ## y-center of polar transformation
            270.,           ## start from angle
            135.,           ## up to angle
            195,            ## start position where to start unwinding
            70,             ## end position where to stop unwinding
            0.4,            ## Anglestep
            0.,             ## RotationAngle
            giBilinear      ## interpolation mode
           )
\endcode

Afterwards the ocr database containing the charset is loaded from disk and
some configuration settings are changed.

\code
if os.getenv("VL_FILE_DIR"):
    db = OcrFontDatabase(os.getenv("VL_FILE_DIR") + "pattern.ocr")
else:
    db = OcrFontDatabase("pattern.ocr")
db.classificationType = ocCorrelation
db.threshold = 148

\endcode

Then a SegmentList object is created with a capacity of 256 characters. Afterwards
the image is segmented and the segments are sorted.

\code
segmentList = SegmentList(256)
db.segment(imageUnwind, segmentList)

segmentList.sort(db, imageUnwind)
\endcode

Now the actual recognition is started.

\code
text = db.readString(imageUnwind, segmentList)
\endcode

Then the resulting string is displayed in the current window using the
text property of a LineOfInterest object. The actual LOI is only one pixel long,
so it's quite hard to see it.

\code
## show text
center = Point(image.width/2, image.height/2)
dispLoi = LineOfInterest(window, Line(center, center))
dispLoi.text = "Recognized string: \"" + text + "\""
dispLoi.textColor = (255, 0, 0)
\endcode

Finally the program asks the user to press space and destroys the window afterwards.

\code
runMessageLoop("Press space to quit\n")

del window
\endcode

\section ocr_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *
from mvIMPACT.ocr import *

import os

## Load sample file
image = loadImage("unwind.bmp")

window = Window(image)

imageUnwind = unwind (
            image,          ## source image
            400,            ## XExtent
            400,            ## YExtent
            312.,           ## x-center of polar transformation
            234.,           ## y-center of polar transformation
            270.,           ## start from angle
            135.,           ## up to angle
            195,            ## start position where to start unwinding
            70,             ## end position where to stop unwinding
            0.4,            ## Anglestep
            0.,             ## RotationAngle
            giBilinear      ## interpolation mode
           )

if os.getenv("VL_FILE_DIR"):
    db = OcrFontDatabase(os.getenv("VL_FILE_DIR") + "pattern.ocr")
else:
    db = OcrFontDatabase("pattern.ocr")
db.classificationType = ocCorrelation
db.threshold = 148

segmentList = SegmentList(256)
db.segment(imageUnwind, segmentList)

segmentList.sort(db, imageUnwind)

text = db.readString(imageUnwind, segmentList)

## show text
center = Point(image.width/2, image.height/2)
dispLoi = LineOfInterest(window, Line(center, center))
dispLoi.text = "Recognized string: \"" + text + "\""
dispLoi.textColor = (255, 0, 0)

runMessageLoop("Press space to quit\n")

del window

##</matrixdoc>
