#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page presetlut_page Preset LUT example (Python)

\section presetlut_section_purpose Purpose
This sample shows how to use the various predefined palettes available.

\section presetlut_section_intro What it does
The same image is shown with varying preset values used as its window's palette.

\section presetlut_section_description How it works

First an image is loaded from disk and a wedge is drawn into it using a suitable
child image. Then the modified image is displayed in a window.

\code
## load any supported file format
image = loadImage("screw.bmp")

## draw a horizontal wedge into image
wedge = Image(image, 10, 10, 0, image.width-20, 20, 1)
presetWedgeImage(
    wedge,
    PresetColors(),
    PresetTransformation(image.width-20, 1))

## open window to display image
wnd = Window(image)
\endcode

Afterwards a color Palette is created and then preset with all possible preset values.
Each of these is applied to the window and the user is requested to press space to
see set the next one. Finally the window is deleted.
\code
## create false color palette
palette = Palette(256, 3)

print "\nPresetting with predefined monochrome palettes"

presetPalette(palette, pptGrayLinear)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayLinear "
runMessageLoop()

presetPalette(palette, pptGrayInverse)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayInverse "
runMessageLoop()

presetPalette(palette, pptGraySquare)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGraySquare "
runMessageLoop()

presetPalette(palette, pptGraySquareRoot)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGraySquareRoot "
runMessageLoop()

presetPalette(palette, pptGrayExponential )
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayExponential "
runMessageLoop()

presetPalette(palette, pptGrayLogarithmic)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayLogarithmic "
runMessageLoop()

presetPalette(palette, pptGrayGamma)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayGamma "
runMessageLoop()

presetPalette(palette, pptGrayInverseGamma)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayInverseGamma "
runMessageLoop()

print "\nPresetting with predefined color palettes"

presetPalette(palette, pptColorContour8)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorContour8 "
runMessageLoop()

presetPalette(palette, pptColorContour16)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorContour16 "
runMessageLoop()

presetPalette(palette, pptColorDistributed)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorDistributed "
runMessageLoop()

presetPalette(palette, pptColorRainbow)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorRainbow "
runMessageLoop()

presetPalette(palette, pptColorFire)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorFire "
runMessageLoop()

presetPalette(palette, pptColorGrass)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorGrass "
runMessageLoop()

presetPalette(palette, pptColorIce)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorIce "
runMessageLoop()

del wnd
\endcode

\section presetlut_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

## load any supported file format
image = loadImage("screw.bmp")

## draw a horizontal wedge into image
wedge = Image(image, 10, 10, 0, image.width-20, 20, 1)
presetWedgeImage(
    wedge,
    PresetColors(),
    PresetTransformation(image.width-20, 1))

## open window to display image
wnd = Window(image)

## create false color palette
palette = Palette(256, 3)

print "\nPresetting with predefined monochrome palettes"

presetPalette(palette, pptGrayLinear)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayLinear "
runMessageLoop()

presetPalette(palette, pptGrayInverse)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayInverse "
runMessageLoop()

presetPalette(palette, pptGraySquare)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGraySquare "
runMessageLoop()

presetPalette(palette, pptGraySquareRoot)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGraySquareRoot "
runMessageLoop()

presetPalette(palette, pptGrayExponential )
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayExponential "
runMessageLoop()

presetPalette(palette, pptGrayLogarithmic)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayLogarithmic "
runMessageLoop()

presetPalette(palette, pptGrayGamma)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayGamma "
runMessageLoop()

presetPalette(palette, pptGrayInverseGamma)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptGrayInverseGamma "
runMessageLoop()

print "\nPresetting with predefined color palettes"

presetPalette(palette, pptColorContour8)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorContour8 "
runMessageLoop()

presetPalette(palette, pptColorContour16)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorContour16 "
runMessageLoop()

presetPalette(palette, pptColorDistributed)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorDistributed "
runMessageLoop()

presetPalette(palette, pptColorRainbow)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorRainbow "
runMessageLoop()

presetPalette(palette, pptColorFire)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorFire "
runMessageLoop()

presetPalette(palette, pptColorGrass)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorGrass "
runMessageLoop()

presetPalette(palette, pptColorIce)
wnd.colorPalette = palette
wnd.redraw()
print "\tpptColorIce "
runMessageLoop()

del wnd

#</matrixdoc>
