#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page quantize_page Quantize example (Python)

\section quantize_section_purpose Purpose
This example shows how to quantize an image.

\section quantize_section_intro What it does
This example quantizes and image using 4-val error diffusion dithering (Floyd-Steinberg).

\section quantize_section_description How it works

First the image is loaded and a window is opened to display it.

\code
#load and display true color image
image = loadImage("bmw.bmp")
window = Window(image)
\endcode

Then the image is quantized. The function base.quantize returns a pair (2-tuple) consisting
of the quantized image and a color palette.

\code
#quantize the image down to 256 colors
quantImage = quantize(image, 256, dmDither4)
\endcode

Afterwards a second window is opened showing the image and the palette of the window is set to the
palette returned by the quantize function.

\code
#display quantized image
window2 = Window(quantImage[0])
window2.colorPalette = quantImage[1]
\endcode

Finally the message loop is run and afterwards the two windows are destroyed.

\code
#keep sample alive
runMessageLoop()

del window
del window2
\endcode

\section quantize_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

#load and display true color image
image = loadImage("bmw.bmp")
window = Window(image)

#quantize the image down to 256 colors
quantImage = quantize(image, 256, dmDither4)

#display quantized image
window2 = Window(quantImage[0])
window2.colorPalette = quantImage[1]

#keep sample alive
runMessageLoop()

del window
del window2

#</matrixdoc>
