#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page shuffle_page Shuffle example (Python)

\section shuffle_section_purpose Purpose
This example shows how to use palette shuffling.

\section shuffle_section_intro What it does
This example shows how to use palette shuffling in conjunction with multiple thresholds.

\section shuffle_section_description How it works

First the input image is loaded and displayed.

\code
## load and display image
image = loadImage("phone.bmp")
window1 = Window(image)
\endcode

Then a threshold mapping is set up using two thresholds. These are the same thresholds as in the
multithreshold example. Then the threholds are applied to the image.

\code
##threshold image
thresholds = (Threshold(50, pmLess), Threshold(180, pmGreater))
pal = generateThresholdPalette(thresholds, image.datatype)
image = mapLut(image, pal)
\endcode

To make the values of the thresholded image, which lie between 0 and 2, better visible, the
bits 0 and 1 are shuffled to the bits 6 and 7 respectively. This is done by building a list of
instances of class Shuffle with the corresponding fromIndex and toIndex values.

\code
##setup shuffle struct
shuffle = (Shuffle(0, 6), Shuffle(1, 7))
\endcode

Afterwards the shuffle palette is created and the image is mapped using this palette, resulting
in a second image.

\code
##calculate new palette
pal1 = generateShufflePalette(shuffle)
##map the image over the new palette
image2 = mapLut(image, pal1)

\endcode

Finally a second window is opened to display the resulting image. Afterwards, the program waits for space key and
deletes the two windows thereafter.

\code

##display new image
window2 = Window(image2)

##keep sample alive
runMessageLoop()

del window1
del window2
\endcode

\section shuffle_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

## load and display image
image = loadImage("phone.bmp")
window1 = Window(image)

##threshold image
thresholds = (Threshold(50, pmLess), Threshold(180, pmGreater))
pal = generateThresholdPalette(thresholds, image.datatype)
image = mapLut(image, pal)

##setup shuffle struct
shuffle = (Shuffle(0, 6), Shuffle(1, 7))

##calculate new palette
pal1 = generateShufflePalette(shuffle)

##map the image over the new palette
image2 = mapLut(image, pal1)

##display new image
window2 = Window(image2)

##keep sample alive
runMessageLoop()

del window1
del window2

#</matrixdoc>
