#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page sizefilter_page Size filter example (Python)

\section sizefilter_section_purpose Purpose
This sample shows how to use size filtering for blobs.

\section sizefilter_section_intro What it does
An image is loaded, thresholded and then a size filter operation is applied.

\section sizefilter_section_description How it works
First an image is loaded from disk.

\code
##load image
image1 = loadImage("gef.bmp")
\endcode

Then it is binarized using a fixed threshold.
\code
##create binary image
image1 = compareGreater(image1, 110)
\endcode

Afterwards the size filter is applied, which filters all objects being smaller than 300 pixels.

\code
##apply size filter for object smaller than 300
image2 = sizeFilter(image1, 300, smSmaller, osWhite)
\endcode

Then a window is displayed to show the original image.

\code
##display both images
window = Window(image1)
runMessageLoop("Press space bar to show size-filtered image")
\endcode

Afterwards the size filtered image is shown in the same window.

\code
window.buffer = image2
window.redraw()
\endcode

Finally the user is waited for pressing space and the window is destroyed after this has happened.

\code
##keep sample alive
runMessageLoop()

del window
\endcode

\section sizefilter_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

##load image
image1 = loadImage("gef.bmp")

##create binary image
image1 = compareGreater(image1, 110)

##apply size filter for object smaller than 300
image2 = sizeFilter(image1, 300, smSmaller, osWhite)

##display both images
window = Window(image1)
runMessageLoop("Press space bar to show size-filtered image")

window.buffer = image2
window.redraw()

##keep sample alive
runMessageLoop()

del window

#</matrixdoc>
