#!/usr/bin/python

"""
/**
\\xmlonly
mvIMPACT Base
\endxmlonly

\page unwind_page Unwind example (Python)

\section unwind_section_purpose Purpose
This little sample shows, how you can unwind an image.

\section unwind_section_intro What it does
This pre-processing unwinds the text and displays it horizontal, with the effect that the OCR can be executed
without any difficulties. The mvIMPACT library offers the Unwind function.

\section unwind_section_description How it works
First of all the sample loads an image.

\code
#load image
image = loadImage("unwind.bmp")
\endcode

On the basis of some input like the start and end angle,
the size of the font etc. the function unwinds the image.
For this reason and if the position of the characters varies,
it could be possible, that there have to be further pre-processings.

\code
#unwind the image
image2 = unwind(image,       #the image to unwind
                400,         #XExtend
                400,         #YExtend
                312,         #XCenter
                234,         #YCenter
                270,         #start from angle
                135,         #up to angle
                195,         #start position
                 70,         #end position
                0.4,         #anglestep
                  0,         #rotation
                giBilinear)  #interpolation mode
\endcode

Now the sample shows the the source and the processed image.

\code
#display both images
window = Window(image)
window2 = Window(image2)
\endcode

\section unwind_section_source Source file

<matrixdoc>
"""

from mvIMPACT.base import *

#load image
image = loadImage("unwind.bmp")

#unwind the image
image2 = unwind(image,       #the image to unwind
                400,         #XExtend
                400,         #YExtend
                312,         #XCenter
                234,         #YCenter
                270,         #start from angle
                135,         #up to angle
                195,         #start position
                 70,         #end position
                0.4,         #anglestep
                  0,         #rotation
                giBilinear)  #interpolation mode

#display both images
window = Window(image)
window2 = Window(image2)

#keep demo alive until space was pressed
runMessageLoop()

del window
del window2

#</matrixdoc>
