#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <vl_ipl.h>
#include <vl_blob.h>

#include <samples.h>

int main( int /*argc*/, char * /*argv[]*/ )
{
	IPL_BUFHANDLE image = IPL_DONT_CARE;

	// load any supported file format into image
	CHECKSDKCALL(IPL_FlImport(&image, NULL, FL_DONT_CARE, "money.tif"));

	// threshold image
	CHECKSDKCALL(IPL_PtToPoint(image, 80, &image, IPL_GREATER_CST));

	// open window to display image, with associated lut
	IPL_HWND hwnd = 0;
	CHECKSDKCALL(DISP_OpenWindow(image, &hwnd, 0, NULL));

	// keep sample alive until console window is closed or space bar is pressed
	CHECKSDKCALL(DISP_KeepRunning("Hit the space bar to perform blob analysis.\n"));

	// setup blob analysis
	BL_BUFHANDLE features = BL_DONT_CARE;
	CHECKSDKCALL(BL_BufAlloc(&features, BL_FEATUREBUF));

	BL_BUFHANDLE results = BL_DONT_CARE;
	CHECKSDKCALL(BL_BufAlloc(&results, BL_RESULTBUF));

	// add features
	CHECKSDKCALL(BL_FeatureSelect(features, BL_AR_AREA, 0, BL_ADD)); 
	CHECKSDKCALL(BL_FeatureSelect(features, BL_AR_EXTREME_BOX, 0, BL_ADD)); 

	// perform blob analysis
	CHECKSDKCALL(BL_Analyze(image, -1, &image, features, results));

	// print results
	double blobs = 0;
	CHECKSDKCALL(BL_ResultBufInquire(results, 0, 0, 0, (BL_FEATURE)-1, 0, BL_GET_NB_BLOBS, &blobs));

	double items_area = 0;
	CHECKSDKCALL(BL_ResultBufInquire(results, 0, 0, 0, BL_AR_AREA, 0, BL_GET_NB_ITEMS, &items_area));
	double *area = new double[ (int)( blobs * items_area )];
	CHECKSDKCALL(BL_GetResultBufData(results, 0, 0, 0, BL_AR_AREA, 0, area));

	double items_box = 0;
	CHECKSDKCALL(BL_ResultBufInquire(results, 0, 0, 0, BL_AR_EXTREME_BOX, 0, BL_GET_NB_ITEMS, &items_box));
	double* box = new double[ (int)( blobs*items_box )];
	CHECKSDKCALL(BL_GetResultBufData(results, 0, 0, 0, BL_AR_EXTREME_BOX, 0, box));

	for (int i=0; i<blobs; ++i)
		printf("Object %d:\tArea: %g\tBounds: %g, %g, %g, %g\n", i, area[i], box[i*4], box[1+i*4], box[2+i*4], box[3+i*4]);

	// threshold image again. After calling BL_Analyze, objects are labelled and may appear dark in the image.
	CHECKSDKCALL(IPL_PtToPoint(image, 0, &image, IPL_GREATER_CST));

	CHECKSDKCALL(DISP_KeepRunning("Hit the space bar to end.\n"));
	
	delete [] area;
	delete [] box;

	// free allocated buffers
	CHECKSDKCALL(BL_BufFree(features));
	CHECKSDKCALL(BL_BufFree(results));
	CHECKSDKCALL(IPL_BufFreeAll());


	return 0;
}
