// cr2d.cpp : Defines the entry point for the console application.
//

#include <vl_base.h>
#include <vl_file.h>
#include <vl_ipl.h>
#include <vl_display.h>
#include <vl_cr2d.h>
#include <string>
#include <iostream>

#include <samples.h>
#define _InImgFile 		"mvSensor_6.bmp"

int main(int argc, char* argv[])
{
	bool bUI = true;
	char inputFileName[1024], *pinputFileName = NULL;
	memset( inputFileName, 0, 1024 );
	if( argc > 1 )
	{
		_skiparg( );
		_strarg( pinputFileName );

		sprintf( inputFileName, "%s", pinputFileName );

		if ( argc > 1 )
			bUI = false;

	}
	else
		sprintf( inputFileName, "%s", _InImgFile );

	IPL_BUFHANDLE lhSrc = IPL_DONT_CARE;

	//load sample file
	IPL_FlImport(&lhSrc, NULL, FL_DONT_CARE, inputFileName);

	//open window and draw binary image
	IPL_HWND hWnd = 0;
	if (bUI)
		DISP_OpenWindow(lhSrc, &hWnd, 0, 0);

	CR2D_BUFHANDLE config = IPL_DONT_CARE;
	CR2D_BufAlloc(&config);

	IPL_ULONG tmp = 10;
	CR2D_Control(config,CR2D_CFG_SPACING,tmp);
	tmp = 60;
	CR2D_Control(config,CR2D_CFG_CONTRAST,tmp);

	CR2D_BUFHANDLE results = IPL_DONT_CARE;
	CR2D_Decode(lhSrc, &config, &results);

	std::cout << "File: " << inputFileName << std::endl;

	IPL_ULONG candidates=0;
	CR2D_Inquire(results, CR2D_GET_NUMBER_OF_CANDIDATES, 0, 0, &candidates);

	if (candidates)
	{
		IPL_ULONG total_time=0;
		CR2D_Inquire(results, CR2D_GET_TOTAL_DECODING_TIME, 0, 0, &total_time);

		std::cout << " Total decoding time: " << total_time << "\n";
		
		for(unsigned int i=0; i<candidates; ++i)
		{
			std::cout << " Candidate: " << i << "\n";

			IPL_ULONG codes=0;
			CR2D_Inquire(results, CR2D_GET_NUMBER_OF_CODES, 0, 0, &codes);

			for(unsigned int j=0; j<codes; ++j)
			{
				std::cout << "  Code: " << j << "\n";

				std::string decoder;
				int decoder_length = 0;
				CR2D_Inquire(results, CR2D_GET_CODETYPE_LENGTH, i, j, &decoder_length);
				decoder.resize(decoder_length);
				CR2D_Inquire(results, CR2D_GET_CODETYPE, i, j, &decoder[0]);

				std::cout << "   Decoder: " << decoder << "\n";

				std::string code;
				int code_length = 0;
				CR2D_Inquire(results, CR2D_GET_CODEDATA_LENGTH, i, j, &code_length);
				code.resize(code_length);
				CR2D_Inquire(results, CR2D_GET_CODEDATA, i, j, &code[0]);

				std::cout << "   Code: " << code << "\n";

				IPL_POINT location[4];
				memset(&location, 0, sizeof(IPL_POINT)*4);
				CR2D_Inquire(results, CR2D_GET_LOCATION, i, j, &location);

				std::cout << "   Location: " 
					<< location[0].xpos << ", " << location[0].ypos << ", " 
					<< location[1].xpos << ", " << location[1].ypos << ", " 
					<< location[2].xpos << ", " << location[2].ypos << ", " 
					<< location[3].xpos << ", " << location[3].ypos << "\n";

				IPL_ULONG rows=0, cols=0;
				CR2D_Inquire(results, CR2D_GET_DM_ROWS, i, 0, &rows);
				CR2D_Inquire(results, CR2D_GET_DM_COLUMNS, i, 0, &cols);
				std::cout << "   Dimensions: "
					<< rows << ", " << cols << "\n";

				IPL_BOOL mirror=IPL_FALSE;
				CR2D_Inquire(results, CR2D_GET_DM_MIRROR, i, 0, &mirror);
				if (!mirror)
					std::cout << "   Seen from front\n";
				else
					std::cout << "   Seen from back\n";

				IPL_ULONG errors=0;
				CR2D_Inquire(results, CR2D_GET_DM_ERRORS, i, 0, &errors);
				std::cout << "   Errors: " << errors << "\n";

				IPL_DOUBLE readability=0.0;
				CR2D_Inquire(results, CR2D_GET_READABILITY_SCORE, i, 0, &readability);
				std::cout << "   Readability: " << readability << "\n";

				IPL_BOOL black_on_white=IPL_FALSE;
				CR2D_Inquire(results, CR2D_IS_BLACK_ON_WHITE, i, 0, &black_on_white);
				std::cout << "   Black on White: " << black_on_white << "\n";

				IPL_ULONG time = 0;
				CR2D_Inquire(results, CR2D_GET_DECODING_TIME, i, 0, &time);

				std::cout << "   Decoding time: " << time << "\n";
			}
		}
	}
	else
	{
		std::cout << " No candidates found or decodable!" << "\n";
	}

	CR2D_BufFree(config);
	CR2D_BufFree(results);

	//wait until space bar is pressed
	if (bUI)
		DISP_KeepRunning("Press space bar to continue.\n");

	//free buffers
	IPL_BufFree(lhSrc);

	return 0;
}

