//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX sample >>ocr2<<
// Mit Apr 28 10:18:59 CEST 2004
//
// Defines the entry point for the console application.
//
//----------------------------------------------------------------------

#include <vl_base.h>
#include <vl_file.h>
#include <vl_ipl.h>
#include <vl_display.h>
#include <vl_ocr.h>

#include <stdio.h>
#include <system.h>
#include <oscode.h>
#include <samples.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define XOffset 200
#define YOffset 200

#define ROIWidth  240
#define ROIHeight 160

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static IPL_RECT Rect[256];
static IPL_LONG Count = 0;


//----------------------------------------------------------------------
void DrawRect( IPL_HDC Dc, IPL_RECT rect, IPL_COLOR Color )
{
	IPL_POINT From, To;

	From.xpos = XOffset + rect.left;
	From.ypos = YOffset + rect.top;
	To.xpos   = XOffset + rect.left;
	To.ypos   = YOffset + rect.bottom;
	CHECKSDKCALL( DISP_DrawLine( Dc, Color, &From, &To ));

	From.xpos = XOffset + rect.left;
	From.ypos = YOffset + rect.top;
	To.xpos   = XOffset + rect.left;
	To.ypos   = YOffset + rect.bottom;
	CHECKSDKCALL( DISP_DrawLine( Dc, Color, &From, &To ));

	From.xpos = XOffset + rect.left;
	From.ypos = YOffset + rect.top;
	To.xpos   = XOffset + rect.right;
	To.ypos   = YOffset + rect.top;
	CHECKSDKCALL( DISP_DrawLine( Dc, Color, &From, &To ));

	From.xpos = XOffset + rect.right;
	From.ypos = YOffset + rect.top;
	To.xpos   = XOffset + rect.right;
	To.ypos   = YOffset + rect.bottom;
	CHECKSDKCALL( DISP_DrawLine( Dc, Color, &From, &To ));

	From.xpos = XOffset + rect.left;
	From.ypos = YOffset + rect.bottom;
	To.xpos   = XOffset + rect.right;
	To.ypos   = YOffset + rect.bottom;
	CHECKSDKCALL( DISP_DrawLine( Dc, Color, &From, &To ));
}


//----------------------------------------------------------------------
IPL_FRESULT IPL_C_CALLTYPE OnDraw(IPL_HWND Window, IPL_BUFHANDLE Buffer, IPL_HDC Dc, const IPL_RECT* Source, const IPL_RECT* Dest, const IPL_VOID* UserData)
{
	UNUSED( Window );
	UNUSED( Buffer );
	UNUSED( Source );
	UNUSED( Dest );
	UNUSED( UserData );

	IPL_COLOR Color = PALETTERGB( 0,255,0 );

	for ( int i=0; i<Count; i++ )
		DrawRect( Dc, Rect[i], Color );

 	return true ;
}

//----------------------------------------------------------------------
int main( int /*argc*/, char* /*argv[]*/ )
{
	IPL_BUFHANDLE lhBuf = IPL_DONT_CARE;
	IPL_BUFHANDLE lhChildBuf = IPL_DONT_CARE;
	OCR_BUFHANDLE lhBufOcr = IPL_DONT_CARE;

	// Load sample file
	CHECKSDKCALL(IPL_FlImport(&lhBuf, NULL, FL_DONT_CARE, "can.bmp"));


	// Open Window and draw image
	IPL_HWND hWnd = 0;
	CHECKSDKCALL(DISP_Control(DISP_ON_DRAW, (IPL_VOID*)OnDraw));
	CHECKSDKCALL(DISP_OpenWindow(lhBuf, &hWnd, 0, 0));

	// select text region
	CHECKSDKCALL( IPL_BufAllocChild2D(
				&lhChildBuf,
				lhBuf,
				XOffset, YOffset,
				0, 0, 0,
				ROIWidth, ROIHeight ));


// Preprocess image
	CHECKSDKCALL( DISP_UpdateWindow(hWnd, DISP_INVALIDATE) );
	CHECKSDKCALL( DISP_KeepRunning("Press space bar for preprocess image.\n") );

	IPL_LONG RankData[] = { 1, 1, 1,
							1, 1, 1,
							1, 1, 1};
	IPL_BUFHANDLE lhRank = IPL_DONT_CARE;

	// alloc ranking structuring element buffer
	CHECKSDKCALL( IPL_BufAlloc2D( &lhRank, 3, 3, IPL_DTLONG, IPL_STRUCTBUF,  NULL ));
	CHECKSDKCALL( IPL_PutBufData( lhRank, 0, 0, 0, 0, 0, 3, 3, &RankData ));
	CHECKSDKCALL( IPL_MoErode (lhChildBuf, &lhChildBuf, lhRank, 1, IPL_GRAYSCALE ));
	CHECKSDKCALL( IPL_BufFree(lhRank) );


	CHECKSDKCALL( DISP_UpdateWindow(hWnd, DISP_INVALIDATE) );
	CHECKSDKCALL( DISP_KeepRunning("Press space bar for segment image.\n") );

// OCR
	// Open OCR buffer
	IPL_CHAR file[_MAX_PATH] = "";
	if (getenv ("VL_FILE_DIR") != NULL)
		strcpy(file, getenv("VL_FILE_DIR"));
	//strcat(file, "pattern.ocr");
	strncat(file, "/", _MAX_PATH);
	strncat(file, "pattern.ocr", _MAX_PATH);

	CHECKSDKCALL(OCR_Open(&lhBufOcr, file, 0)) ;

	// Adjust parameter
	OCR_CLASSIFICATION cls = OCR_KOR_CLAS ;
	CHECKSDKCALL(OCR_Control(lhBufOcr, OCR_CLASSIFICATIONTYPE, &cls)) ;
	long tmp = 127;
	CHECKSDKCALL(IPL_StatBestThreshold(lhChildBuf, &tmp));
	CHECKSDKCALL(OCR_Control(lhBufOcr, OCRSEG_THRESHOLD, &tmp)) ;
	tmp = 15;
	OCR_Control( lhBufOcr, OCRCHAR_MINWIDTH, &tmp );
	tmp = 25;
	OCR_Control( lhBufOcr, OCRCHAR_MINHEIGHT, &tmp );
	tmp = 30;
	OCR_Control( lhBufOcr, OCRCHAR_MAXWIDTH, &tmp ) ;
	tmp = 40;
	OCR_Control( lhBufOcr, OCRCHAR_MAXHEIGHT, &tmp ) ;


	// Segment image
	CHECKSDKCALL(OCR_Segment(
		lhBufOcr,		// ocr buffer
		lhChildBuf,		// image buffer
		256,			// max count
		Rect,           // array of segment rectangles
		&Count));		// segment count

//    InvalidateRect((HWND)hWnd, NULL, true);
	CHECKSDKCALL( DISP_UpdateWindow(hWnd, DISP_INVALIDATE) );
	CHECKSDKCALL(DISP_KeepRunning("Press space bar for split large segments.\n"));


	// Split large segments
	CHECKSDKCALL(OCR_SplitSegment(
		lhBufOcr,		// ocr buffer
		lhChildBuf,		// image buffer
		256,			// max count
		Rect,           // array of segment rectangles
		&Count));		// segment count

	CHECKSDKCALL( DISP_UpdateWindow(hWnd, DISP_INVALIDATE) );
	CHECKSDKCALL( DISP_KeepRunning("Press space bar for merge small segments.\n") );

	CHECKSDKCALL(OCR_MergeSegment(
		lhBufOcr,		// ocr buffer
		lhChildBuf,	    // image buffer
		256,			// max count
		Rect,           // array of segment rectangles
		&Count));		// segment count

	CHECKSDKCALL( DISP_UpdateWindow(hWnd, DISP_INVALIDATE) );
	CHECKSDKCALL( DISP_KeepRunning("Press space bar for classify segments.\n") );

	CHECKSDKCALL(OCR_SortSegment(
		lhBufOcr,		// ocr buffer
		lhChildBuf,     // image buffer
		Rect,           // array of segment rectangles
		Count));		// segment count


	char Txt[256] = "";
	for(int j=0; j<Count; j++)
	{
		OCR_RESULT Result[2] ;
		IPL_LONG maxres = 1;
		CHECKSDKCALL(OCR_ReadChar(
			lhBufOcr,		// ocr buffer
			lhChildBuf,     // image buffer
			&Rect[j],		// segment rectangle
			&maxres,
			Result ));      // result

		Txt[j] = Result[0].ascii;
		Txt[j+1] = 0 ;
	}


	// For text display
	IPL_COLOR Color = PALETTERGB( 255,255,255 );
	DISP_OOISETHANDLE oois = DISP_DONT_CARE;
	DISP_OOIHANDLE TextL = DISP_DONT_CARE;
	if( strlen(Txt) )
	{
		IPL_POINT TextPos[2] = {{XOffset+ROIWidth/2,YOffset},{XOffset+ROIWidth/2,YOffset}};
		CHECKSDKCALL(DISP_OoiSetAlloc( &oois ));
		CHECKSDKCALL(DISP_OoiAlloc( oois, &TextL, DISP_LOI_TOOL, TextPos, sizeof(TextPos)/sizeof(TextPos[0]), NULL ));
		CHECKSDKCALL( DISP_ControlOoi( oois, TextL, DISP_OOI_COLOR, &Color ));
		CHECKSDKCALL( DISP_ControlOoi( oois, TextL, DISP_OOI_TEXT, Txt ));
		CHECKSDKCALL( DISP_ControlOoi( oois, TextL, DISP_OOI_TEXT_COLOR, &Color ));
		CHECKSDKCALL( DISP_ControlWindow( DISP_OOISET, hWnd, &oois ));
	}

	CHECKSDKCALL( DISP_UpdateWindow(hWnd, DISP_INVALIDATE) );
	CHECKSDKCALL( DISP_KeepRunning("Press space bar to close.\n") );

	//free buffers
	CHECKSDKCALL(DISP_OoiFree( oois, TextL ));
	CHECKSDKCALL(DISP_OoiSetFree( oois ));
	CHECKSDKCALL(OCR_Close(lhBufOcr)) ;
	CHECKSDKCALL(IPL_BufFree(lhBuf));

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------



