#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <samples.h>

void create_normal_aoi(DISP_OOISETHANDLE oois, const IPL_RECT& rect, 
				const IPL_CHAR* text, IPL_BOOL clipped,
				IPL_COLOR color, IPL_COLOR text_color)
{
	// create an aoi in the set
	IPL_POINT aoi_points[5] =
	{
		{ rect.left, rect.top },
		{ rect.right, rect.top },
		{ rect.right, rect.bottom },
		{ rect.left, rect.bottom },
		{ rect.left, rect.top },
	};

	DISP_OOIHANDLE aoi = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiAlloc(
		oois, &aoi, DISP_AOI_TOOL, 
		aoi_points, sizeof(aoi_points)/sizeof(aoi_points[0]), NULL));

	// set some ooi properties
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_COLOR, &color));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT, text));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT_COLOR, &text_color));
}

void create_handle_color_aoi(DISP_OOISETHANDLE oois, const IPL_RECT& rect, 
				const IPL_CHAR* text, IPL_COLOR handle_line_color, 
				IPL_COLOR handle_fill_color)
{
	// create an aoi in the set
	IPL_POINT aoi_points[5] =
	{
		{ rect.left, rect.top },
		{ rect.right, rect.top },
		{ rect.right, rect.bottom },
		{ rect.left, rect.bottom },
		{ rect.left, rect.top },
	};

	DISP_OOIHANDLE aoi = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiAlloc(
		oois, &aoi, DISP_AOI_TOOL, 
		aoi_points, sizeof(aoi_points)/sizeof(aoi_points[0]), NULL));

	// set some ooi properties
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT, text));
	IPL_COLOR text_color = PALETTERGB(255, 0, 0);
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT_COLOR, &text_color));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_HANDLE_LINE_COLOR, &handle_line_color));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_HANDLE_FILL_COLOR, &handle_fill_color));
	IPL_BOOL selected = IPL_TRUE;
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_SELECTED, &selected));
}

void create_handle_size_aoi(DISP_OOISETHANDLE oois, const IPL_RECT& rect, 
				const IPL_CHAR* text, IPL_LONG size)
{
	// create an aoi in the set
	IPL_POINT aoi_points[5] =
	{
		{ rect.left, rect.top },
		{ rect.right, rect.top },
		{ rect.right, rect.bottom },
		{ rect.left, rect.bottom },
		{ rect.left, rect.top },
	};

	DISP_OOIHANDLE aoi = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiAlloc(
		oois, &aoi, DISP_AOI_TOOL, 
		aoi_points, sizeof(aoi_points)/sizeof(aoi_points[0]), NULL));

	// set some ooi properties
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT, text));
	IPL_COLOR text_color = PALETTERGB(255, 0, 0);
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT_COLOR, &text_color));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_HANDLE_SIZE, &size));
	IPL_BOOL selected = IPL_TRUE;
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_SELECTED, &selected));
}

void create_text(DISP_OOISETHANDLE oois, const IPL_RECT& rect, 
				const IPL_CHAR* text, IPL_BOOL clipped,
				IPL_COLOR text_color)
{
	// create an aoi in the set
	IPL_POINT aoi_points[5] =
	{
		{ rect.left, rect.top },
		{ rect.right, rect.top },
		{ rect.right, rect.bottom },
		{ rect.left, rect.bottom },
		{ rect.left, rect.top },
	};

	DISP_OOIHANDLE aoi = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiAlloc(
		oois, &aoi, DISP_AOI_TOOL, 
		aoi_points, sizeof(aoi_points)/sizeof(aoi_points[0]), NULL));

	// set some ooi properties
	IPL_COLOR color = PALETTERGB(0, 0, 0);
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_COLOR, &color));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT, text));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT_COLOR, &text_color));

	IPL_BOOL visible = IPL_FALSE;
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_LINES_VISIBLE, &visible));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_HANDLES_VISIBLE, &visible));
}

IPL_RECT create_ooi_rect(int x, int y)
{
	IPL_RECT pos;
	pos.left = x; 
	pos.right = x + 90; 
	pos.top = y; 
	pos.bottom = y + 90;

	return pos;
}

int main(int /*argc*/, char * /*argv[]*/)
{
	// load any supported file format, into image
	IPL_BUFHANDLE image = IPL_DONT_CARE;
	CHECKSDKCALL(IPL_FlImport(&image, NULL, FL_DONT_CARE, "screw.bmp"));

	// create an ooiset
	DISP_OOISETHANDLE oois = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiSetAlloc(&oois));

	IPL_RECT pos;
	pos.left = 10; 
	pos.right = 600; 
	pos.top = 10; 
	pos.bottom = 50;
	create_text(oois, pos,
		"AOIs in different colors", IPL_TRUE, PALETTERGB(0, 0, 0));

	create_normal_aoi(oois, 
		create_ooi_rect(10, 60), 
		"Red", IPL_TRUE, PALETTERGB(255, 0, 0), PALETTERGB(255, 0, 0));

	create_normal_aoi(oois, 
		create_ooi_rect(110, 60), 
		"Green", IPL_TRUE, PALETTERGB(0, 255, 0), PALETTERGB(0, 255, 0));
	
	create_normal_aoi(oois, 
		create_ooi_rect(210, 60), 
		"Blue", IPL_TRUE, PALETTERGB(0, 0, 255), PALETTERGB(0, 0, 255));

	create_normal_aoi(oois, 
		create_ooi_rect(310, 60), 
		"Black", IPL_TRUE, PALETTERGB(0, 0, 0), PALETTERGB(0, 0, 0));

	create_normal_aoi(oois, 
		create_ooi_rect(410, 60), 
		"Gray", IPL_TRUE, PALETTERGB(127, 127, 127), PALETTERGB(127, 127, 127));

	create_normal_aoi(oois, 
		create_ooi_rect(510, 60), 
		"White", IPL_TRUE, PALETTERGB(255, 255, 255), PALETTERGB(255, 255, 255));

	pos.top = 160; 
	pos.bottom = 200;
	create_text(oois, pos,
		"AOIs with different handle colors", IPL_TRUE, PALETTERGB(0, 0, 0));

	create_handle_color_aoi(oois, 
		create_ooi_rect(10, 210), 
		"Red/Black", PALETTERGB(255, 0, 0), PALETTERGB(0, 0, 0));

	create_handle_color_aoi(oois, 
		create_ooi_rect(110, 210), 
		"Green/Gray", PALETTERGB(0, 255, 0), PALETTERGB(127, 127, 127));
	
	create_handle_color_aoi(oois, 
		create_ooi_rect(210, 210), 
		"Blue/White", PALETTERGB(0, 0, 255), PALETTERGB(255, 255, 255));

	create_handle_color_aoi(oois, 
		create_ooi_rect(310, 210), 
		"Black/Black", PALETTERGB(0, 0, 0), PALETTERGB(0, 0, 0));

	create_handle_color_aoi(oois, 
		create_ooi_rect(410, 210), 
		"Gray/Gray", PALETTERGB(127, 127, 127), PALETTERGB(127, 127, 127));

	create_handle_color_aoi(oois, 
		create_ooi_rect(510, 210), 
		"White/White", PALETTERGB(255, 255, 255), PALETTERGB(255, 255, 255));

	pos.top = 310; 
	pos.bottom = 350;
	create_text(oois, pos,
		"AOIs with different handle sizes", IPL_TRUE, PALETTERGB(0, 0, 0));

	create_handle_size_aoi(oois, 
		create_ooi_rect(10, 360), 
		"2", 2);

	create_handle_size_aoi(oois, 
		create_ooi_rect(110, 360), 
		"5", 5);
	
	create_handle_size_aoi(oois, 
		create_ooi_rect(210, 360), 
		"10", 10);

	create_handle_size_aoi(oois, 
		create_ooi_rect(310, 360), 
		"20", 20);

	create_handle_size_aoi(oois, 
		create_ooi_rect(410, 360), 
		"30", 30);

	create_handle_size_aoi(oois, 
		create_ooi_rect(510, 360), 
		"40", 40);

	// associate the ooiset with the thread (and all windows created thereafter)
	CHECKSDKCALL(DISP_Control(DISP_OOISET, &oois));

	// open window to display image
	IPL_HWND hwnd = 0;
	CHECKSDKCALL(DISP_OpenWindow(image, &hwnd, 0, NULL));

	// keep sample alive until console window is closed or space bar is pressed
	CHECKSDKCALL(DISP_KeepRunning("Click the space bar to end program.\n"));

	// free allocated buffers
	CHECKSDKCALL(IPL_BufFreeAll());

	return 0;
}

