#include <vl_base.h>
#include <vl_device.h>
#include <samples.h>

void showproperties(DEV_HANDLE device) // device handle
{

  // get property list
  if (!DEV_GetCapsList(device))
  {
    printf("No property list available !\n\n");
	return;
  }

  //get the number of properties
  DEV_RANGETYPE rangetype = (DEV_RANGETYPE)0;
  IPL_VOID *p = NULL;
  IPL_LONG identifier = 0;
  IPL_CHAR *identifiername = NULL;
  IPL_CHAR *unitname = NULL;
  CHECKSDKCALL(DEV_InquireCapsList(device,0,DEV_NUMBER_PROP,NULL,NULL,NULL,&rangetype,&p));
  IPL_LONG propcount = *(IPL_LONG *)p;
  IPL_LONG i, j;

  for (i=0; i < propcount; i++)
  {
    // inquire property by index
    rangetype = (DEV_RANGETYPE)0;
	IPL_VOID *p = NULL;
    identifier = 0;
    identifiername = NULL;
    CHECKSDKCALL(DEV_InquireCapsList(device,i,DEV_INDEX,&identifier,&identifiername,&unitname,&rangetype,&p));

	switch (rangetype)
	{
	 case DCTL_LNG_RANGE:  {
                              // assign pointer to structure 
	                          DEV_LNG_RANGE *ps = (DEV_LNG_RANGE *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_LONG curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %ld\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_LNG_RANGE\n");

							  // get data
							  printf("Min: %ld, max: %ld, increment: %ld, default: %ld\n",
								      ps->Min,ps->Max,ps->Increment,ps->Default);
						   }
						   break;

	 case DCTL_ULNG_RANGE:  {
                              // assign pointer to structure 
	                          DEV_ULNG_RANGE *ps = (DEV_ULNG_RANGE *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_ULONG curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %ld\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_ULNG_RANGE\n");

							  // get data
							  printf("Min: %ld, max: %ld, increment: %ld, default: %ld\n",
								      ps->Min,ps->Max,ps->Increment,ps->Default);
							}
						    break;

	 case DCTL_DBL_RANGE:  {
                              // assign pointer to structure 
	                          DEV_DBL_RANGE *ps = (DEV_DBL_RANGE *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_DOUBLE curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %f\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_DBL_RANGE\n");

							  // get data
							  printf("Min: %f, max: %f, increment: %f, default: %f\n",
								      ps->Min,ps->Max,ps->Increment,ps->Default);
							}
						    break;

	 case DCTL_LNG_ENUM:  {
                              // assign pointer to structure 
	                          DEV_LNG_ENUM *ps = (DEV_LNG_ENUM *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_LONG curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %ld\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_LNG_ENUM\n");

							  // number of entries
							  printf("Entries: %ld\n",ps->Entries);

							  // default value
							  printf("Default: %ld\n",ps->Default);

						      // get helper strings
							  printf("Helper strings: ");
							  for (IPL_LONG i=0; i < ps->Entries; i++)
                                if (i == ps->Entries-1) // last value ? 
							       printf("%s",ps->NamePtr[i]);
								else
							       printf("%s,",ps->NamePtr[i]);

                              printf("\n");

						      // get data
							  printf("Data: ");
							  for (j=0; j < ps->Entries; j++)
                                if (j == ps->Entries-1) // last value ?
							       printf("%ld",ps->DataPtr[j]);
								else
							       printf("%ld,",ps->DataPtr[j]);


							  printf("\n");
							}
						    break;

	 case DCTL_DBL_ENUM:  {
                              // assign pointer to structure 
	                          DEV_DBL_ENUM *ps = (DEV_DBL_ENUM *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_DOUBLE curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %f\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_DBL_ENUM\n");

							  // number of entries
							  printf("Entries: %ld\n",ps->Entries);

							  // default value
							  printf("Default: %f\n",ps->Default);

						      // get helper strings
							  printf("Helper strings: ");
							  for (j=0; j < ps->Entries; j++)
                                if (j == ps->Entries-1) // last value ?
							       printf("%s",ps->NamePtr[j]);
								else
							       printf("%s,",ps->NamePtr[j]);

                              printf("\n");

						      // get data
							  printf("Data: ");
							  for (j=0; j < ps->Entries; j++)
                                if (j == ps->Entries-1) // last value ?
							       printf("%f",ps->DataPtr[j]);
								else
							       printf("%f,",ps->DataPtr[j]);

							  printf("\n");
							}
						    break;

	 case DCTL_NL_LNG_RANGE:
							{
                              // assign pointer to structure 
	                          DEV_NL_LNG_RANGE *ps = (DEV_NL_LNG_RANGE *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_LONG curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %ld\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_NL_LNG_RANGE\n");

							  // number of entries
							  printf("Entries: %ld\n",ps->Entries);

							  // get data
							  printf("Min: %ld, max: %ld, default: %ld\n",
								      ps->Min,ps->Max,ps->Default);

						      // get data
							  printf("Data: ");
							  for (IPL_LONG i=0; i < ps->Entries; i++)
                                if (i == ps->Entries-1) // last value ? 
							       printf("%ld",ps->DataPtr[i]);
								else
							       printf("%ld,",ps->DataPtr[i]);

							  printf("\n");

						   }
						   break;


	 case DCTL_NL_ULNG_RANGE:
							{
                              // assign pointer to structure 
	                          DEV_NL_ULNG_RANGE *ps = (DEV_NL_ULNG_RANGE *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_ULONG curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %ld\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_NL_ULNG_RANGE\n");

							  // number of entries
							  printf("Entries: %ld\n",ps->Entries);

							  // get data
							  printf("Min: %ld, max: %ld, default: %ld\n",
								      ps->Min,ps->Max,ps->Default);

						      // get data
							  printf("Data: ");
							  for (IPL_LONG i=0; i < ps->Entries; i++)
                                if (i == ps->Entries-1) // last value ? 
							       printf("%ld",ps->DataPtr[i]);
								else
							       printf("%ld,",ps->DataPtr[i]);

							  printf("\n");

						   }
						   break;

	 case DCTL_NL_DBL_RANGE:
							{
                              // assign pointer to structure 
	                          DEV_NL_DBL_RANGE *ps = (DEV_NL_DBL_RANGE *)p;

							  // get name
							  printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_DOUBLE curvalue=0;
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier,&curvalue))
								printf("Current value: %f\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_NL_DBL_RANGE\n");

							  // number of entries
							  printf("Entries: %ld\n",ps->Entries);

							  // get data
							  printf("Min: %f, max: %f, default: %f\n",
								      ps->Min,ps->Max,ps->Default);

						      // get data
							  printf("Data: ");
							  for (IPL_LONG i=0; i < ps->Entries; i++)
                                if (i == ps->Entries-1) // last value ? 
							       printf("%f",ps->DataPtr[i]);
								else
							       printf("%f,",ps->DataPtr[i]);

							  printf("\n");

						   }
						   break;

	 case DCTL_STR_VALUE:
							{
                              // assign pointer to structure 
	                          DEV_STR_VALUE *ps = (DEV_STR_VALUE *)p;

							  // get name
							  if (identifiername != NULL)
							     printf("Id name: %s\n",identifiername);

							  // get current value
							  IPL_CHAR* curvalue = new IPL_CHAR[ps->MaxLength+1];
							  if (DEV_InquireCaps(device,(DEV_CAPSTYPE)identifier, curvalue))
								printf("Current value: %s\n",curvalue);
							  else
								printf("Current value: <<Device Property Access Error>>\n");
							  delete [] curvalue;

							  // get identifier
							  printf("Id: %ld\n",identifier);

							  // get unit name
							  if (unitname != NULL)
							     printf("Unit name: %s\n",unitname);

							  // get range type
							  printf("Range type: DEV_STR_ARRAY\n");

							  // maximum string length
							  printf("Maximum length: %ld\n",ps->MaxLength);

							  printf("\n");

						   }
						   break;


	 default:;  // unknown range type

	}

	printf("\n");

  }

}

int main(int /*argc*/, char* /*argv[]*/ )
{
	// open device
	DEV_HANDLE device = IPL_DONT_CARE;
    CHECKSDKCALL(DEV_Open(&device, "matrixvision.ddc"));

    // get a property list and show properties.
	// the properties returned by the dummy driver are not functional.
	// they are just shown for demonstration.
	showproperties(device);

	// wait
	printf("Press RETURN key to continue");
	getchar();

	// close device
    CHECKSDKCALL(DEV_Close(device));

	return 0;
}
