/**
\xmlonly
mvIMPACT OCR
\endxmlonly

\page ocr_page Simple Ocr example (CPP)

\section ocr_section_purpose Purpose
This sample shows a rather simple ocr problem after a special preprocessing step.

\section ocr_section_intro What it does
First the source image is unwinded, because the characters are arranged on an arc. Then
the ocr is done on the resulting image.

\section ocr_section_description How it works

First the input image is loaded and a window is opened to display it.

\code
		// Load sample file
		Image image = loadImage("unwind.bmp");

		Window window = Window(image);
\endcode

Then the interesting part of the image is unwinded.

\code
		Image imageUnwind = unwind (
			image,			// source image
			400,			// XExtent
			400,			// YExtent
			312.,			// x-center of polar transformation
			234.,			// y-center of polar transformation
			270.,			// start from angle
			135.,			// up to angle
			195,			// start position where to start unwinding
			70,				// end position where to stop unwinding
			0.4,			// Anglestep
			0.,				// RotationAngle
			giBilinear		// interpolation mode
			);
\endcode

Afterwards the ocr database containing the charset is loaded from disk and
some configuration settings are changed.

\code
		std::string dbFileName;
		if (getenv("VL_FILE_DIR") != NULL)
			dbFileName = getenv("VL_FILE_DIR") + std::string("pattern.ocr");
		else
			dbFileName = std::string("pattern.ocr");	

		OcrFontDatabase db = OcrFontDatabase(dbFileName);
		db.setClassificationType(ocCorrelation);
		db.setThreshold(148);

\endcode

Then a SegmentList object is created with a capacity of 256 characters. Afterwards
the image is segmented and the segments are sorted.

\code
		SegmentList segmentList = SegmentList(256);
		db.segment(imageUnwind, segmentList);
\endcode

Now the actual recognition is started.

\code
		segmentList.sort(db, imageUnwind);
		std::string text = db.readString(imageUnwind, segmentList);
\endcode

Then the resulting string is displayed in the current window using the
text property of a LineOfInterest object. The actual LOI is only one pixel long,
so it's quite hard to see it.

\code


		// show text
		Point center = Point(image.getWidth()/2, image.getHeight()/2);
		LineOfInterest dispLoi = LineOfInterest(window, Line(center, center));
		dispLoi.setText("Recognized string: \"" + text + "\"");
		dispLoi.setTextColor(RGBColor(255,0,0));
\endcode

Finally the program asks the user to press space and ends afterwards.

\code
		runMessageLoop("Press space to quit\n");
\endcode

\section ocr_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_ocr.h"

#include <system.h>

int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		// Load sample file
		Image image = loadImage("unwind.bmp");

		Window window = Window(image);

		Image imageUnwind = unwind (
			image,			// source image
			400,			// XExtent
			400,			// YExtent
			312.,			// x-center of polar transformation
			234.,			// y-center of polar transformation
			270.,			// start from angle
			135.,			// up to angle
			195,			// start position where to start unwinding
			70,				// end position where to stop unwinding
			0.4,			// Anglestep
			0.,				// RotationAngle
			giBilinear		// interpolation mode
			);

		std::string dbFileName;
		if (getenv("VL_FILE_DIR") != NULL)
			dbFileName = getenv("VL_FILE_DIR") + std::string("pattern.ocr");
		else
			dbFileName = std::string("pattern.ocr");		

		OcrFontDatabase db = OcrFontDatabase(dbFileName);
		db.setClassificationType(ocCorrelation);
		db.setThreshold(148);

		SegmentList segmentList = SegmentList(256);
		db.segment(imageUnwind, segmentList);

		segmentList.sort(db, imageUnwind);

		std::string text = db.readString(imageUnwind, segmentList);

		// show text
		Point center = Point(image.getWidth()/2, image.getHeight()/2);
		LineOfInterest dispLoi = LineOfInterest(window, Line(center, center));
		dispLoi.setText("Recognized string: \"" + text + "\"");
		dispLoi.setTextColor(RGBColor(255,0,0));

		runMessageLoop("Press space to quit\n");
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}
	return 0;
}

//</matrixdoc>
