//------------------------------------------------------------------------------------------
/* 'blob_video' is a demo for Blob Analysis by using live snaped images. It is tested with the
 * MATRIX VISION GmbH - mvBlueLYNX 4XX. The demo looks for white objects on a dark background.
 * (eg: image - images/money.gif)
 *
 */
//------------------------------------------------------------------------------------------
#include <stdio.h>
#include <system.h>

#include <vl_file.h>
#include <vl_ipl.h>
#include <vl_base.h>

#include <vl_display.h>
#include <vl_device.h>

#include <vl_blob.h>

#include <samples.h>

#ifdef MALLOC_TRACE
	#include <mcheck.h>
#endif	// MALLOC_TRACE

//------------------------------------------------------------------------------------------
#define MAX_OBJ_CNT			100
#define MAX_AOI_CNT			100
#define MAX_AREA_SIZE		100
#define MIN_THRESHOLD		90

#define AREA_SIZE			MAX_OBJ_CNT
#define BOX_SIZE			( 4 * MAX_OBJ_CNT )
#define TEXT_SIZE			1024
#define COLOR_LUT_SIZE		10

//------------------------------------------------------------------------------------------
static int ocnt = 0;
static DISP_OOISETHANDLE oois = DISP_DONT_CARE;
static DISP_OOIHANDLE aoi[MAX_AOI_CNT];
static long glob_threshold = 0;
static int polarity = 0;

//------------------------------------------------------------------------------------------
COLORREF colTab[COLOR_LUT_SIZE] =
{
	PALETTERGB( 255,   0,   0 ),
	PALETTERGB(   0, 255,   0 ),
	PALETTERGB(   0,   0, 255 ),
	PALETTERGB( 255, 255,   0 ),
	PALETTERGB( 255,   0, 255 ),
	PALETTERGB(   0, 255, 255 ),
	PALETTERGB( 127, 255,   0 ),
	PALETTERGB(   0, 127, 255 ),
	PALETTERGB( 255, 127,   0 ),
	PALETTERGB( 127,   0, 255 )
};
//------------------------------------------------------------------------------------------
void exit_demo( const char *s, long err_code )
{
	fprintf( stderr, "*** blob_video: %s - err: %ld!!\n", s, err_code );
	exit( err_code );
}

//------------------------------------------------------------------------------------------
void mk_dummy_img_buf( IPL_BUFHANDLE image, IPL_BUFHANDLE *dummy )
{
	IPL_LONG xsize = 0, ysize = 0;
	IPL_LONG datatype = IPL_IMAGEBUF;

	if( !IPL_BufInquire( image, IPL_XSIZE, &xsize))
		exit_demo("get x size failed", IPL_GetErrorCode( ));

	if( !IPL_BufInquire( image, IPL_YSIZE, &ysize))
		exit_demo("get y size failed", IPL_GetErrorCode( ));

	if( !IPL_BufInquire( image, IPL_DATATYPE, &datatype))
		exit_demo("get data type failed", IPL_GetErrorCode( ));

	if( !IPL_BufAlloc2D( dummy, xsize, ysize, (IPL_BUFFER_DATATYPE)datatype, IPL_IMAGEBUF, NULL ))
		exit_demo("out of memory(IPL_BufAlloc2D)", IPL_GetErrorCode( ));
}

//------------------------------------------------------------------------------------------
void blb_init( BL_BUFHANDLE *features, BL_BUFHANDLE *results )
{
	if( !BL_BufAlloc( features, BL_FEATUREBUF ))
		exit_demo("blb_init - out of memory (features)", IPL_GetErrorCode( ));

	if( !BL_BufAlloc( results,  BL_RESULTBUF ))
		exit_demo("blb_init - out of memory (results)", IPL_GetErrorCode( ));

	if( !BL_SetProcMode( *features, BLOC_CTLID_STOP_AFTER_INDEX, MAX_OBJ_CNT ))
		exit_demo("blb_init - set stop feature failed", IPL_GetErrorCode( ));

	if( !BL_FeatureSelect( *features, BL_AR_AREA, 0, BL_ADD ))
		exit_demo("blb_init - set feature: 'area' failed", IPL_GetErrorCode( ));

	if( !BL_FeatureSelect( *features, BL_AR_EXTREME_BOX, 0, BL_ADD ))
		exit_demo("blb_init - set feature: 'extreme box' failed", IPL_GetErrorCode( ));
}

//------------------------------------------------------------------------------------------
int img_preproc( IPL_BUFHANDLE src_img, IPL_BUFHANDLE *dst_img )
{
	int ret;
	long threshold = 0, min_threshold = 0;

	if( glob_threshold != 0 )
		min_threshold = glob_threshold;
	else
		min_threshold = MIN_THRESHOLD;

	if(( ret = IPL_StatBestThreshold( src_img, &threshold )))
	{
		if( threshold < min_threshold )
			threshold = min_threshold;
			
		dprintf((" == threshold: %3ld\n", threshold ));

		if(( ret = IPL_PtToPoint( src_img, threshold, dst_img, polarity ? IPL_LESS_CST : IPL_GREATER_CST )))
		{
			if( !( ret = IPL_MoReconstruct( *dst_img, dst_img, IPL_DONT_CARE, IPL_KILL_BORDER_BLOBS, IPL_BINARY )))
				fprintf( stderr, "*** blob_video: img_preproc - delete border blobs faild - err: -%ld!!\n", IPL_GetErrorCode( ));
		}
		else
			fprintf( stderr, "*** blob_video: img_preproc - binarize faild - err: -%ld!!\n", IPL_GetErrorCode( ));
	}
	else
		fprintf( stderr, "*** blob_video: img_preproc - get best threshold faild - err: -%ld!!\n", IPL_GetErrorCode( ));

	return ret;
}

//------------------------------------------------------------------------------------------
void draw_aoi( int area, int l , int t , int r , int b )
{
	IPL_POINT boxPoints[5];
	memset( boxPoints, 0, 5 * sizeof( IPL_POINT ));

	char AreaArray[TEXT_SIZE];
	memset( AreaArray, 0, TEXT_SIZE );

	if( area > MAX_AREA_SIZE )
	{
		memset( AreaArray, 0, TEXT_SIZE );
		sprintf( AreaArray, "A: %8d", area );
		dprintf(( " == A: %8d\n", area ));

		boxPoints[0].xpos = l;
		boxPoints[0].ypos = t;

		boxPoints[1].xpos = r;
		boxPoints[1].ypos = t;

		boxPoints[2].xpos = r;
		boxPoints[2].ypos = b;

		boxPoints[3].xpos = l;
		boxPoints[3].ypos = b;

		boxPoints[4].xpos = l;
		boxPoints[4].ypos = t;

		aoi[ocnt] = DISP_DONT_CARE;
		DISP_OoiAlloc( oois, &aoi[ocnt], DISP_AOI_TOOL, boxPoints, sizeof( boxPoints )/sizeof( boxPoints[0] ), NULL );

		COLORREF color = colTab[ocnt];
		DISP_ControlOoi( oois, aoi[ocnt], DISP_OOI_COLOR, &color );

		COLORREF textColor = colTab[ocnt];
		DISP_ControlOoi( oois, aoi[ocnt], DISP_OOI_TEXT, AreaArray );
		DISP_ControlOoi( oois, aoi[ocnt], DISP_OOI_TEXT_COLOR, &textColor );
		ocnt++;
	}
}

//------------------------------------------------------------------------------------------
int main( int argc, char *argv[] )
{
#ifdef MALLOC_TRACE
	mtrace();
#endif	// MALLOC_TRACE

	IPL_BUFHANDLE		imageHandle = IPL_DONT_CARE;
	IPL_BUFHANDLE		dummyimageHandle = IPL_DONT_CARE;
	IPL_BUFHANDLE		binimageHandle = IPL_DONT_CARE;
	BL_BUFHANDLE		featuresHandle = BL_DONT_CARE;
	BL_BUFHANDLE		resultsHandle = BL_DONT_CARE;
	DEV_HANDLE 			deviceHandle = IPL_DONT_CARE;
	IPL_HWND 			hwndGrab = 0;

	double blobs = 0;
	double area[AREA_SIZE], box[BOX_SIZE];
	int box_index = 0;

	if( argc > 0 )
	{
		_skiparg( );
		_intarg( polarity );
		_intarg( glob_threshold );
	}

	memset( aoi, DISP_DONT_CARE, MAX_AOI_CNT );

	blb_init( &featuresHandle, &resultsHandle );

	if( !DEV_Open( &deviceHandle, "matrixvision.ddc" ))
		exit_demo("open device failed", IPL_GetErrorCode( ));

	if( !DEV_Control( deviceHandle, DEV_SINGLE, 0 ))
		exit_demo("set controle single failed", IPL_GetErrorCode( ));

	if( !DEV_Grab( deviceHandle, &imageHandle ))
		exit_demo("grab image failed", IPL_GetErrorCode( ));

	if( !DISP_OpenWindow( imageHandle, &hwndGrab, 0, NULL ))
		exit_demo("open windows failed", IPL_GetErrorCode( ));

	mk_dummy_img_buf( imageHandle, &dummyimageHandle );

	DISP_OoiSetAlloc( &oois );
#ifdef EXPORT_IMGS
	static int img_cnt = 0;
	static char msg[256];
	memset(msg,0,256);
#endif	// EXPORT_IMGS
	while( !_kbhit( ))
	{
		blobs = 0;
		if( !DEV_Grab( deviceHandle, &imageHandle ))
			exit_demo("grab image failed", IPL_GetErrorCode( ));

#ifdef EXPORT_IMGS
		sprintf(msg, "snapimg_%02d.bmp", img_cnt );
		IPL_FlExport( imageHandle, IPL_DONT_CARE, IPL_BMP, msg );
#endif	// EXPORT_IMGS

		img_preproc( imageHandle, &binimageHandle );

#ifdef EXPORT_IMGS
		sprintf(msg, "binimg_%02d.bmp", img_cnt++ );
		IPL_FlExport( binimageHandle, IPL_DONT_CARE, IPL_BMP, msg);
#endif	// EXPORT_IMGS

		if( resultsHandle == IPL_DONT_CARE )
		{
			if( !BL_BufAlloc( &resultsHandle,  BL_RESULTBUF ))
				exit_demo("blb_init - out of memory (results)", IPL_GetErrorCode( ));
		}

		if( BL_Analyze( binimageHandle, IPL_DONT_CARE, &dummyimageHandle, featuresHandle, resultsHandle ))
		{
			BL_ResultBufInquire( resultsHandle, 0, 0, 0, (BL_FEATURE)-1, 0, BL_GET_NB_BLOBS, &blobs );
			dprintf((" == BLOBS: %.0f\n", blobs ));

			if( blobs < MAX_OBJ_CNT )
			{
				memset( area, 0, AREA_SIZE );
				BL_GetResultBufData( resultsHandle, 0, 0, 0, BL_AR_AREA, 0, area );

				memset( box, 0, BOX_SIZE );
				BL_GetResultBufData( resultsHandle, 0, 0, 0, BL_AR_EXTREME_BOX, 0, box );

				ocnt = 0;
				for( int i = 0; i < blobs; ++i )
				{
					box_index = 4 * i;

					dprintf(("Object %3d:\tArea: %8g\tBounds: l/t: %3g/%3g, r/b:%3g/%3g\n",
							i, area[i], box[box_index], box[box_index + 1], box[box_index + 2], box[box_index + 3]));

					draw_aoi((int)area[i], (int)box[box_index], (int)box[box_index+1], (int)box[box_index+2], (int)box[box_index+3] );
				}

				DISP_ControlWindow( DISP_OOISET, hwndGrab, &oois );
			}
		}
		else if( IPL_GetErrorCode2( ) == BL_STOPPED_AFTER_INDEX )
			fprintf( stderr, "*** blob_video: stoppend after MAX_OBJ_CNT blobs\n");
		else
			fprintf( stderr, "*** blob_video: BL_Analyze failed (err: %ld)\n", IPL_GetErrorCode2( ));


		DISP_UpdateWindow( hwndGrab, DISP_INVALIDATE );
		DISP_KeepRunning( NULL );

		dprintf((" == ocnt: %d (blobs: %f)\n", ocnt, blobs ));
		if( blobs > 0 )
		{
			for( int i = 0;  i < ocnt; i++ )
			{
				if( aoi[i] != DISP_DONT_CARE )
				{
					DISP_OoiFree( oois, aoi[i] );
					aoi[i] = DISP_DONT_CARE;
				}
			}
		}
		if( resultsHandle != IPL_DONT_CARE )
		{
			BL_BufFree( resultsHandle );
			resultsHandle = IPL_DONT_CARE;
		}
	}	// while( !_kbhit( ))

	DISP_OoiSetFree( oois );

	DISP_CloseWindow( hwndGrab );
	DEV_Close( deviceHandle );

	BL_BufFree( featuresHandle );
	BL_BufFree( resultsHandle );

	IPL_BufFree( dummyimageHandle );
	IPL_BufFree( binimageHandle );
	IPL_BufFree( imageHandle );

	return 0;
}

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
