//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
// pat_sample.cpp : Defines the entry point for the console application.
//

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
#include <stdio.h>
#include <math.h>
#include <system.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <vl_device.h>
#include <vl_pat.h>
#include <vl_ipl.h>

#include <assert.h>

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
int main(int /*argc*/, char* /*argv[]*/)
{

	// load an image from file
	IPL_BUFHANDLE image = IPL_DONT_CARE;
	DEV_HANDLE deviceHandle = IPL_DONT_CARE;

	assert(DEV_Open( &deviceHandle, "matrixvision.ddc" ));
	assert(DEV_Control( deviceHandle, DEV_SINGLE, 0 ));
	assert(DEV_Grab( deviceHandle, &image ));

	// create an ooiset
	DISP_OOISETHANDLE oois = DISP_DONT_CARE;
	assert(DISP_OoiSetAlloc(&oois));

	// generate aoi point set
	IPL_POINT aoipoints[5];
	aoipoints[0].xpos = (IPL_LONG)100;
	aoipoints[0].ypos = (IPL_LONG)100;
	aoipoints[1].xpos = (IPL_LONG)200;
	aoipoints[1].ypos = (IPL_LONG)100;
	aoipoints[2].xpos = (IPL_LONG)200;
	aoipoints[2].ypos = (IPL_LONG)200;
	aoipoints[3].xpos = (IPL_LONG)100;
	aoipoints[3].ypos = (IPL_LONG)200;
	aoipoints[4].xpos = (IPL_LONG)100;
	aoipoints[4].ypos = (IPL_LONG)100;

	// create aoi
	DISP_OOIHANDLE aoi = DISP_DONT_CARE;
	assert(DISP_OoiAlloc(oois, &aoi, DISP_AOI_TOOL, aoipoints, sizeof(aoipoints)/sizeof(aoipoints[0]),NULL));


	// open window to display image
	IPL_HWND hwnd = 0;
	assert(DISP_OpenWindow(image, &hwnd, 0, NULL));

	// attach OOI set to window
	assert(DISP_ControlWindow(DISP_OOISET, hwnd, &oois));

	// update display to view AOI
	assert(DISP_UpdateWindow(hwnd, DISP_INVALIDATE));

	// keep display alive
	assert(DISP_KeepRunning("Hit space bar to search for model.\n"));


	// inquire position of AOI
	IPL_LONG left = 0;
	assert(DISP_InquireOoi(oois, aoi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+0), &left));
	IPL_LONG right = 0;
	assert(DISP_InquireOoi(oois, aoi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+1), &right));
	IPL_LONG top = 0;
	assert(DISP_InquireOoi(oois, aoi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+2), &top));
	IPL_LONG bottom = 0;
	assert(DISP_InquireOoi(oois, aoi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+3), &bottom));

	// allocate model buffer
	IPL_BUFHANDLE model = PAT_DONT_CARE;
	IPL_LONG widthmodel = abs(right - left);
	IPL_LONG heightmodel = abs(bottom - top);
	IPL_LONG xposmodel = left;
	IPL_LONG yposmodel = top;
	assert(PAT_BufAlloc(&model, image, xposmodel, yposmodel, widthmodel, heightmodel, PAT_MODEL));

	// set search parameters
	IPL_DOUBLE acceptancelevel = 95.0;
	assert(PAT_SetControl(model, PAT_ACCEPTANCE_LEVEL, &acceptancelevel));
	IPL_DOUBLE certaintylevel = 60.0;
	assert(PAT_SetControl(model, PAT_CERTAINTY_LEVEL, &certaintylevel));
	IPL_LONG expectedmatches = 1;
	assert(PAT_SetControl(model, PAT_NB_MATCHES, &expectedmatches));

	// start search in image buffer
	PAT_BUFHANDLE resultbuf = PAT_DONT_CARE;
	DISP_OOIHANDLE aoimodel = DISP_DONT_CARE;

	// keep display alive
	assert(DISP_KeepRunning("Hit space bar 4 live mode.\n"));

	if(aoi != IPL_DONT_CARE)
	{
		assert(DISP_OoiFree( oois, aoi ));
		aoi = IPL_DONT_CARE;
	}

	while( !_kbhit())
	{
		assert(DEV_Grab( deviceHandle, &image ));

	    assert(PAT_SearchModel(image, model, &resultbuf));

	    // get number of actual matches in image buffer
		IPL_LONG nbmatches=0;
	    assert(PAT_GetResultBufData(resultbuf, 0, 0, 0, 0, PAT_NB_MATCHES_FOUND, &nbmatches));

		if (nbmatches > 0)
		{
			// inquire model position and match score
			IPL_DOUBLE score=0,xpos=0,ypos=0;
			IPL_LONG m = 0;
			assert(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_SCORE, &score));
			assert(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_XPOS, &xpos));
			assert(PAT_GetResultBufData(resultbuf, 0, 0, 0, m, PAT_YPOS, &ypos));



			// generate aoi point set for the model
			IPL_POINT aoipoints_model[5];
			IPL_LONG leftmodel = (IPL_LONG)(xpos - (widthmodel/2)) + 1;
			IPL_LONG topmodel = (IPL_LONG)(ypos - (heightmodel/2)) + 1;

			aoipoints_model[0].xpos = leftmodel;
			aoipoints_model[0].ypos = topmodel;
			aoipoints_model[1].xpos = (IPL_LONG)(leftmodel + widthmodel);
			aoipoints_model[1].ypos = (IPL_LONG)topmodel;
			aoipoints_model[2].xpos = (IPL_LONG)(leftmodel + widthmodel);
			aoipoints_model[2].ypos = (IPL_LONG)(topmodel + heightmodel);
			aoipoints_model[3].xpos = (IPL_LONG)leftmodel;
			aoipoints_model[3].ypos = (IPL_LONG)(topmodel + heightmodel);
			aoipoints_model[4].xpos = (IPL_LONG)leftmodel;
			aoipoints_model[4].ypos = (IPL_LONG)topmodel;

			// create aoi
			assert(DISP_OoiAlloc(oois, &aoimodel, DISP_AOI_TOOL, aoipoints_model, sizeof(aoipoints_model)/sizeof(aoipoints_model[0]),NULL));

			// assign green color for model AOI
			COLORREF color = PALETTERGB(0, 255, 0);
			assert(DISP_ControlOoi(oois, aoimodel, DISP_OOI_COLOR, &color));

			printf("Model found at x:%f, y:%f score:%f\n", xpos, ypos, score);
		}

		// update display to view model AOI
		assert(DISP_UpdateWindow(hwnd, DISP_INVALIDATE));

		if(aoimodel != IPL_DONT_CARE )
		{
			assert(DISP_OoiFree( oois, aoimodel ));
			aoimodel = IPL_DONT_CARE;
		}

		if( resultbuf != IPL_DONT_CARE )
		{
			assert(PAT_BufFree(resultbuf));
			resultbuf = IPL_DONT_CARE;
		}
	}
		// keep display alive
	//DISP_KeepRunning("Hit space bar to quit program.");

	// free model and result buffer
	PAT_BufFree(model);
	PAT_BufFree(resultbuf);

	// free AOI's
	DISP_OoiFreeAll(oois);

	// free OOI sets
	DISP_OoiSetFreeAll();

	// free image
	IPL_BufFree(image);

	DEV_Close( deviceHandle );

	return 0;
}

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------

