//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>blob-afs<<
// Mit Aug 13 11:31:13 CEST 2003
//----------------------------------------------------------------------
#include <time.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_device.h>
#include <vl_display.h>
#include <vl_ipl.h>
#include <vl_blob.h>

#include <pgutils.h>
#include <samples.h>

#include "blob-afs.h"
#include "afsapi.h"
#include "blbproc.h"
#include "afstools.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static BLOBS ini_blb_list[ ] =
{
	FILL_BLOB_LIST( AREA, 0 ),
	FILL_BLOB_LIST( PERIMETER, 0 ),
	FILL_BLOB_LIST( ROUNDNESS, 0 ),
	FILL_BLOB_LIST( NUMBER_HOLES, 0 ),
	FILL_BLOB_LIST( MAJOR_AXIS_LENGTH, 180 ),
	FILL_BLOB_LIST( MINOR_AXIS_LENGTH, 180 ),
	FILL_BLOB_LIST( FERET_MAX, 180 ),
	FILL_BLOB_LIST( FERET_MIN, 180 ),
	FILL_BLOB_LIST( FERET_RATIO, 180 ),
	FILL_BLOB_LIST( ROUGHNESS, 0 ),
	FILL_BLOB_LIST( AREA_HOLES, 0 ),
	FILL_BLOB_LIST( AREA_PERFORATION, 0 ),
	FILL_BLOB_LIST( FIBER_LENGTH, 0 ),
	FILL_BLOB_LIST( FIBER_WIDTH, 0 ),
	FILL_BLOB_LIST( MAX, /*18*/0 ),
	FILL_BLOB_LIST( SUM, /*18*/0 ),
	FILL_BLOB_LIST( SQUARED_SUM, /*18*/0 ),
	FILL_BLOB_LIST( MEAN, /*18*/0 ),
	FILL_BLOB_LIST( STDDEV, /*18*/0 ),
	FILL_BLOB_LIST( SKEWNESS, /*18*/0 ),
	FILL_BLOB_LIST( KURTOSIS, /*18*/0 ),
	FILL_BLOB_LIST( SKEWNESS_FAC, /*18*/0 ),
	FILL_BLOB_LIST( KURTOSIS_FAC, /*18*/0 ),
	FILL_BLOB_LIST( ENTROPY, /*18*/0 ),
	FILL_BLOB_LIST( ENERGY, 0 ),
	FILL_BLOB_LIST( CONTRAST, 0 ),
	FILL_BLOB_LIST( ENTROPY_CCMTX, 0 ),
	FILL_BLOB_LIST( POLAR_DIST_MAX, 0 ),
	FILL_BLOB_LIST( POLAR_DIST_MIN, 0 ),
	/*FILL_BLOB_LIST( EXTREME_BOX, 0 ),*/
};

//----------------------------------------------------------------------
static BL_BUFHANDLE features = BL_DONT_CARE;
static BL_BUFHANDLE results = BL_DONT_CARE;

static BLOBCLASS class_list[MAX_CLASS_NB];
static BLOBCLASS tmp_res;

static unsigned int class_list_size = 0;
static unsigned int read_bin_file_flag = 0, set_feat4classify_flag = 0;
static unsigned int *feat_list, feat_list_cnt = 0, max_feat_list_cnt = DEFAULT_FEAT_LIST_CNT;
static unsigned long gl_blobs = 0;
static unsigned long max_time_ms = DEFAULT_TIME_MS;
static int process_flag = 0;
static int blob_init_flag = 0;

static struct timeval start_tv, diff_tv;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
void print_class_infos( BLOBCLASS bc, int blobs, FILE *f )
{
	dprintf((" ++ " TOOL_PREFIX ": print_class_infos\n" ));

	fprintf( f, CL_NAME_STR " %s\n", bc.name );

	dprintf((" == " TOOL_PREFIX ": print_class_infos: %d\n", blobs ));
	for( int b = 0; b < blobs; b++ )
	{
		for( int i = 0; i < bc.list_size; i++ )
			fprintf( f, ID_STR " %d "
						TIME_MS_STR " %f "
						DELTA_STR " %f "
						RANGE_MIN_STR " %f "
						RANGE_MAX_STR " %f "
						SUM_CNT_STR " %ld "
						SUM_STR " %f "
						SUM_2_STR " %f\n",
					bc.blb_list[i].ID, bc.blb_list[i].time_ms, bc.blb_list[i].delta,
					bc.blb_list[i].range.x1, bc.blb_list[i].range.x2,
					bc.blb_list[i].sum_cnt, bc.blb_list[i].sum,
					bc.blb_list[i].sum_2 );
	}

	dprintf((" -- " TOOL_PREFIX ": print_class_infos\n" ));
}

//----------------------------------------------------------------------
int get_proc_flag( )
{
	dprintf((" -> " TOOL_PREFIX ": get_proc_flag - process_flag: %d\n", process_flag ));
	return process_flag;
}

//----------------------------------------------------------------------
void clear_blbuf( BL_BUFHANDLE *blbuf )
{
	dprintf((" ++ " TOOL_PREFIX ": clear_blbuf (%ld)\n", *blbuf ));
	if( *blbuf != BL_DONT_CARE )
	{
		BL_BufFree( *blbuf );
		*blbuf = BL_DONT_CARE;
	}
	dprintf((" -- " TOOL_PREFIX ": clear_blbuf (%ld)\n", *blbuf ));
}

//----------------------------------------------------------------------
double get_calc_time_ms( )
{
	dprintf((" -> " TOOL_PREFIX ": get_calc_time_ms (sec: %ld, usec: %ld)\n", diff_tv.tv_sec, diff_tv.tv_usec ));
	return (double)(diff_tv.tv_sec * 1000 + diff_tv.tv_usec/1000.);
}

//----------------------------------------------------------------------
long get_obj_cnt( )
{
	dprintf((" -> " TOOL_PREFIX ": get_obj_cnt\n"));
	return gl_blobs;
}

//----------------------------------------------------------------------
int get_feat_cnt( )
{
	dprintf((" -> " TOOL_PREFIX ": get_feat_cnt\n"));
	return feat_list_cnt;
}

//----------------------------------------------------------------------
int get_max_feat_nb( )
{
	dprintf((" -> " TOOL_PREFIX ": get_max_feat_nb\n"));
	return max_feat_list_cnt;
}

//----------------------------------------------------------------------
long get_max_time_ms( )
{
	dprintf((" -> " TOOL_PREFIX ": get_max_time_ms\n"));
	return max_time_ms;
}

//----------------------------------------------------------------------
int get_max_feat( )
{
	dprintf((" -> " TOOL_PREFIX ": get_max_feat\n" ));
	return ( sizeof( ini_blb_list )/sizeof( ini_blb_list[0] ));
}

//----------------------------------------------------------------------
void set_max_feat_nb( int val )
{
	dprintf((" ++ " TOOL_PREFIX ": set_max_feat_nb\n" ));

	if( val <= class_list[REF_CLASS].list_size )
		max_feat_list_cnt = val;
	else
		max_feat_list_cnt = class_list[REF_CLASS].list_size;

	if( val <= 0 )
		max_feat_list_cnt = 1;

	dprintf((" -- " TOOL_PREFIX ": set_max_feat_nb (%d)\n", max_feat_list_cnt ));
}

//----------------------------------------------------------------------
void set_max_time_ms( long val )
{
	dprintf((" ++ " TOOL_PREFIX ": set_max_time_ms\n" ));

	if( val <= MAX_TIME_MS )
		max_time_ms = val;
	else
		max_time_ms = MAX_TIME_MS;

	if( val < MIN_TIME_MS )
		max_time_ms = MIN_TIME_MS;

	dprintf((" -- " TOOL_PREFIX ": set_max_time_ms (%ld)\n", max_time_ms ));
}

//----------------------------------------------------------------------
void set_ref_time( BLOBCLASS blb_class )
{
	dprintf((" ++ " TOOL_PREFIX ": set_ref_time\n"));
	for( int i = 0; i < class_list[REF_CLASS].list_size; i++ )
		blb_class.blb_list[i].time_ms = class_list[REF_CLASS].blb_list[i].time_ms;
	dprintf((" -- " TOOL_PREFIX ": set_ref_time\n"));
}

//----------------------------------------------------------------------
void set_single_feat( BL_BUFHANDLE featHandle, BL_FEATURE feat, double *ctrl )
{
	dprintf((" ++ " TOOL_PREFIX ": set_single_feat (%ld, 0x%x (%p))\n", featHandle, feat, ctrl ));
	{ CHECK_RET( BL_FeatureSelect( featHandle, feat, ctrl, BL_ADD )); }
	dprintf((" -- " TOOL_PREFIX ": set_single_feat - 0x%x (%p)\n", feat, ctrl ));
}

//----------------------------------------------------------------------
void rm_single_feat( BL_BUFHANDLE featHandle, BL_FEATURE feat, double *ctrl )
{
	dprintf((" ++ " TOOL_PREFIX ": rm_single_feat (%ld) \n", featHandle ));
	CHECK_RET( BL_FeatureSelect( featHandle, feat, ctrl, BL_REMOVE ));
	dprintf((" -- " TOOL_PREFIX ": rm_single_feat - 0x%x (%p)\n", feat, ctrl ));
}

//----------------------------------------------------------------------
int set_features( BL_BUFHANDLE featHandle, BLOBS *blb_list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": set_features (size: %d)\n", list_size ));
	int ret = IPL_FALSE;
	if( featHandle && blb_list && list_size )
	{
		for( int i = 0; i < list_size; i++ )
			set_single_feat( featHandle, blb_list[i].feat, blb_list[i].ctrlVal == 0. ? NULL : &blb_list[i].ctrlVal );

		ret = IPL_TRUE;
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": set_features - wrong input values: featHandle: %ld, blb_list: %p, list_size: %d!!",
						featHandle, blb_list, list_size );

	dprintf((" -- " TOOL_PREFIX ": set_features (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int rm_features( BL_BUFHANDLE featHandle, BLOBS *blb_list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": rm_features (size: %d)\n", list_size ));
	int ret = IPL_FALSE;
	if( featHandle && blb_list && list_size )
	{
		for( int i = 0; i < list_size; i++ )
		{
			rm_single_feat( featHandle, blb_list[i].feat, NULL );
		}
		ret = IPL_TRUE;
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": rm_features - wrong input values: featHandle: %ld, blb_list: %p, list_size: %d!!",
						featHandle, blb_list, list_size );

	dprintf((" -- " TOOL_PREFIX ": rm_features (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void get_items( BL_BUFHANDLE res, BLOBS *blb )
{
	dprintf((" ++ " TOOL_PREFIX ": get_items\n"));
	double items = 0;

	CHECK_RET( BL_ResultBufInquire( res, 0, 0, 0, blb->feat, 0, BL_GET_NB_ITEMS, &items ));
	blb->items = (int)items;

	dprintf((" -- " TOOL_PREFIX ": get_items %s(0x%x), items: %d\n", blb->name, blb->feat, blb->items ));
}

//----------------------------------------------------------------------
void get_single_res( BL_BUFHANDLE resHandle, BLOBS *blb, int blb_nb )
{
	dprintf((" ++ " TOOL_PREFIX ": get_single_res\n"));

	allocprintf( blb->results, i );
	if(( blb->results = (double*) malloc((int)( blb_nb * blb->items )* sizeof(double))) != NULL )
	{
		allocprintf( blb->results, o );
		CHECK_RET( BL_GetResultBufData( resHandle, 0, 0, 0, blb->feat, 0, blb->results ));
	}
	else
		fprintf( stderr,"*** " TOOL_PREFIX ": get_single_res - out of memory!!\n");

	dprintf((" -- " TOOL_PREFIX ": get_single_res\n"));
}

//----------------------------------------------------------------------
void get_results( BL_BUFHANDLE resHandle, BLOBS *blb_list, int list_size, int blobs )
{
	dprintf((" ++ " TOOL_PREFIX ": get_results (resHandle: %ld, blb_list: %p, list_size: %d, blobs: %d)\n",
					resHandle, blb_list, list_size, blobs ));
	if( resHandle && blobs )
	{
		for( int i = 0; i < list_size; i++ )
		{
			get_items( resHandle, &blb_list[i] );
			get_single_res( resHandle, &blb_list[i], blobs );

#ifdef DEBUG
			for( int b  = 0; b < blobs; b++ )
				if( blb_list[i].items == 1 )
					printf(" == " TOOL_PREFIX ": get_blb_res(%3d) - %s - id: %d feat: 0x%x i: %d rb: %p(%f) cv: %.0f tms: %.3f d: %.3f rmin: %.3f rmax: %.3f\n", b,
						blb_list[i].name, blb_list[i].ID, blb_list[i].feat, blb_list[i].items,
						blb_list[i].results, blb_list[i].results[b], blb_list[i].ctrlVal, blb_list[i].time_ms,
						blb_list[i].delta, blb_list[i].range.x1, blb_list[i].range.x2 );
#endif	// DEBUG
		}
	}
	dprintf((" -- " TOOL_PREFIX ": get_results\n"));
}

//----------------------------------------------------------------------
int init_blb_list( BLOBS *blb_list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": init_blb_list\n"));
	int ret = IPL_FALSE;
	if( blb_list )
	{
		for( int i = 0; i < list_size; i++ )
		{
			blb_list[i].ID = ini_blb_list[i].ID;
			blb_list[i].feat = ini_blb_list[i].feat;
			blb_list[i].name = ini_blb_list[i].name;
			blb_list[i].items = ini_blb_list[i].items;
			blb_list[i].results = ini_blb_list[i].results;
			blb_list[i].ctrlVal = ini_blb_list[i].ctrlVal;
			blb_list[i].delta = ini_blb_list[i].delta;
			blb_list[i].time_ms = ini_blb_list[i].time_ms;
			blb_list[i].range.x1 = ini_blb_list[i].range.x1;
			blb_list[i].range.x2 = ini_blb_list[i].range.x2;

			blb_list[i].sum_cnt = ini_blb_list[i].sum_cnt;
			blb_list[i].sum = ini_blb_list[i].sum;
			blb_list[i].sum_2 = ini_blb_list[i].sum_2;
		}

#ifdef DEBUG
		for( int i = 0; i < list_size; i++ )
			printf(" == " TOOL_PREFIX ": init_blb_list - %s - id: %d feat: 0x%x i: %d rb: %p cv: %.0f tms: %.3f d: %.3f rmin: %.3f rmax: %.3f\n",
						blb_list[i].name, blb_list[i].ID, blb_list[i].feat, blb_list[i].items,
						blb_list[i].results, blb_list[i].ctrlVal, blb_list[i].time_ms,
						blb_list[i].delta, blb_list[i].range.x1, blb_list[i].range.x2 );
#endif	// DEBUG

		ret = IPL_TRUE;
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": init_blb_list - empty list (%p)\n", blb_list );

	dprintf((" -- " TOOL_PREFIX ": init_blb_list (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void free_buf_list( BLOBS *blb_list )
{
	dprintf((" ++ " TOOL_PREFIX ": free_buf_list\n"));

	freeprintf( blb_list, i );
		blb_list = (BLOBS*)clear_buf( blb_list );
	freeprintf( blb_list, o );

	dprintf((" -- " TOOL_PREFIX ": free_buf_list\n"));
}

//----------------------------------------------------------------------
void del_res_buf( BLOBS *blb )
{
	dprintf((" ++ " TOOL_PREFIX ": del_res_buf\n"));

	freeprintf( blb->results, i );
		blb->results = (double*)clear_buf( blb->results );
	freeprintf( blb->results, o );

	dprintf((" -- " TOOL_PREFIX ": del_res_buf\n"));
}

//----------------------------------------------------------------------
void clear_bufs( BLOBS *blb_list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": clear_bufs\n"));
	for( int i = 0; i < list_size; i++ )
		del_res_buf( &blb_list[i] );
	dprintf((" -- " TOOL_PREFIX ": clear_bufs\n"));
}

//----------------------------------------------------------------------
int malloc_blb_list( BLOBS **blb_list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": malloc_blb_list - %p\n", blb_list ));

	int ret = IPL_TRUE;
	if( *blb_list != NULL )
		free_buf_list( *blb_list );

	dprintf((" == " TOOL_PREFIX ": malloc_blb_list - %p\n", *blb_list ));
	if( *blb_list == NULL )
	{
		allocprintf( *blb_list, i );
		if(( *blb_list = (BLOBS*)malloc( sizeof(BLOBS) * list_size )) == NULL )
		{
			fprintf( stderr, "*** " TOOL_PREFIX ": malloc_blb_list - out of memory!!\n");
			ret = IPL_FALSE;
		}
		allocprintf( *blb_list, o );
	}

	dprintf((" -- " TOOL_PREFIX ": malloc_blb_list (%d-%p)\n", ret, *blb_list ));
	return ret;
}

//----------------------------------------------------------------------
int init_globals( DEV_HANDLE device, IPL_BUFHANDLE *src, BL_BUFHANDLE feat,
				BLOBS *blb_list, int size, int loop_cnt, int polarity )
{
	dprintf((" ++ " TOOL_PREFIX ": init_globals (size: %d, loop_cnt: %d, polarity: %d)\n", size, loop_cnt, polarity ));

	int ret = IPL_FALSE;
	IPL_BUFHANDLE dst = IPL_DONT_CARE;
	BL_BUFHANDLE res = BL_DONT_CARE;
	double time_ms = 0.;

	if( *src != IPL_DONT_CARE )
	{
		for( int i = 0; i < size; i++ )
		{
			for( int c = 0; c < loop_cnt; c++ )
			{
				CHECK_RET( DEV_Grab( device, src ));
				mk_bin_img( *src, &dst, polarity );

				set_single_feat( feat, blb_list[i].feat, blb_list[i].ctrlVal == 0. ? NULL : &blb_list[i].ctrlVal );

				clear_blbuf( &res );
				CHECK_RET(( ret = BL_BufAlloc( &res, BL_RESULTBUF )));

				StartTimer( &start_tv );
					if( !( ret = BL_Analyze( dst, *src, &dst, feat, res )))
					{
						if(IPL_GetErrorCode2( ) == BL_STOPPED_AFTER_INDEX )
							fprintf( stderr, "\n *** " TOOL_PREFIX ": init_globals - Stopped after %d objects.\n\n ", MAX_BLB_CNT );
						else
							fprintf( stderr, "*** " TOOL_PREFIX ": init_globals - BL_Analyze failed (err: %ld)\n", IPL_GetErrorCode2( ));
					}
				ElapsedTime( &start_tv, &diff_tv, 30 );

				if( ret )
				{
					time_ms = (double)(diff_tv.tv_sec * 1000 + diff_tv.tv_usec/1000.);
					dprintf((" == " TOOL_PREFIX ": init_globals - %s time_ms: %.3f\n", blb_list[i].name, time_ms ));

					if( blb_list[i].time_ms == 0. )
						blb_list[i].time_ms = time_ms;
					else if( time_ms < blb_list[i].time_ms )
						blb_list[i].time_ms = time_ms;

					double blobs = 0;
					CHECK_RET( BL_ResultBufInquire( res, 0, 0, 0, (BL_FEATURE)-1, 0, BL_GET_NB_BLOBS, &blobs ));

					dprintf((" == " TOOL_PREFIX ": init_globals - blobs: %.0f\n", blobs ));

					if( blobs > 0 )
					{
						get_items( res, &blb_list[i] );
						get_single_res( res, &blb_list[i], (int)blobs );

						if( blb_list[i].items == 1 )
						{
							if( blb_list[i].range.x1 == 0 )
								blb_list[i].range.x1 = blb_list[i].results[0];
							else if( blb_list[i].results[0] < blb_list[i].range.x1 )
								blb_list[i].range.x1 = blb_list[i].results[0];

							if( blb_list[i].results[0] > blb_list[i].range.x2 )
								blb_list[i].range.x2 = blb_list[i].results[0];
						}
					}
					else
						fprintf( stderr, "*** " TOOL_PREFIX ": init_globals - no obj found!!\n");
				}

				del_res_buf( &blb_list[i] );

				dprintf((" == " TOOL_PREFIX ": init_globals - %s - id: %d feat: 0x%x i: %d rb: %p cv: %.0f tms: %.3f d: %.3f rmin: %.3f rmax: %.3f\n",
							blb_list[i].name, blb_list[i].ID, blb_list[i].feat, blb_list[i].items,
							blb_list[i].results, blb_list[i].ctrlVal, blb_list[i].time_ms,
							blb_list[i].delta, blb_list[i].range.x1, blb_list[i].range.x2 ));

				rm_single_feat( feat, blb_list[i].feat, NULL );
			}
		}
		clear_img( &dst );
		clear_blbuf( &res );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": init_globals - no image found (%ld)!!\n", *src );


	dprintf((" -- " TOOL_PREFIX ": init_globals (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int geti_gui_init( char *buf, char *gui_prop, int *val )
{
	dprintf((" ++ " TOOL_PREFIX ": geti_gui_init - buf >>%s<<, gui_prop >>%s<<\n", buf, gui_prop ));
	int ret = -1;

	if( strncmp( buf, gui_prop, strlen ( gui_prop )) == 0 )
	{
		*val = atoi( buf + strlen ( gui_prop ) + 1 );
		ret = 0;
	}

	dprintf((" -- " TOOL_PREFIX ": geti_gui_init (ret: %d, gui_prop: %s val: %d)\n", ret, gui_prop, *val ));
	return ret;
}

//----------------------------------------------------------------------
int check_ival( int ival, int imin, int imax, int idefault )
{
	dprintf((" ++ " TOOL_PREFIX ": check_ival (ival: %d, imin: %d, imax: %d, idefault: %d)\n", ival, imin, imax, idefault ));
	int ret = idefault;

	if( ival >= imin && ival <= imax )
		ret = ival;

	dprintf((" -- " TOOL_PREFIX ": check_ival (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int read_from_init( BLOBS *blb_list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": read_from_init\n" ));
	int ret = IPL_FALSE;
	FILE *f = NULL;
	char buf[MAX_CHAR_SIZE], tmp[MAX_BUF_SIZE], *s0, *s1;
	int mcnt = 0, pf = -1, pol = get_polarity( );
	int th = get_threshold( ), auto_th = -1;
	int len1, len2;

	memset( buf, 0, MAX_CHAR_SIZE );
	memset( tmp, 0, MAX_BUF_SIZE );

	if(( f = fopen( INIT_FILE, FREAD )) != NULL )
	{
		for( int i = 0; i < list_size; i++ )
		{
			rewind( f );
			for(  ; fgets( buf, MAX_CHAR_SIZE, f ) != NULL;  )
			{
				if( geti_gui_init( buf, INIT_STR_PROC, &pf ) >= 0 )
				{
					process_flag = check_ival( pf, 0, 1, process_flag );
				}
				else if( geti_gui_init( buf, INIT_STR_MAXFCNT, &mcnt ) >= 0 )
				{
					set_max_feat_nb( check_ival( mcnt, 1, get_max_feat( ), max_feat_list_cnt ));
				}
				else if( geti_gui_init( buf, INIT_STR_POL, &pol ) >= 0 )
				{
					set_polarity( check_ival( pol, 0, 1, get_polarity( )));
				}
				else if( geti_gui_init( buf, INIT_STR_AUTO_TH, &auto_th ) >= 0 )
				{
					use_auto_threshold( check_ival( auto_th, 0, 1, 0 ));
				}
				else if( geti_gui_init( buf, INIT_STR_TH, &th ) >= 0 )
				{
					set_threshold( check_ival( th, 0, 255, get_threshold( )));
				}
				else if( strncmp( buf, blb_list[i].name, strlen ( blb_list[i].name )) == 0 )
				{
					s0 = strstr( buf, TIME_MS_STR );
					s1 = strstr( buf, RANGE_MIN_STR );

					len1 = strlen( TIME_MS_STR ) + 1 ;
					len2 = strlen( s0 ) - strlen( s1 ) - 1;

					strncpy( tmp, s0 + len1,  len2 - len1 );
					dprintf((" == " TOOL_PREFIX ": read_from_init - " TIME_MS_STR " >%s<\n", tmp ));
					blb_list[i].time_ms = atof( tmp );

					memset( tmp, 0, MAX_BUF_SIZE );
					s0 = strstr( buf, RANGE_MAX_STR );

					len1 = strlen( RANGE_MIN_STR ) + 1 ;
					len2 = strlen( s1 ) - strlen( s0 ) - 1;

					strncpy( tmp, s1 + len1,  len2 - len1 );
					dprintf((" == " TOOL_PREFIX ": read_from_init - " RANGE_MIN_STR " >%s<\n", tmp ));
					blb_list[i].range.x1 = atof( tmp );

					memset( tmp, 0, MAX_BUF_SIZE );
					s1 = strstr( buf, "\n" );

					len1 = strlen( RANGE_MAX_STR ) + 1 ;
					len2 = strlen( s0 ) - strlen( s1 );

					strncpy( tmp, s0 + len1,  len2 - len1 );
					dprintf((" == " TOOL_PREFIX ": read_from_init - " RANGE_MAX_STR " >%s<\n", tmp ));
					blb_list[i].range.x2 = atof( tmp );

					memset( tmp, 0, MAX_BUF_SIZE );
					break;
				}
			}
		}

		fclose( f );

		ret = IPL_TRUE;
	}

	dprintf((" -- " TOOL_PREFIX ": read_from_init (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int check_init_file( )
{
	dprintf((" ++ " TOOL_PREFIX ": check_init_file\n" ));

	int ret = IPL_FALSE;
	FILE *f = NULL;

	if(( f = fopen( INIT_FILE, FREAD )) != NULL )
	{
		fclose(f);
		ret = IPL_TRUE;
		blob_init_flag = 1;
	}
	dprintf((" -- " TOOL_PREFIX ": check_init_file (%d)\n", ret  ));
	return ret;
}

//----------------------------------------------------------------------
#define WGUIINI_I( a, b )							\
		fprintf( f, INIT_STR_##a "=%d\n", b );

#define WGUIINI_L( a, b )							\
		fprintf( f, INIT_STR_##a "=%ld\n", b );

int write2init( BLOBS *blb_list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": write2init\n" ));

	int ret = IPL_FALSE;

	if( blob_init_flag )
	{
		FILE *f = NULL;
		static time_t t;

		if(( f = fopen( INIT_FILE, FWRITE )) != NULL )
		{
			fprintf( f, "\n");
			time( &t );
			fprintf( f, "## %s\n", ctime( &t ));

			fprintf( f, "\n[GUI]\n");
			WGUIINI_I( PROC, process_flag );
			WGUIINI_I( MAXFCNT, max_feat_list_cnt );
			WGUIINI_I( POL, get_polarity( ));
			WGUIINI_I( AUTO_TH, get_threshold_mode( ));
			WGUIINI_L( TH, get_threshold( ));

			fprintf( f, "\n[GLOBALS]\n");
			for( int i = 0; i < list_size; i++ )
			{
				dprintf((" == " TOOL_PREFIX ": write2init - %s " TIME_MS_STR " %f " RANGE_MIN_STR " %f " RANGE_MAX_STR " %f\n",
						blb_list[i].name, blb_list[i].time_ms, blb_list[i].range.x1, blb_list[i].range.x2 ));

				fprintf( f, "%s " TIME_MS_STR " %f " RANGE_MIN_STR " %f " RANGE_MAX_STR " %f\n",
						blb_list[i].name, blb_list[i].time_ms, blb_list[i].range.x1, blb_list[i].range.x2 );
			}

			fclose( f );
			ret = IPL_TRUE;
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": write2init - file %s not opened!!\n", INIT_FILE );
	}

	dprintf((" -- " TOOL_PREFIX ": write2init (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void w_on_close( )
{
	dprintf((" ++ " TOOL_PREFIX ": w_on_close\n" ));

	write2init( class_list[REF_CLASS].blb_list, class_list[REF_CLASS].list_size );

	dprintf((" -- " TOOL_PREFIX ": " TOOL_PREFIX ": w_on_close\n" ));
}

//----------------------------------------------------------------------
void wr_res2file( BL_BUFHANDLE featHandle, BL_BUFHANDLE resHandle,
				BLOBS *blb_list, int list_size, int blobs, char *fname )
{
	dprintf((" ++ " TOOL_PREFIX ": wr_res2file (fname: %s, blobs: %d, blb_list_size: %d)\n",
					fname, blobs, list_size ));
	FILE *f = NULL;
	static time_t t;

	if( featHandle && resHandle && blobs && list_size && blb_list )
	{
		if(( f = fopen( fname, "w" )) != NULL )
		{
			time( &t );
			fprintf( f, "\n## %s\n", ctime( &t ));
			fprintf( f, "Blob count: %3d\n", blobs );
			float time_ms = ( float )(diff_tv.tv_sec * 1000 + diff_tv.tv_usec/1000.);
			fprintf( f, "Analyse for %d props: %.3f msec\n", list_size, time_ms );
			printf( "Analyse for %d props: %.3f msec\n", list_size, time_ms );

			for( int c = 0; c < blobs; c++ )
			{
				fprintf( f, "\nBlob Number: %3d\n", c );
				fprintf( f, "================\n");
				if( blb_list[AREA].results[c] > MIN_BLB_AREA )
				{
					for( int i = 0; i < list_size; i++ )
					{
						if( blb_list[i].items == 1 )
							fprintf( f, "%s = %.3f\n", blb_list[i].name, blb_list[i].results[c] );
						else if( blb_list[i].items == 2 )
							fprintf( f, "%s = %.3f / %.3f\n", blb_list[i].name, blb_list[i].results[2*c], blb_list[i].results[2*c + 1] );
						else if( blb_list[i].items == 4 )
							fprintf( f, "%s = %.3f / %.3f / %.3f / %.3f\n", blb_list[i].name,
										blb_list[i].results[4*c], blb_list[i].results[4*c + 1], blb_list[i].results[4*c + 2], blb_list[i].results[4*c + 3] );
					}
				}
				else
					fprintf( f, "%s = %.3f\n", blb_list[AREA].name, blb_list[AREA].results[c] );
			}

			fclose( f );
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": wr_res2file - no file called >%s< found\n", fname );
	}

	dprintf((" -- " TOOL_PREFIX ": wr_res2file\n"));
}

//----------------------------------------------------------------------
int fget_class_cnt( FILE *f )
{
	dprintf((" ++ " TOOL_PREFIX ": fget_class_cnt\n"));
	int ret = 0;
	char buf[MAX_CHAR_SIZE];
	memset( buf, 0, MAX_CHAR_SIZE );

	rewind( f );
	for(  ; fgets( buf, MAX_CHAR_SIZE, f ) != NULL;  )
	{
		dprintf((" == " TOOL_PREFIX ": buf: >%s< c: >%s<\n", buf, CL_CNT_STR ));
		if( strncmp( buf, CL_CNT_STR, strlen( CL_CNT_STR )) == 0 )
		{
			ret = atoi( buf + strlen( CL_CNT_STR ) + 1 );
			dprintf((" == " TOOL_PREFIX ": fget_class_cnt - ret: %d\n", ret ));
			break;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": fget_class_cnt (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int fget_ival( char *buf, char *s )
{
	dprintf((" ++ " TOOL_PREFIX ": fget_ival (%s, %s)\n", buf, s ));
	int ret = -1, len = strlen( s ), sl0 = 0, sl1 = 0;
	char *s0 = NULL, *s1 = NULL, tmp[MAX_BUF_SIZE];
	memset( tmp, 0, MAX_BUF_SIZE );

	s0 = strstr( buf, s );
	sl0 = strlen( s0 );
	s1 = strstr( s0 + len + 2, " " );
	sl1 = strlen( s1 );

	if( s1 == NULL )
	{
		s1 = strstr( s0 + len + 2, "\n" );
		sl1 = 2;
	}
	else
		sl1 = strlen( s1 );

	dprintf((" == " TOOL_PREFIX ": fget_ival - l0: %d, l1: %d, s1: >%s<, s0: >%s<\n", sl0, sl1, s1, s0 ));

	strncpy( tmp, s0 + len + 1, sl0 - sl1 - 1 );
	dprintf((" == " TOOL_PREFIX ": fget_ival - tmp: >%s<\n", tmp ));
	ret = atoi( tmp );

	dprintf((" -- " TOOL_PREFIX ": fget_ival (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
double fget_fval( char *buf, char *s )
{
	dprintf((" ++ " TOOL_PREFIX ": fget_fval (%s, %s)\n", buf, s ));
	double ret = -1;
	int len = strlen( s ), sl0 = 0, sl1 = 0;
	char *s0 = NULL, *s1 = NULL, tmp[MAX_BUF_SIZE];
	memset( tmp, 0, MAX_BUF_SIZE );

	s0 = strstr( buf, s );
	sl0 = strlen( s0 );
	s1 = strstr( s0 + len + 2, " " );

	if( s1 == NULL )
	{
		s1 = strstr( s0 + len + 2, "\n" );
		sl1 = 2;
	}
	else
		sl1 = strlen( s1 );

	dprintf((" == " TOOL_PREFIX ": fget_fval - l0: %d, l1: %d, s1: >%s<, s0: >%s<\n", sl0, sl1, s1, s0 ));

	strncpy( tmp, s0 + len + 1, sl0 - sl1 );
	dprintf((" == " TOOL_PREFIX ": fget_fval - tmp: >%s<\n", tmp ));
	ret = atof( tmp );

	dprintf((" -- " TOOL_PREFIX ": fget_fval (%f)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int get_file_val( char *buf, int index, int class_index )
{
	dprintf((" ++ " TOOL_PREFIX ": get_file_val (i: %d/cc: %d) - %s\n", index, class_index, buf ));
	int ret = -1, len = 0, itmp = 0;
	char tmp[MAX_BUF_SIZE];
	static int iEntryCount = 0;

	switch( index )
	{
		case 0:
			if( strncmp( buf, CL_NAME_STR, strlen( CL_NAME_STR )) == 0 )
			{
				memset( tmp, 0, MAX_BUF_SIZE );
				len = strlen( CL_NAME_STR ) + 1;
				strncpy( tmp, buf + len, strlen( buf ) - len - strlen("\n"));
				dprintf((" == " TOOL_PREFIX ": cn: >%s<\n", tmp ));
				mk_new_blb_class( tmp );
				iEntryCount = 0;
				ret = 0;
			}
			else if( strncmp( buf, "\n", strlen( "\n" )) == 0 )
			{
				ret = 0;
			}
			else
				fprintf( stderr, "*** " TOOL_PREFIX ": get_file_val - no class name found!!\n");
			break;

		default:
			if( strncmp( buf, "\n", strlen( "\n" )) != 0 )
			{
				itmp = fget_ival( buf, ID_STR );
				if( itmp >= 0 && iEntryCount <= class_list[class_index].list_size )
				{
					class_list[class_index].blb_list[itmp].delta = fget_fval( buf, DELTA_STR );
					class_list[class_index].blb_list[itmp].time_ms = fget_fval( buf, TIME_MS_STR );

					class_list[class_index].blb_list[itmp].range.x1 = fget_fval( buf, RANGE_MIN_STR );
					class_list[class_index].blb_list[itmp].range.x2 = fget_fval( buf, RANGE_MAX_STR );

					class_list[class_index].blb_list[itmp].sum_cnt = fget_ival( buf, SUM_CNT_STR );
					class_list[class_index].blb_list[itmp].sum = fget_fval( buf, SUM_STR );
					class_list[class_index].blb_list[itmp].sum_2 = fget_fval( buf, SUM_2_STR );

					dprintf((" == " TOOL_PREFIX ": get_file_val - index: %d/%d, id: %d, D: %f, t: %f, min: %f, max: %f, sc: %ld, s: %f, s2: %f\n",
											class_index, class_list_size, itmp,
											class_list[class_index].blb_list[itmp].delta,
											class_list[class_index].blb_list[itmp].time_ms,
											class_list[class_index].blb_list[itmp].range.x1,
											class_list[class_index].blb_list[itmp].range.x2,
											class_list[class_index].blb_list[itmp].sum_cnt,
											class_list[class_index].blb_list[itmp].sum,
											class_list[class_index].blb_list[itmp].sum_2 ));
					ret = 0;
					iEntryCount++;
				}
			}
			else
				ret = 0;

			break;
	}

	return ret;
}

//----------------------------------------------------------------------
int check_and_read_bin( )
{
	dprintf((" ++ " TOOL_PREFIX ": check_and_read_bin\n"));
	int ret = 1, cnt = 0, index = 0, cc = class_list_size;
	FILE *f = NULL;
	char buf[MAX_CHAR_SIZE];
	memset( buf, 0, MAX_CHAR_SIZE );

	if(( f = fopen( BIN_FILE, FREAD )) != NULL )
	{
		cnt = fget_class_cnt( f );
		dprintf((" == " TOOL_PREFIX ": check_and_read_bin - cnt: %d\n", cnt ));

		for(  ; fgets( buf, MAX_CHAR_SIZE, f ) != NULL;  )
		{
			if( strncmp( buf, CL_NAME_STR, strlen( CL_NAME_STR )) == 0 )
			{
				index = 0;
				cc = class_list_size;
			}

			if( get_file_val( buf, index++, cc ) < 0 )
			{
				fprintf( stderr, "*** " TOOL_PREFIX ": check_and_read_bin - get values from bin file failed!!\n >%s<", buf );
				ret = 0;
				break;
			}

			memset( buf, 0, MAX_CHAR_SIZE );
		}

		fclose( f );

		if( cnt + 1 != (int)class_list_size )
		{
			fprintf( stderr, "*** " TOOL_PREFIX ": check_and_read_bin - read data from bin file failed(aspect: %d, is: %d)!!\n", cnt + 1, class_list_size );
			read_bin_file_flag = 0;
			ret = 0;
		}
		else
			read_bin_file_flag = 1;
	}

	dprintf((" -- " TOOL_PREFIX ": check_and_read_bin (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void save_class_img( BLBIMG *img, BYTE *data, unsigned long x, unsigned long y,
					unsigned long width, unsigned long height, unsigned long pitch, unsigned int bpp )
{
	dprintf((" ++ " TOOL_PREFIX ": save_class_img (x: %ld, y: %ld, width: %ld, height: %ld, pitch: %ld, bpp: %d)\n",
								x, y, width, height, pitch, bpp ));

	if( data )
	{
		if( width > 0 && height > 0 && x < width && y < height && width <= pitch && bpp > 0 )
		{
			if( bpp == 8 )
			{
				if( img )
				{
					img->xpos = x;
					img->ypos = y;
					img->width = width;
					img->height = height;
					img->pitch = pitch;
					img->bpp = bpp;

					allocprintf( img->data, i );
					if(( img->data = (BYTE*) malloc( sizeof(BYTE) * pitch * height * bpp/8 )) != NULL )
					{
						allocprintf( img->data, o );
						cp_img( data, img->data, width, height, img->pitch, pitch );
					}
				}
				else
					fprintf( stderr, "*** " TOOL_PREFIX ": save_class_img - empty destination (%p)!!\n", img );
			}
			else
				fprintf( stderr, "*** " TOOL_PREFIX ": save_class_img - can save just 8bit gray images (bpp: %d)!!\n", bpp );
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": save_class_img - wrong image size (x/y: %ld/%ld, w/h/p: %ld/%ld/%ld, bpp: %d)!!\n",
										x, y, width, height, pitch, bpp );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": save_class_img - no image data (%p)!!\n", data );

	dprintf((" -- " TOOL_PREFIX ": save_class_img\n" ));
}

//----------------------------------------------------------------------
int get_blob_init_flag( ){	return blob_init_flag; }

//----------------------------------------------------------------------
void init_blb_globals( DEV_HANDLE devHandle, IPL_BUFHANDLE *imgHandle )
{
	mk_blb_initdlg( );

	clear_blbuf( &results );
	CHECK_RET(( BL_BufAlloc( &results, BL_RESULTBUF )));

	init_globals( devHandle, imgHandle, features, class_list[REF_CLASS].blb_list, class_list[REF_CLASS].list_size, 5, get_polarity( ));
	blob_init_flag = 1;

	write2init( class_list[REF_CLASS].blb_list, class_list[REF_CLASS].list_size );
	rm_blb_initdlg( );
}

//----------------------------------------------------------------------
int init_blobs( )
{
	dprintf((" ++ " TOOL_PREFIX ": init_blobs\n" ));
	int ret = 0;

	class_list[REF_CLASS].list_size = sizeof( ini_blb_list )/sizeof( ini_blb_list[0] );
	memset( class_list[REF_CLASS].name, 0, MAX_BUF_SIZE );
	sprintf( class_list[REF_CLASS].name, "REFERENCE_CLASS" );

	if( malloc_blb_list( &class_list[REF_CLASS].blb_list, class_list[REF_CLASS].list_size ))
	{
		if(( ret = init_blb_list( class_list[REF_CLASS].blb_list, class_list[REF_CLASS].list_size )))
		{
			CHECK_RET(( ret = BL_BufAlloc( &features, BL_FEATUREBUF )));
			CHECK_RET(( ret = BL_SetProcMode( features, BLOC_CTLID_STOP_AFTER_INDEX, MAX_BLB_CNT )));
			CHECK_RET(( ret = BL_BufAlloc( &results, BL_RESULTBUF )));

			if( check_init_file( ) )
				read_from_init( class_list[REF_CLASS].blb_list, class_list[REF_CLASS].list_size );

			class_list[REF_CLASS].img_cnt = 0;
			class_list[REF_CLASS].img = NULL;

			allocprintf( feat_list, i );
			if(( feat_list = (unsigned int*)malloc( class_list[REF_CLASS].list_size * sizeof( unsigned int ))) == NULL )
				fprintf(stderr, "*** " TOOL_PREFIX ": init_blobs - out of memory\n" );
			else
			{
				allocprintf( feat_list, o );
				class_list_size++;
				ret = check_and_read_bin( );
			}
		}
	}

#ifdef DEBUG
	for( unsigned int i = 0; i < class_list_size; i++ )
		print_class_infos( class_list[i], 1, stdout );
#endif	// DEBUG

	dprintf((" -- " TOOL_PREFIX ": init_blobs\n" ));
	return ret;
}

//----------------------------------------------------------------------
int set_all_features( int index )
{
	dprintf((" -> " TOOL_PREFIX ": set_all_features - index: %d, name: %s\n", index, class_list[index].name ));
	return set_features( features, class_list[index].blb_list, class_list[index].list_size );
}

//----------------------------------------------------------------------
int rm_all_features( int index )
{
	dprintf((" -> " TOOL_PREFIX ": rm_all_features - index: %d, name: %s\n", index, class_list[index].name ));
	return rm_features( features, class_list[index].blb_list, class_list[index].list_size );
}

//----------------------------------------------------------------------
int set_all_tmp_features( )
{
	dprintf((" -> " TOOL_PREFIX ": set_all_tmp_features\n" ));
	return set_features( features, tmp_res.blb_list, tmp_res.list_size );
}

//----------------------------------------------------------------------
int rm_all_tmp_features( )
{
	dprintf((" -> " TOOL_PREFIX ": rm_all_tmp_features\n" ));
	return rm_features( features, tmp_res.blb_list, tmp_res.list_size );
}

//----------------------------------------------------------------------
void free_blobs( )
{
	dprintf((" ++ " TOOL_PREFIX ": free_blobs\n" ));

	for( unsigned int i = 0; i < class_list_size; i++ )
	{
		clear_bufs( class_list[i].blb_list, class_list[i].list_size );
		free_buf_list( class_list[i].blb_list );
	}

	freeprintf( feat_list, i );
		feat_list = (unsigned int*)clear_buf( feat_list );
	freeprintf( feat_list, o );

	clear_tmp_buf( );
	free_buf_list( tmp_res.blb_list );

	clear_blbuf( &features );
	clear_blbuf( &results );

	dprintf((" -- " TOOL_PREFIX ": free_blobs\n" ));
}

//----------------------------------------------------------------------
void clear_tmp_buf( )
{
	dprintf((" ++ " TOOL_PREFIX ": clear_tmp_buf\n" ));
	clear_bufs( tmp_res.blb_list, tmp_res.list_size );
	dprintf((" -- " TOOL_PREFIX ": clear_tmp_buf\n" ));
}

//----------------------------------------------------------------------
void init_tmp_buf( )
{
	dprintf((" ++ " TOOL_PREFIX ": init_tmp_buf\n"));
	tmp_res.list_size = class_list[REF_CLASS].list_size;
	memset( tmp_res.name, 0, MAX_BUF_SIZE );
	sprintf( tmp_res.name, "ACTUAL_CLASS" );

	if( malloc_blb_list( &tmp_res.blb_list, tmp_res.list_size ))
	{
		if( init_blb_list( tmp_res.blb_list, tmp_res.list_size ))
		{
			if( class_list_size )
				set_ref_time( tmp_res );

		}
	}
	dprintf((" -- " TOOL_PREFIX ": init_tmp_buf\n"));
}

//----------------------------------------------------------------------
int get_tmp_res( )
{
	dprintf((" ++ " TOOL_PREFIX ": get_tmp_res\n" ));
	int ret = 0;
	double blobs = 0;
	CHECK_RET( BL_ResultBufInquire( results, 0, 0, 0, (BL_FEATURE)-1, 0, BL_GET_NB_BLOBS, &blobs ));

	dprintf((" == " TOOL_PREFIX ": get_blb_res - blobs: %.1f\n", blobs ));
	if( blobs > 0 )
	{
		get_results( results, tmp_res.blb_list, tmp_res.list_size, (int)blobs );

#if DEBUG
		print_class_infos( tmp_res, (int)blobs, stdout );
#endif	// DEBUG

		ret = (int)blobs;
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": keine obj gefunde!!\n");

	dprintf((" -- " TOOL_PREFIX ": get_tmp_res\n" ));
	return ret;
}

//----------------------------------------------------------------------
#ifdef EXPORT_IMGS
static int img_c = 0;
#endif	// EXPORT_IMGS
int analyse( IPL_BUFHANDLE greyimgHandle, IPL_BUFHANDLE *binimgHandle )
{
	dprintf((" ++ " TOOL_PREFIX ": analyse\n" ));
	int ret = IPL_TRUE;
	IPL_BUFHANDLE dummy = IPL_DONT_CARE;

#ifdef EXPORT_IMGS
	char msg[MAX_BUF_SIZE];
	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "analyse_grey_%03d.bmp", img_c );
	CHECK_RET( IPL_FlExport( greyimgHandle, IPL_DONT_CARE, IPL_BMP, msg ));
	printf("see: %s\n",msg);

	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "analyse_bin_%03d.bmp", img_c );
	CHECK_RET( IPL_FlExport( *binimgHandle, IPL_DONT_CARE, IPL_BMP, msg ));
	printf("see: %s\n",msg);
	img_c++;
#endif	// EXPORT_IMGS

	clear_blbuf( &results );
	CHECK_RET(( ret = BL_BufAlloc( &results, BL_RESULTBUF )));

	StartTimer( &start_tv );
		if( !( ret = BL_Analyze( *binimgHandle, greyimgHandle, &dummy/*binimgHandle*/, features, results )))
		{
			ret = IPL_GetErrorCode2( );
			fprintf( stderr, "*** " TOOL_PREFIX ": analyse - BL_Analyze failed err: %d!!\n", ret );
		}
	ElapsedTime( &start_tv, &diff_tv, 30 );

	clear_img( &dummy );

	dprintf((" -- " TOOL_PREFIX ": analyse (calc: %f ms)\n", get_calc_time_ms( )));
	return ret;
}

//----------------------------------------------------------------------
int check_found_blobs( int blobs, double **bb_buf, unsigned int *ilist )
{
	dprintf((" ++ " TOOL_PREFIX ": check_found_blobs (%d)\n", blobs ));
	int ret = 0, icnt = 0;
	double *area_buf = NULL, *tmp_buf = NULL;

	if( blobs > 0 )
	{
		allocprintf( area_buf, i );
		if(( area_buf = (double*) malloc( blobs * sizeof(double))) != NULL )
		{
			allocprintf( area_buf, o );
			CHECK_RET( BL_GetResultBufData( results, 0, 0, 0, BL_AR_AREA, 0, area_buf ));
		}
		else
			fprintf( stderr,"*** " TOOL_PREFIX ": check_found_blobs - out of memory!!\n");

		allocprintf( tmp_buf, i );
		if(( tmp_buf = (double*) malloc(( blobs * 4 )* sizeof(double))) != NULL )
		{
			allocprintf( tmp_buf, o );
			CHECK_RET( BL_GetResultBufData( results, 0, 0, 0, BL_AR_EXTREME_BOX, 0, tmp_buf ));
		}
		else
			fprintf( stderr,"*** " TOOL_PREFIX ": check_found_blobs - out of memory!!\n");

		for( int i = 0; i < blobs; i++ )
		{
			if( area_buf[i] > MIN_BLB_AREA )
				ilist[icnt++] = i;
		}

		for( int i = 0; i < icnt; i++ )
		{
			tmp_buf[4*i] = tmp_buf[ 4*ilist[i] ];
			tmp_buf[4*i+1] = tmp_buf[ 4*ilist[i]+1 ];
			tmp_buf[4*i+2] = tmp_buf[ 4*ilist[i]+2 ];
			tmp_buf[4*i+3] = tmp_buf[ 4*ilist[i]+3 ];
		}
	}

	ret = icnt;


	freeprintf( *bb_buf, i );
		*bb_buf = (double*)clear_buf( *bb_buf );
	freeprintf( *bb_buf, o );

	allocprintf( *bb_buf, i );
	if(( *bb_buf = (double*) malloc(( ret * 4 )* sizeof(double))) != NULL )
	{
		allocprintf( *bb_buf, o );
		memcpy( *bb_buf, tmp_buf, 4 * ret * sizeof(double));

#ifdef DEBUG
		for( int c = 0; c < 4*ret; c++ )
			printf(" == " TOOL_PREFIX ": check_found_blobs - ret: %d -  bb_buf[%3d]: %.0f\n", ret, c, tmp_buf[c] );
#endif	// DEBUG
	}

	freeprintf( area_buf, i );
		area_buf = (double*)clear_buf( area_buf );
	freeprintf( area_buf, o );

	freeprintf( tmp_buf, i );
		 tmp_buf = (double*)clear_buf( tmp_buf );
	freeprintf( tmp_buf, o );

	dprintf((" -- " TOOL_PREFIX ": check_found_blobs (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int get_bounding_box( IPL_BUFHANDLE greyimgHandle, IPL_BUFHANDLE *binimgHandle,
					double **res_buf, unsigned int *ilist )
{
	dprintf((" ++ " TOOL_PREFIX ": get_bounding_box\n" ));

	double blb_cnt = 0;
	int ok_blb_cnt = 0;

#ifdef EXPORT_IMGS
	CHECK_RET( IPL_FlExport( greyimgHandle, IPL_DONT_CARE, IPL_BMP, "get_bounding_box_grey.bmp" ));
	CHECK_RET( IPL_FlExport( *binimgHandle, IPL_DONT_CARE, IPL_BMP, "get_bounding_box_bin.bmp" ));
#endif	// EXPORT_IMGS

	if( set_feat4classify_flag )
		rm_feat4classify( );

	set_single_feat( features, BL_AR_AREA, NULL );
	set_single_feat( features, BL_AR_EXTREME_BOX, NULL );

	clear_blbuf( &results );
	CHECK_RET(( BL_BufAlloc( &results, BL_RESULTBUF )));

	if( BL_Analyze( *binimgHandle, greyimgHandle, binimgHandle, features, results ))
	{
		CHECK_RET( BL_ResultBufInquire( results, 0, 0, 0, (BL_FEATURE)-1, 0, BL_GET_NB_BLOBS, &blb_cnt ));
		dprintf((" == " TOOL_PREFIX ": get_bounding_box - cnt: %.0f\n", blb_cnt ));

		if( blb_cnt > 0 )
		{
			if( blb_cnt < MAX_BLB_CNT )
				ok_blb_cnt = check_found_blobs( (int)blb_cnt, res_buf, ilist );
			else
			{
				fprintf( stderr, "*** " TOOL_PREFIX ": get_bounding_box - to many blobs found (%.0f, max: %d)!!\n", blb_cnt, MAX_BLB_CNT );
				blb_cnt = 0.;
			}
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": get_bounding_box - no blob in the image!!\n");
	}
	else
	{
		if(IPL_GetErrorCode2( ) == BL_STOPPED_AFTER_INDEX )
			fprintf( stderr, "*** " TOOL_PREFIX ": get_bounding_box - Stopped after %d objects.\n", MAX_BLB_CNT );
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": get_bounding_box - BL_Analyze failed (err: %ld)\n", IPL_GetErrorCode2( ));
	}

	rm_single_feat( features, BL_AR_AREA, NULL );
	rm_single_feat( features, BL_AR_EXTREME_BOX, NULL );

	dprintf((" -- " TOOL_PREFIX ": get_bounding_box (%d)\n", ok_blb_cnt ));
	return ok_blb_cnt;
}

//----------------------------------------------------------------------
int mk_new_blb_class( char *class_name )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_new_blb_class (%s, %d)\n", class_name, class_list_size ));
	if( class_list_size < MAX_CLASS_NB )
	{
		memset( class_list[class_list_size].name, 0, MAX_BUF_SIZE );
		sprintf( class_list[class_list_size].name, "%s", class_name );
		class_list[class_list_size].list_size = class_list[REF_CLASS].list_size;

		if( malloc_blb_list( &class_list[class_list_size].blb_list, class_list[class_list_size].list_size ))
		{
			if( init_blb_list( class_list[class_list_size].blb_list, class_list[class_list_size].list_size ))
				set_ref_time( class_list[class_list_size] );
		}

		class_list[class_list_size].img_cnt = 0;
		class_list[class_list_size].img = NULL;

		class_list_size++;

		if( class_list_size > 2 )
			read_bin_file_flag = 1;
	}
	else
		note_or_err_msg( AN_ERROR, "Maximal possibile number of classes reached", 0 );

	dprintf((" -- " TOOL_PREFIX ": mk_new_blb_class (%d)\n", class_list_size ));
	return class_list_size;
}

//----------------------------------------------------------------------
void rm_blb_class( char *class_name )
{
	dprintf((" ++ " TOOL_PREFIX ": rm_blb_class\n"));

	for( unsigned int i = 0; i < class_list_size; i++ )
	{
		if( strncmp( class_list[i].name, class_name, strlen( class_list[i].name )) == 0 )
		{
			clear_bufs( class_list[i].blb_list, class_list[i].list_size );
			free_buf_list( class_list[i].blb_list );
			class_list_size--;
			break;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": rm_blb_class\n"));
}

//----------------------------------------------------------------------
int get_class_cnt( )
{
	dprintf((" -> " TOOL_PREFIX ": get_class_cnt (%d)\n", class_list_size ));

	return class_list_size;
}

//----------------------------------------------------------------------
int get_blb_class_name( unsigned int index, char *name )
{
	dprintf((" ++ " TOOL_PREFIX ": get_blb_class_name (%d)\n", index ));
	int ret = -1;
	if( index < class_list_size )
	{
		strncpy( name, class_list[index].name, strlen( class_list[index].name ));
		dprintf((" == " TOOL_PREFIX ": get_blb_class_name - %s\n", name ));
		ret = 0;
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": get_blb_class_name - wrong index(%d >= %d)\n", index, class_list_size );

	dprintf((" -- " TOOL_PREFIX ": get_blb_class_name (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int get_class_index( char *class_name )
{
	dprintf((" ++ " TOOL_PREFIX ": get_class_index\n"));

	int ret = -1;
	for( unsigned int i = 0; i < class_list_size; i++ )
	{
		if( strncmp( class_list[i].name, class_name, strlen( class_list[i].name )) == 0 )
		{
			ret = i;
			break;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": get_class_index (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void save_blob2class( BLOBCLASS bc, int index, int obj_index )
{
	dprintf((" ++ " TOOL_PREFIX ": save_blob2class (index: %d, obj_index: %d)\n", index, obj_index ));

	for(int i = 0; i < class_list[index].list_size; i++ )
	{
		dprintf((" == " TOOL_PREFIX ": save_blob2class - items(%s): %d, res: %.0f\n",
					bc.blb_list[i].name, bc.blb_list[i].items, bc.blb_list[i].results[obj_index] ));

		if( bc.blb_list[i].items == 1 )
		{
			if( class_list[index].blb_list[i].range.x1 == 0 )
				class_list[index].blb_list[i].range.x1 = bc.blb_list[i].results[obj_index];
			else if( bc.blb_list[i].results[obj_index] < class_list[index].blb_list[i].range.x1 )
				class_list[index].blb_list[i].range.x1 = bc.blb_list[i].results[obj_index];

			if( bc.blb_list[i].results[obj_index] > class_list[index].blb_list[i].range.x2 )
				class_list[index].blb_list[i].range.x2 = bc.blb_list[i].results[obj_index];

			class_list[index].blb_list[i].sum += bc.blb_list[i].results[obj_index];

			double r_2 = bc.blb_list[i].results[obj_index] * bc.blb_list[i].results[obj_index];
			class_list[index].blb_list[i].sum_2 += r_2;

			class_list[index].blb_list[i].sum_cnt++;
		}
	}

#ifdef DEBUG
	print_class_infos( class_list[index], 1, stdout );
#endif	//DEBUG

	dprintf((" -- " TOOL_PREFIX ": save_blob2class\n" ));
}

//----------------------------------------------------------------------
int save_settings2bin( )
{
	dprintf((" ++ " TOOL_PREFIX ": save_settings2bin\n" ));
	int ret = -1;
	FILE *f = NULL;

	if(( f = fopen( BIN_FILE, FWRITE )) != NULL )
	{
		fprintf( f, CL_CNT_STR " %d\n\n", class_list_size - 1 );

		for( unsigned int i = 1; i < class_list_size; i++ )
		{
			print_class_infos( class_list[i], 1, f );
			fprintf( f, "\n");
		}
		fclose( f );
		ret = 0;
	}

	dprintf((" -- " TOOL_PREFIX ": save_settings2bin (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void delete_class( int index )
{
	dprintf((" ++ " TOOL_PREFIX ": delete_class (index: %d)\n", index ));

#ifdef DEBUG
	print_class_infos( class_list[index], 1, stdout );
#endif	// DEBUG

	for( unsigned int i = index; i < class_list_size - 1; i++ )
	{
		strcpy( class_list[i].name, class_list[i+1].name );
		class_list[i].list_size = class_list[i+1].list_size;
		memcpy( class_list[i].blb_list, class_list[i+1].blb_list, sizeof(BLOBS));
		class_list[i].img_cnt = class_list[i+1].img_cnt;
		class_list[i].img = class_list[i+1].img;
	}

	clear_bufs( class_list[class_list_size - 1].blb_list, class_list[class_list_size - 1].list_size );
	free_buf_list( class_list[class_list_size - 1].blb_list );

	class_list_size--;

	save_settings2bin( );

	dprintf((" -- " TOOL_PREFIX ": delete_class\n" ));
}

//----------------------------------------------------------------------
void teach_all_results( int index, int obj_index, BYTE * /*img_data*/, unsigned long /*width*/, unsigned long /*height*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": teach_all_results (index: %d, obj_index: %d)\n", index, obj_index ));

	int ret, blobs = 0;

	if( set_feat4classify_flag )
		rm_feat4classify( );

	set_features( features, tmp_res.blb_list, tmp_res.list_size );

	get_bin_img( );

	if(( ret = calc_res( )))
	{
		blobs = get_tmp_res( );
		dprintf((" == " TOOL_PREFIX ": teach_all_results - cnt: %d\n", blobs ));

		if( blobs > 0 )
		{
			if( obj_index < blobs )
				save_blob2class( tmp_res, index, obj_index );
			else
				fprintf( stderr, "*** " TOOL_PREFIX ": teach_all_results - wrong blob index!!\n");
		}
		else
			fprintf( stderr,"*** " TOOL_PREFIX ": teach_all_results - no blob in the image!!\n");
	}

#ifdef DEBUG
	print_class_infos( tmp_res, blobs, stdout );
#endif	// DEBUG

	clear_bufs( tmp_res.blb_list, tmp_res.list_size );
	rm_features( features, tmp_res.blb_list, tmp_res.list_size );

	save_settings2bin( );

	dprintf((" -- " TOOL_PREFIX ": teach_all_results\n" ));
}

//----------------------------------------------------------------------
int check_if_overlap( INTERVAL *list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": check_if_overlap - list_size: %d\n", list_size ));
	int ret = 0;

	for( int i = 0; i < list_size - 1; i++ )
	{
		if( list[i].x2 >= list[i+1].x1 )
		{
			dprintf(("*** " TOOL_PREFIX ": INTERSECT (%f/%f) (%f/%f)\n", list[i].x1, list[i].x2, list[i+1].x1, list[i+1].x2 ));
			ret = 1;
			break;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": check_if_overlap (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int check_interval( INTERVAL *list, int list_size )
{
	dprintf((" ++ " TOOL_PREFIX ": check_interval (%d)\n", list_size ));
	int ret;

#ifdef DEBUG
	for( int k = 0; k < list_size; k++ )
		printf(" => " TOOL_PREFIX ": check_interval - (%3d) %f/%f\n", k, list[k].x1, list[k].x2 );
#endif	// DEBUG

	quicksort_iv( list, 0, list_size - 1 );

#ifdef DEBUG
	printf("\n");
	for( int k = 0; k < list_size; k++ )
		printf(" <= " TOOL_PREFIX ": check_interval - (%3d) %f/%f\n", k, list[k].x1, list[k].x2 );
#endif	// DEBUG

	ret = check_if_overlap( list, list_size );

	dprintf((" -- " TOOL_PREFIX ": check_interval (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int set_feat4classify( )
{
	dprintf((" ++ " TOOL_PREFIX ": set_feat4classify (feat_list_cnt: %d)\n", feat_list_cnt ));
	int ret = -1;
	if( features != IPL_DONT_CARE )
	{
		set_single_feat( features, BL_AR_EXTREME_BOX, NULL );

		for( unsigned int i = 0; i < feat_list_cnt; i++ )
		{
			dprintf((" == " TOOL_PREFIX ": set_feat4classify - name: %s, feat: 0x%x, index: %d\n",
				tmp_res.blb_list[feat_list[i]].name, tmp_res.blb_list[feat_list[i]].feat, feat_list[i] ));

			set_single_feat( features, tmp_res.blb_list[feat_list[i]].feat,
					tmp_res.blb_list[feat_list[i]].ctrlVal == 0. ? NULL : &(tmp_res.blb_list[feat_list[i]].ctrlVal));
		}
		set_feat4classify_flag = 1;
		ret = 0;
	}

	dprintf((" -- " TOOL_PREFIX ": set_feat4classify (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
int rm_feat4classify( )
{
	dprintf((" ++ " TOOL_PREFIX ": rm_feat4classify\n" ));
	int ret = -1;

	if( features != IPL_DONT_CARE )
	{
		rm_single_feat( features, BL_AR_EXTREME_BOX, NULL );
		for( unsigned int i = 0; i < feat_list_cnt; i++ )
			rm_single_feat( features, tmp_res.blb_list[feat_list[i]].feat, &( tmp_res.blb_list[feat_list[i]].ctrlVal ));

		set_feat4classify_flag = 0;
		ret = 0;
	}

	dprintf((" -- " TOOL_PREFIX ": rm_feat4classify (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void del_res4classify( )
{
	dprintf((" ++ " TOOL_PREFIX ": del_res4classify\n" ));

	for( unsigned int i = 0; i < feat_list_cnt; i++ )
		del_res_buf( &tmp_res.blb_list[feat_list[i]] );

	dprintf((" -- " TOOL_PREFIX ": del_res4classify\n" ));
}

//----------------------------------------------------------------------
/* TODO: ???????? needed ????????*/
int check_feat_usr_cnt( unsigned int * /*feat_list*/, unsigned int /*feat_list_cnt*/ )
{
	return 0;
}

//----------------------------------------------------------------------
void check_feat_usr_time( unsigned int *flist, unsigned int *flist_cnt )
{
	dprintf((" ++ " TOOL_PREFIX ": check_feat_usr_time\n" ));
	INTERVAL *tmp = NULL;
	BLOBS lblb;
	double tsum = 0.;

	if( *flist_cnt > 0 )
	{
		allocprintf( tmp, i );
		if(( tmp = (INTERVAL *)malloc( *flist_cnt * sizeof(INTERVAL))) != NULL )
		{
			allocprintf( tmp, o );
			memset( tmp, 0, *flist_cnt * sizeof( INTERVAL ));

			for( unsigned int i = 0; i < *flist_cnt; i++ )
			{
				lblb = class_list[REF_CLASS].blb_list[flist[i]];
				tmp[i].x1 = lblb.time_ms;
				tmp[i].x2 = flist[i];
				dprintf((" == " TOOL_PREFIX ": check_feat_usr_time - Feat: %s ==> (%.0f)t: %f\n", lblb.name, tmp[i].x2, tmp[i].x1 ));
			}

			quicksort_iv( tmp, 0, *flist_cnt - 1 );

#ifdef DBBUG
			for( unsigned int i = 0; i < *flist_cnt; i++ )
				printf(" => " TOOL_PREFIX ": check_feat_usr_time - (%.0f)t: %f\n", tmp[i].x2, tmp[i].x1 );
#endif	// DBBUG

			for( unsigned int i = 0; i < *flist_cnt; i++ )
			{
				dprintf((" == " TOOL_PREFIX ": check_feat_usr_time - (%.0f)t: %f\n", tmp[i].x2, tmp[i].x1 ));
				tsum += tmp[i].x1;
				dprintf((" == " TOOL_PREFIX ": check_feat_usr_time - tsum: %f\n", tsum ));
				if( (unsigned long) tsum > max_time_ms + 50 )
				{
					*flist_cnt = i;
					for( unsigned int j = 0; j < *flist_cnt; j++ )
						feat_list[j] = (unsigned int)tmp[j].x2;
					break;
				}
			}
		}
	}

	dprintf((" == " TOOL_PREFIX ": check_feat_usr_time - feat_list_cnt: %3d\n", feat_list_cnt ));

	freeprintf( tmp, i );
		tmp = (INTERVAL*)clear_buf( tmp );
	freeprintf( tmp, o );

	dprintf((" -- " TOOL_PREFIX ": check_feat_usr_time\n" ));
}

//----------------------------------------------------------------------
#include <math.h>

void get_range(  )
{
	dprintf((" ++ " TOOL_PREFIX ": get_range\n" ));
	double var = 0.0, stddev = 0.0, fact = 0.0, avg = 0.0;
	BLOBS lblb;
	int lcnt = class_list[REF_CLASS].list_size;
	dprintf((" == " TOOL_PREFIX ": get_range - lcnt: %d, class_list_size: %d\n",lcnt, class_list_size ));

	for( int i = 0; i < lcnt; i++ )
	{
		for( unsigned int j = 1; j < class_list_size; j++ )
		{
			lblb = class_list[j].blb_list[i];
			fact = ( 1.0/(double)lblb.sum_cnt );
			avg = fact * lblb.sum;

			var = fact * lblb.sum_2 - avg * avg;
			stddev = sqrt( var );
			dprintf((" == " TOOL_PREFIX ": get_range - Feat: %s class: %s ==> var: %f, stddev: %f, avg: %f (%f,%f)\n",
									lblb.name, class_list[j].name, var, stddev, avg,
									avg - 3*stddev, avg + 3*stddev ));

			class_list[j].blb_list[i].range.x1 = avg - 3*stddev;
			class_list[j].blb_list[i].range.x2 = avg + 3*stddev;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": get_range\n" ));
}

//----------------------------------------------------------------------
int classify( int blb_index )
{
	dprintf((" ++ " TOOL_PREFIX ": classify (feat_list_cnt: %d)\n", feat_list_cnt ));
	int ret = -1;
	BLOBS tblb, lblb;
	unsigned int matches[50];
	memset( matches, 0, 50 );

	for( unsigned int i = 0; i < feat_list_cnt; i++ )
	{
		for( unsigned int j = 1; j < class_list_size; j++ )
		{
			tblb = tmp_res.blb_list[feat_list[i]];
			lblb = class_list[j].blb_list[feat_list[i]];

			dprintf((" == " TOOL_PREFIX ": classify - feat: >%s< class:  >%s< ++ %f << %f << %f\n",
					tblb.name, class_list[j].name,
					lblb.range.x1, tblb.results[blb_index], lblb.range.x2 ));

			if( tblb.results[blb_index] >= lblb.range.x1 &&
				tblb.results[blb_index] <= lblb.range.x2 )
			{
				dprintf((" == " TOOL_PREFIX ": classify - feat: %s OK4 class:  %s\n\n", tblb.name, class_list[j].name ));
				matches[j-1]++;
				break;
			}
		}
	}

	for( unsigned int i = 0; i < class_list_size - 1; i++ )
	{
		dprintf((" == " TOOL_PREFIX ": classify - matches[%3d]: %3d\n", i, matches[i] ));
		if( matches[i] > feat_list_cnt/2 )
			ret = i + 1;
	}

	dprintf((" -- " TOOL_PREFIX ": classify (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int get_res4classify( )
{
	dprintf((" ++ " TOOL_PREFIX ": get_res4classify\n" ));
	int ret = -1;
	double blobs = 0, tms = 0, box[4*MAX_BLB_CNT];
	char msg[MAX_BUF_SIZE];

	if( results != IPL_DONT_CARE )
	{
		BL_ResultBufInquire( results, 0, 0, 0, (BL_FEATURE)-1, 0, BL_GET_NB_BLOBS, &blobs );

		dprintf((" == " TOOL_PREFIX ": get_res4classify - blobs: %d\n", (int)blobs ));
		if( blobs > 0 )
		{
			gl_blobs = (long)blobs;
			tms = get_calc_time_ms( );
			memset( box, 0, 4 * MAX_BLB_CNT * sizeof(double));
			BL_GetResultBufData( results, 0, 0, 0, BL_AR_EXTREME_BOX, 0, box );

			for( int i = 0; i < (int)blobs; i++ )
			{
				for( unsigned int j = 0; j < feat_list_cnt; j++ )
				{
					get_items( results, &tmp_res.blb_list[feat_list[j]] );
					get_single_res( results, &tmp_res.blb_list[feat_list[j]], (int)blobs );

					dprintf((" == " TOOL_PREFIX ": get_res4classify - %s - %f, time: %.3f ms\n",
							tmp_res.blb_list[feat_list[j]].name, tmp_res.blb_list[feat_list[j]].results[i], tms ));
				}

				dprintf((" == " TOOL_PREFIX ": get_res4classify - blob: %3d, %f, %f, %f, %f\n", i, box[4*i], box[4*i+1], box[4*i+2], box[4*i+3]));
				if(((long)( box[4*i+2] - box[4*i] ) * (long)( box[4*i+3] - box[4*i+1] )) >= MIN_BLB_AREA )
				{
					memset( msg, 0, MAX_BUF_SIZE );
					if(( ret = classify( i )) >= 0 )
						sprintf( msg, "%s (%.3f ms)", class_list[ret].name, tms );
					else
						sprintf( msg, "??UNKNOWN?? (%.3f ms)", tms );

					display_bounding_box( msg, (long)box[4*i], (long)box[4*i+1], (long)box[4*i+2], (long)box[4*i+3], ret - 1 );
				}

				del_res4classify( );
			}
		}
	}

	del_res4classify( );

	dprintf((" -- " TOOL_PREFIX ": get_res4classify (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void displ_feat_list( char *msg, long msg_len )
{
	dprintf((" ++ " TOOL_PREFIX ": displ_feat_list\n" ));
	char stmp[ MAX_CHAR_SIZE ];
	memset( stmp, 0, MAX_CHAR_SIZE );
	memset( msg, 0, msg_len );

	if( feat_list_cnt > max_feat_list_cnt )
		sprintf( msg, "\nPossibile features: %d\n"
					"Use the first %d features\n\n", feat_list_cnt ,max_feat_list_cnt );
	else
		sprintf( msg, "\nUse %d features\n\n", feat_list_cnt );

	for( unsigned int i = 0; i < feat_list_cnt; i++ )
	{
		sprintf( stmp, " %s (%3d)", class_list[REF_CLASS].blb_list[feat_list[i]].name, feat_list[i] );
		strncat( msg, stmp, strlen( stmp ));

		memset( stmp, 0, MAX_CHAR_SIZE );
		sprintf( stmp, "\n");
		strncat( msg, stmp, strlen( stmp ));
	}

	dprintf((" -- " TOOL_PREFIX ": displ_feat_list\n" ));
}

//----------------------------------------------------------------------
int check_feat( )
{
	int ret = -1;
	dprintf((" ++ " TOOL_PREFIX ": check_feat (list cnt: %d)\n", class_list_size - 1 ));

	feat_list_cnt = 0;
	if( read_bin_file_flag && class_list_size > 2 )
	{
		get_range( );

		INTERVAL *mins = NULL;
		allocprintf( mins, i );
		if(( mins = (INTERVAL*) malloc(( class_list_size - 1 ) * sizeof(INTERVAL))) != NULL )
		{
			allocprintf( mins, o );
			memset( mins, 0, (class_list_size - 1) * sizeof( INTERVAL ));

			for( int i = 0; i < class_list[REF_CLASS].list_size; i++ )
			{
				for( unsigned int j = REF_CLASS + 1; j < class_list_size; j++ )
				{
					mins[j-1].x1 = class_list[j].blb_list[i].range.x1;
					mins[j-1].x2 = class_list[j].blb_list[i].range.x2;
				}

				if( !check_interval( mins, class_list_size - 1 ))
					feat_list[feat_list_cnt++] = i;
			}

			freeprintf( mins, i );
				mins = (INTERVAL*)clear_buf( mins );
			freeprintf( mins, o );
		}
	}
	else
		note_or_err_msg( AN_ERROR, "No class taught! Plese teach some objects first", 0 );

	if( feat_list_cnt > 0 )
	{
		if( feat_list_cnt > max_feat_list_cnt )
			feat_list_cnt = max_feat_list_cnt;

		ret = 0;
	}

	dprintf((" -- " TOOL_PREFIX ": check_feat (ret: %d, feat_list_cnt: %d, max_feat_list_cnt: %d)\n", ret, feat_list_cnt, max_feat_list_cnt ));

	return ret;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
