//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>blob-afs<<
// Mit Aug 13 11:31:13 CEST 2003
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>
#include <vl_ipl.h>
#include <vl_file.h>

#include <mvslib.h>
#include <mvsensor.h>
#include <pgutils.h>
#include <samples.h>

#include "blob-afs.h"
#include "afsapi.h"
#include "blbproc.h"

#ifdef MALLOC_TRACE
	#include <mcheck.h>
#endif	// MALLOC_TRACE

//-------------------------------------------------------------------
//-------------------------------------------------------------------
#define USE_DOUBLE_BUFFERING

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static IPL_BUFHANDLE grey_img = IPL_DONT_CARE;
static IPL_BUFHANDLE bin_img = IPL_DONT_CARE;
static DEV_HANDLE device = IPL_DONT_CARE;

static BYTE *shm_buffer = NULL;
static BYTE *img_buffer = NULL;
static unsigned long width = 0, height = 0, pitch = 0, bpp = 0;

static long threshold = 0;
unsigned int run_flag = 0;
static int trig_mode = -1;

static unsigned int analyse_flag = 0;
static unsigned int polarity = 0;
static unsigned long err_code = 0;
static unsigned int dbi_flag = 0;
static unsigned int th_flag = 0;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
void *clear_buf( void *buf )
{
	dprintf((" ++ " TOOL_PREFIX ": clear_buf (%p)\n", buf ));
	if( buf != NULL )
	{
		free( buf );
		buf = NULL;
	}

	dprintf((" -- " TOOL_PREFIX ": clear_buf (%p)\n", buf ));

	return buf;
}

//----------------------------------------------------------------------
void clear_img( IPL_BUFHANDLE *img )
{
	dprintf((" ++ " TOOL_PREFIX ": clear_img (%ld)\n", *img ));
	if( *img != IPL_DONT_CARE )
	{
		IPL_BufFree( *img );
		*img = IPL_DONT_CARE;
	}
	dprintf((" -- " TOOL_PREFIX ": clear_img (%ld)\n", *img ));
}

//----------------------------------------------------------------------
void close_all( )
{
	dprintf((" ++ " TOOL_PREFIX ": close_all\n" ));
	free_blobs( );
	if( device != IPL_DONT_CARE )
	{
		DEV_Close( device );
		device = IPL_DONT_CARE;
	}

	if( img_buffer != NULL )
		IPL_ReleasePtr( grey_img, IPL_LOCK_READ, 0, 0, 0, img_buffer );

	clear_img( &grey_img );
	clear_img( &bin_img );

	dprintf((" -- " TOOL_PREFIX ": close_all\n" ));
}

//----------------------------------------------------------------------
long get_err_code( )
{
	dprintf((" -> " TOOL_PREFIX ": get_err_code\n" ));
	return err_code;
}

//----------------------------------------------------------------------
void init_device( DEV_HANDLE *devHandle )
{
	dprintf((" ++ " TOOL_PREFIX ": init_device\n" ));

	if( !DEV_Open( devHandle, "matrixvision.ddc" ))
		exit_demo( "open device failed", IPL_GetErrorCode( ));

	if( !DEV_Control( *devHandle, DEV_SINGLE, 0 ))
		exit_demo( "set control single failed", IPL_GetErrorCode( ));

#ifdef USE_DOUBLE_BUFFERING
	int dbuf = IPL_TRUE;
	if( !DEV_Control( device, DEV_SEQUENTIAL_MODE, &dbuf ))
		exit_demo( "set control sequential mode failed", IPL_GetErrorCode( ));
#endif	//USE_DOUBLE_BUFFERING

	if( !DEV_InquireCaps( device, DEV_TRIGGER, &trig_mode ))
		exit_demo( "get trigger mode failed", IPL_GetErrorCode( ));

	if( trig_mode != TRIG_SW && trig_mode != TRIG_AUTO )
	{
		int sw_trigmode = TRIG_SW;
		if( !DEV_ControlCaps( device, DEV_TRIGGER, (IPL_VOID*)&sw_trigmode ))
			exit_demo( "set SW trigger mode failed", IPL_GetErrorCode( ));
	}

	dprintf((" -- " TOOL_PREFIX ": init_device\n" ));
}

//----------------------------------------------------------------------
void get_img_val( DEV_HANDLE devHandle, IPL_BUFHANDLE *imgHandle, BYTE **buffer,
				unsigned long *iwidth, unsigned long *iheight, unsigned long *ibpp, unsigned long *ipitch )
{
	dprintf((" ++ " TOOL_PREFIX ": get_img_val\n" ));

	if( !DEV_Grab( devHandle, imgHandle ))
		exit_demo( "grab image failed", IPL_GetErrorCode( ));

	if( !IPL_BufInquire( *imgHandle, IPL_XSIZE, iwidth ))
		exit_demo( "inquier image width failed", IPL_GetErrorCode( ));
	if( !IPL_BufInquire( *imgHandle, IPL_YSIZE, iheight ))
		exit_demo( "inquier image height failed", IPL_GetErrorCode( ));
	if( !IPL_BufInquire( *imgHandle, IPL_NB_BANDS, ibpp ))
		exit_demo( "inquier image bit per pixel failed", IPL_GetErrorCode( ));

	if( !IPL_GetPtr( *imgHandle, IPL_LOCK_READ, 0, 0, 0, (IPL_LONG*)ipitch, buffer ))
		exit_demo( "inquier image height failed", IPL_GetErrorCode( ));

	if( trig_mode != TRIG_SW && trig_mode != TRIG_AUTO )
	{
		if( !DEV_ControlCaps( device, DEV_TRIGGER, (IPL_VOID*)&trig_mode ))
			exit_demo( "reset the trigger mode failed", IPL_GetErrorCode( ));
	}

	dprintf((" -- " TOOL_PREFIX ": get_img_val\n" ));
}

//----------------------------------------------------------------------
long get_threshold( )
{
	return threshold;
}

//----------------------------------------------------------------------
void set_threshold( unsigned long val )
{
	dprintf((" -> " TOOL_PREFIX ": set_threshold (val: %ld)\n", val ));
	threshold = val;
}

//----------------------------------------------------------------------
void use_auto_threshold( unsigned int flag )
{
	dprintf((" -> " TOOL_PREFIX ": use_auto_threshold (flag: %d)\n", flag ));
	th_flag = flag;
}

//----------------------------------------------------------------------
 unsigned int get_threshold_mode( )
{
	dprintf((" -> " TOOL_PREFIX ": get_threshold_mode (%d)\n", th_flag ));
	return th_flag;
}

//----------------------------------------------------------------------
int mk_bin_img( IPL_BUFHANDLE src, IPL_BUFHANDLE *dst, int polarity )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_bin_img (src: %ld, dst: %ld, pol: %d)\n", src, *dst, polarity ));

	int ret = 0;

	if( !th_flag )
		CHECK_RET(( ret = IPL_StatBestThreshold( src, &threshold )));

	dprintf((" == " TOOL_PREFIX ": mk_bin_img - treshold: %ld\n", threshold ));

	CHECK_RET(( ret = IPL_PtToPoint( src, threshold, dst, polarity ? IPL_GREATER_CST : IPL_LESS_CST )));
	CHECK_RET(( ret = IPL_MoReconstruct( *dst, dst, IPL_DONT_CARE, IPL_KILL_BORDER_BLOBS, IPL_BINARY )));

#ifdef EXPORT_IMGS
	printf("see: bin_file.bmp\n");
	CHECK_RET( IPL_FlExport( *dst, IPL_DONT_CARE, IPL_BMP, "bin_file.bmp" ));
#endif	// EXPORT_IMGS

	if( ret )	ret = threshold;

	dprintf((" -- " TOOL_PREFIX ": mk_bin_img (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void displ_binimg( unsigned int flag )
{
	dprintf((" ++ " TOOL_PREFIX ": displ_binimg(%d)\n", flag ));
	dbi_flag = flag;
	dprintf((" -- " TOOL_PREFIX ": displ_binimg\n" ));
}

//----------------------------------------------------------------------
void run_analyse( unsigned int flag )
{
	dprintf((" ++ " TOOL_PREFIX ": run_analyse(%d)\n", flag ));
	analyse_flag = flag;
	dprintf((" -- " TOOL_PREFIX ": run_analyse\n" ));
}

//----------------------------------------------------------------------
void single_snap( )
{
	dprintf((" ++ " TOOL_PREFIX ": single_snap\n"));

	if( !DEV_Grab( device, &grey_img ))
		exit_demo( "grab single image failed", IPL_GetErrorCode( ));

#ifdef EXPORT_IMGS
	printf("see: single_snap.bmp\n");
	CHECK_RET( IPL_FlExport( grey_img, IPL_DONT_CARE, IPL_BMP, "single_snap.bmp" ));
#endif	// EXPORT_IMGS

	dprintf((" -- " TOOL_PREFIX ": single_snap\n"));
}

//----------------------------------------------------------------------
void display_all( )
{
	dprintf((" ++ " TOOL_PREFIX ": display_all\n"));

	IPL_LONG ipitch = 0;
	BYTE *ibuff = NULL;

	if( dbi_flag )
	{
		if( bin_img != IPL_DONT_CARE )
		{
			if( IPL_GetPtr( bin_img, IPL_LOCK_READ, 0, 0, 0, &ipitch, &ibuff ))
			{
				cp_img( ibuff, shm_buffer, width, height, pitch, ipitch );
				IPL_ReleasePtr( bin_img, IPL_LOCK_READ, 0, 0, 0, ibuff );
			}
			else
				fprintf( stderr, "*** " TOOL_PREFIX ": display_all - no binary image found!\n");

		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": display_all - no binary image found!\n");
	}
	else
		cp_img( img_buffer, shm_buffer, width, height, pitch, pitch );

	displ_img( );

	dprintf((" -- " TOOL_PREFIX ": display_all\n"));
}

//----------------------------------------------------------------------
void get_child_img( BYTE *child_buf, unsigned long x, unsigned long y, unsigned long w, unsigned long h )
{
	dprintf((" ++ " TOOL_PREFIX ": get_child_img - x: %ld, y: %ld, w: %ld, h: %ld\n", x, y, w, h ));

	cp_chimg( img_buffer, child_buf, x, y, w, h, pitch, w );

	dprintf((" -- " TOOL_PREFIX ": get_child_img\n"));
}

//----------------------------------------------------------------------
int get_blob_pos( IPL_RECT *rect, unsigned int *ilist )
{
	dprintf((" ++ " TOOL_PREFIX ": get_blob_pos\n"));

	int ret = 0;
	double *results = NULL;

	if(( ret = get_bounding_box( grey_img, &bin_img, &results, ilist )) > 0 )
	{
		dprintf((" == " TOOL_PREFIX ": get_blob_pos - cnt: %d\n", ret ));
		memset( rect, 0, sizeof(IPL_RECT) * ret );
		for( int i = 0; i < ret; i++ )
		{
			dprintf((" == " TOOL_PREFIX ": get_blob_pos - results: %.0f %.0f %.0f %.0f\n",
									results[4*i], results[4*i+1], results[4*i+2], results[4*i+3] ));

			rect[i].left   = (long)results[4*i];
			rect[i].top    = (long)results[4*i+1];
			rect[i].right  = (long)results[4*i+2];
			rect[i].bottom = (long)results[4*i+3];

			dprintf((" == " TOOL_PREFIX ": get_blob_pos - rect: %ld %ld %ld %ld\n",
									rect[i].left, rect[i].top, rect[i].right, rect[i].bottom ));
		}
	}

	freeprintf( results, i );
		results = (double*)clear_buf( results );
	freeprintf( results, o );


	dprintf((" -- " TOOL_PREFIX ": get_blob_pos (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int calc_res( )
{
	int ret = analyse( grey_img, &bin_img );
	dprintf((" ++ " TOOL_PREFIX ": calc_res - grey_img: %ld, bin_img: %ld\n",  grey_img, bin_img ));

	if( ret == BL_STOPPED_AFTER_INDEX )
	{
		fprintf( stderr, "*** " TOOL_PREFIX ": calc_res - Stopped after %d objects.\n ", MAX_BLB_CNT );
		err_code = ret;
		ret = IPL_FALSE;
	}
	else
		err_code = 0;

	dprintf((" -- " TOOL_PREFIX ": calc_res (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int get_bin_img( )
{
	dprintf((" -> " TOOL_PREFIX ": get_bin_img - grey_img: %ld, bin_img: %ld\n",  grey_img, bin_img ));

#ifdef EXPORT_IMGS
	printf("see: get_bin_img_fom.bmp\n");
	CHECK_RET( IPL_FlExport( grey_img, IPL_DONT_CARE, IPL_BMP, "get_bin_img_fom.bmp" ));
#endif	// EXPORT_IMGS

	clear_img( &bin_img );
	return mk_bin_img( grey_img, &bin_img, polarity );
}

static double frame_ps = 0.0;
//----------------------------------------------------------------------
double get_frame_ps( )
{
	return frame_ps;
}

//----------------------------------------------------------------------
void snap_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": snap_img\n"));
	static unsigned int calc_ok = IPL_TRUE;
	static struct timeval s_tv, d_tv;

	while( run_flag )
	{
		pgEventPoll( );

		StartTimer( &s_tv );

		if( run_flag )
		{
			if( !DEV_Grab( device, &grey_img ))
				exit_demo( "grab image failed", IPL_GetErrorCode( ));
		}

		if( run_flag && ( analyse_flag || dbi_flag ))
			get_bin_img( );

		if( run_flag && analyse_flag )
			calc_ok = calc_res( );

		display_all( );

		if( run_flag && analyse_flag && calc_ok )
			get_res4classify( );

		update_status_line( calc_ok );

		ElapsedTime( &s_tv, &d_tv, 30 );

		double time_ms = (double)(d_tv.tv_sec * 1000 + d_tv.tv_usec/1000. );
		frame_ps = 1000.0/time_ms;

		dprintf((" == " TOOL_PREFIX ": snap_img - time_ms: %.2f, frame_ps: %.2f\n", time_ms, frame_ps ));
	}

	run_flag = 0;
	dprintf((" -- " TOOL_PREFIX ": snap_img\n"));
}

//----------------------------------------------------------------------
int get_polarity( )
{
	dprintf((" -> " TOOL_PREFIX ": get_polarity\n" ));
	return polarity;
}

//----------------------------------------------------------------------
void set_polarity( unsigned int val )
{
	dprintf((" ++ " TOOL_PREFIX ": set_polarity - val: %d\n", val ));
	polarity = val;
	dprintf((" -- " TOOL_PREFIX ": set_polarity\n" ));
}

//----------------------------------------------------------------------
void init_blb_globals2fl( )
{
	init_blb_globals( device, &grey_img );
}

//----------------------------------------------------------------------
int main( int argc, char *argv[] )
{
#ifdef MALLOC_TRACE
	mtrace();
#endif	// MALLOC_TRACE

	char msg[MAX_BUF_SIZE];
	memset( msg, 0, MAX_BUF_SIZE );
	sprintf( msg, "mvBlueLYNX BLOB AFS TOOL - " __DATE__ );

	check_init_file( );

	pghandle wApp = init_pg( argc, argv, msg );
	init_device( &device );
	get_img_val( device, &grey_img, &img_buffer, &width, &height, &bpp, &pitch );

	get_bin_img( );

	pghandle wBox = mk_buttons( wApp );
	pghandle wStatLine = mk_statline( wBox, PG_DERIVE_AFTER, 20 );
	mk_statLineLabel( wStatLine );

	get_img4displ( &shm_buffer, width, height );
	cp_img( img_buffer, shm_buffer, width, height, pitch, pitch );

	if( !init_blobs( ))
		exit_demo( "init the blob struct failed", 0 );
	else
		init_tmp_buf( );

	mk_disp( wStatLine /*wBox*/, PG_DERIVE_AFTER );

	init_gui( );
	pgUpdate( );

	int pflag = get_proc_flag( );
	if( pflag == 1 )
		evtLiveON( NULL );
	else if( pflag == 2 )
		evtClassify( NULL );

	pgEventLoop( );

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------



