//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#include <stdio.h>

#include "crtool.h"
#include "ftools.h"
#include "prop.h"
#include "gui.h"
#include "preproc.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \brief Checks if the point is in the rectangle.*/
int pt_in_rect( const IPL_RECT rc, const IPL_POINT pt )
{
	dprintf((" ++ " TOOL_PREFIX ": pt_in_rect\n"));
	int ret = RET_ERROR;

	if( (unsigned)rc.left < pt.xpos  && pt.xpos < (unsigned)rc.right &&
		(unsigned)rc.top < pt.ypos && pt.ypos < (unsigned)rc.bottom )
		ret = RET_SUCCESS;

	dprintf((" -- " TOOL_PREFIX ": pt_in_rect\n"));
	return ret;
}

//----------------------------------------------------------------------
/*! \brief Changes the \a struct \a timeval into a \a long [ \a msec ].*/
long ch_tv2ms( const struct timeval tv )
{
	dprintf((" ==> " TOOL_PREFIX ": ch_tv2ms (%ld, %ld)\n", tv.tv_sec, tv.tv_usec ));
	return tv.tv_sec * 1000 + tv.tv_usec/1000;
}

//----------------------------------------------------------------------
/*! \brief Creates a \a PGM image header.*/
char *pgm_header( char *buf, const int w, const int h )
{
	dprintf((" ==> " TOOL_PREFIX ": pgm_header (w/h: %d/%d)\n", w, h ));
	sprintf( buf, "P5\n\n%d\n%d\n255\n", w, h );
	return buf;
}

//----------------------------------------------------------------------
/*! \brief Creates a \a PGM image.*/
int mk_pgm( BYTE *sbuf, BYTE **dbuf, const unsigned int width,
					const unsigned int height, const unsigned int pitch, const unsigned int bpp )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_pgm\n"));

	int ret = -1;
	char pgmh[32];
	int hlen;
	unsigned int image_file_size = width * height * bpp;
	unsigned int fsize;

	if( sbuf != NULL )
	{
		pgm_header( pgmh, width, height );
		hlen = strlen( pgmh );

		fsize = image_file_size + hlen;

		if( *dbuf != NULL )
		{
			free( *dbuf );
			*dbuf = NULL;
		}

		if(( *dbuf = (BYTE *) malloc( fsize * sizeof(BYTE))) != NULL )
		{
			memcpy( *dbuf + 0, pgmh, hlen );
			for( unsigned int y = 1; y < height; y++ )
				memcpy( *dbuf + hlen + y * width, sbuf + y * pitch, width );

			ret = fsize;
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": mk_pgm - out of memory!!\n");
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": mk_pgm - wrong source buffer!!\n");

	dprintf((" -- " TOOL_PREFIX ": mk_pgm (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
/*! \brief Saves the settings into the \a init file.*/
int save_ini_file( FILE *f )
{
	dprintf((" ++ "  TOOL_PREFIX ": save_ini_file\n"));
	static time_t t;
	int ret = RET_SUCCESS;
	int tmp_cnt = 0, ini_cnt = prop_get_prop_list_size( );
	dprintf((" == "  TOOL_PREFIX ": save_ini_file - ini cnt: %d\n", ini_cnt ));
	CR_PROP tmp_prop;

	if( f != NULL )
	{
		time( &t );
		fprintf( f, "##-------------------------------##\n");
		fprintf( f, "## " TOOL_PREFIX " ini file (mvBL)        ##\n");
		fprintf( f, "## %s", ctime( &t ));
		fprintf( f, "##-------------------------------##\n\n");

		fprintf( f, "[FILE]\n");
		tmp_prop = prop_get_prop( CR_USE_INI_FILE );
		fprintf( f, "%s=%d\n", tmp_prop.name, tmp_prop.default_val );

		fprintf( f, "\n##---------------------##\n");
		fprintf( f, "## Decoder settings ##\n");
		fprintf( f, "##---------------------##\n");
		fprintf( f, "\n[DECODER]\n");
		for( int i = CR_USE_INI_FILE + 1; i < ini_cnt - 1; i++ )
		{
			tmp_prop = prop_get_prop((CR_PROPS_ID)i );
			if( strncmp( tmp_prop.name, "IMG_CLEANING_MODE", strlen ( "IMG_CLEANING_MODE" )) == 0 )
			{
				tmp_cnt = i;
				break;
			}
			else
				fprintf( f, "%s=%d\n", tmp_prop.name, tmp_prop.default_val );
		}

		fprintf( f, "\n##------------------------##\n");
		fprintf( f, "## Preprocessing settings ##\n");
		fprintf( f, "##------------------------##\n");
		fprintf( f, "\n[IMAGECLEANING]\n");

		for( int i = tmp_cnt; i < tmp_cnt + 2; i++ )
		{
			tmp_prop = prop_get_prop((CR_PROPS_ID)i );
			fprintf( f, "%s=%d\n", tmp_prop.name, tmp_prop.default_val );
		}

		fprintf( f, "\n[CELLIMPROVEMENT]\n");

		tmp_cnt += 2;
		for( int i = tmp_cnt; i < tmp_cnt + 2; i++ )
		{
			tmp_prop = prop_get_prop((CR_PROPS_ID)i );
			fprintf( f, "%s=%d\n", tmp_prop.name, tmp_prop.default_val );
		}

		fprintf( f, "\n[SYMBOLELOCATION]\n");

		tmp_cnt += 2;
		for( int i = tmp_cnt; i < ini_cnt; i++ )
		{
			tmp_prop = prop_get_prop((CR_PROPS_ID)i );
			fprintf( f, "%s=%d\n", tmp_prop.name, tmp_prop.default_val );
		}
	}
	else
		ret = RET_ERROR;

	dprintf((" -- "  TOOL_PREFIX ": save_ini_file (%d)\n", ret ));

	return ret;
}

//----------------------------------------------------------------------
/*! \brief Reads the settings from the \a init file.*/
int read_ini_file( )
{
	dprintf((" ++ "  TOOL_PREFIX ": read_ini_file\n" ));

	int ret = RET_ERROR;
	int tmp_val, nui_falg = 1;
	char *fname = INI_FILE_NAME;
	FILE *f = NULL;
	char buf[MAX_CHAR_SIZE];
	memset( buf, 0, MAX_CHAR_SIZE );

	CR_PROP tmp_prop, tmp_prop_ui = prop_get_prop( CR_USE_INI_FILE );
	int ini_cnt = prop_get_prop_list_size( );

	dprintf((" == "  TOOL_PREFIX ": read_ini_file - ini_cnt: %d\n", ini_cnt ));

	if(( f = fopen( fname, "r" )) != NULL )
	{
		rewind( f );
		for(  ; fgets( buf, MAX_CHAR_SIZE, f ) != NULL;  )
		{
			dprintf((" == " TOOL_PREFIX ": read_ini_file: %s\n", buf ));

			for( int i = CR_USE_INI_FILE; i < ini_cnt; i++ )
			{
				tmp_prop = prop_get_prop((CR_PROPS_ID)i );
				if( strncmp( buf, tmp_prop.name, strlen ( tmp_prop.name )) == 0 )
				{
					tmp_val = atoi( buf + strlen ( tmp_prop.name ) + 1 );
					prop_set_prop( (CR_PROPS_ID)i, prop_check_val( tmp_prop, tmp_val ));

					if( strncmp( tmp_prop.name, tmp_prop_ui.name, strlen( tmp_prop_ui.name )) == 0 )
					{
						if( tmp_val == 0 )
						{
							nui_falg = 0;
							break;
						}
					}
				}
			}

			if(!nui_falg)
				break;
		}

		fclose( f );
	}

#ifdef DEBUG
	for( int i = 1; i < ini_cnt; i++ )
	{
		tmp_prop = prop_get_prop((CR_PROPS_ID)i );
		printf(" == "  TOOL_PREFIX ": read_ini_file - %s=%d\n", tmp_prop.name, tmp_prop.default_val );
	}
#endif	// DEBUG

	dprintf((" -- "  TOOL_PREFIX ": read_ini_file (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
/*! \brief Check if the piont match.*/
int check_pt( const IPL_POINT pt, const unsigned int x, const unsigned int y )
{
	dprintf((" ++ " TOOL_PREFIX ": check_pt\n" ));

	int ret = RET_ERROR;
	unsigned int pos_offset = 5;

	if( x >= pt.xpos - pos_offset && x <= pt.xpos + pos_offset &&
		y >= pt.ypos - pos_offset && y <= pt.ypos + pos_offset )
		ret = RET_SUCCESS;

	dprintf((" -- " TOOL_PREFIX ": check_pt(%d)\n", ret ));
	return ret;
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------
