//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#include "crtool.h"
#include "ftools.h"
#include "gui.h"

//----------------------------------------------------------------------
static SPIN_EDIT se = { 0,{ 0, 0, NULL, -1, MVCOLOR_BLACK },0,0,0 };
char msg[MSG_SIZE];
static long se_cnt = 0;

//----------------------------------------------------------------------
/*! \brief Delete the spin edit.*/
void delete_se(  )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	widget_delete( &se.wBox );
	widget_delete( &se.si.wBox );
	widget_delete( &se.si.wText );
	widget_delete( &se.wBtnUp );
	widget_delete( &se.wBtnDown );
	widget_delete( &se.wLabel );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//----------------------------------------------------------------------
/*! \brief Set the spin edit value and display it.*/
void set_se_val( const int val )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	se_cnt = val;
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%ld", se_cnt );
	se.si.msg = msg;
	repl_txt_si( se.si );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//----------------------------------------------------------------------
/*! \brief Event button up.*/
int evtBtnUp( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	++se_cnt;
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%ld", se_cnt );
	se.si.msg = msg;
	repl_txt_si( se.si );
	set_val_from_se( se_cnt );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Event button down.*/
int evtBtnDown( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	--se_cnt;

	if( se_cnt < 0 )
		se_cnt = 0;

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%ld", se_cnt );
	se.si.msg = msg;
	repl_txt_si( se.si );
	set_val_from_se( se_cnt );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//----------------------------------------------------------------------
/*! \brief Event text edit (not implemented yet).*/
int evtTextEdit( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "%ld", se_cnt );
	se.si.msg = msg;
	repl_txt_si( se.si );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return 0;
}

//----------------------------------------------------------------------
/*! \brief Make the spin edit.*/
static void mk_spinedit( const pghandle parent, const char *text, pghandle *wSEText, pghandle *wBtnUp, pghandle *wBtnDown,
						pgevthandler eBtnUp, pgevthandler eBtnDown, pgevthandler eTextEdit, const int tbox_w )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int btn_h = 9;

	*wSEText = pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_LEFT,
				PG_WP_SIZE, tbox_w,
				PG_WP_MARGIN, 0,
				PG_WP_TEXT, pgNewString( text ),
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, eTextEdit, NULL );

	*wBtnUp = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_AFTER, *wSEText );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_MARGIN, 0,
				PG_WP_SIZE, btn_h,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, eBtnUp, NULL );

	*wBtnDown = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_AFTER, *wBtnUp );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_BOTTOM,
				PG_WP_MARGIN, 0,
				PG_WP_SIZE, btn_h,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, eBtnDown, NULL );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//----------------------------------------------------------------------
/*! \brief Make a spin edit.*/
void mk_spin_edit( const pghandle parent, const long value, const char *label )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int lab_len = 0, size = 0;
	memset( msg, 0, MSG_SIZE );
	int len = get_x_size( "88888" );

	if( label != NULL )
	{
		lab_len = get_x_size( label );
		size = len + lab_len + 30;
	}
	else
		size = len + 20;

	dprintf((" ++ " TOOL_PREFIX ": %s - val: %ld, label: >>%s<<, len: %d, lab_len: %d, size: %d\n", __FUNCTION__,
					value, label, len, lab_len, size ));

	se_cnt = value;

	se.wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_AFTER, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_LEFT,
				PG_WP_MARGIN, 0,
				PG_WP_SIZE, size,
				0 );

	se.si.wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_INSIDE, se.wBox );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_LEFT,
				PG_WP_SIZE, size,
				0 );

	sprintf( msg, "%4ld", value );
	mk_spinedit( se.si.wBox, msg, &se.si.wText, &se.wBtnUp, &se.wBtnDown, &evtBtnUp, &evtBtnDown, &evtTextEdit, len );

	if( label != NULL )
	{
		se.wLabel = pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, se.wBox );
		pgSetWidget( PGDEFAULT,
					PG_WP_SIDE, PG_S_RIGHT,
					PG_WP_SIZE, lab_len + 10,
					PG_WP_TEXT, pgNewString( label ),
					0 );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
