//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>displtest<<
// Mit Sep  8 12:52:57 CEST 2004
//----------------------------------------------------------------------
/*! \file displtest.cpp
 *  \brief MATRIX VISION GmbH - sample for PPC mvIMPACT
 *
 * $Id: displtest.cpp,v 1.1 2004/09/09 14:21:09 td Exp $
 *
 *
 * This sample shows how to combine the mvIMPACT Display library (vl_dispplay)
 * with the PicoGUI API functions to create buttons.
 *
 */
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>

#include <assert.h>
#include <samples.h>

#include <pgtools.h>

//-------------------------------------------------------------------
//-------------------------------------------------------------------
#define TOOL_PREFIX							"displtest"

//-------------------------------------------------------------------
//-------------------------------------------------------------------
static IPL_BUFHANDLE imageHandle = IPL_DONT_CARE;			/* mvIMPACT image handle */
static DEV_HANDLE deviceHandle   = IPL_DONT_CARE;			/* mvIMPACT device handle */
static IPL_HWND hwnd = 0;									/* mvIMPACT windows handle */
static pghandle wPanel = 0, wLive = 0, wStop = 0;			/* PicoGUI widget handles */
static bool run_flag = false;								/* flag for live snap */
static CPGTools *pPGT = NULL;								/* class for PicoGUI objects */

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/* grab a image and display it */
void snap_loop( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	while( run_flag )
	{
		/* check if there are events */
		pgEventPoll( );

		/* grab and dispay the image */
		if( run_flag )
			CHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));
		if( run_flag )
			CHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/* start the live snap */
int evtLiveOn( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	run_flag = true;
	snap_loop( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/* stop the live snap */
int evtLiveOff( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	run_flag = false;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/* event funtion for close the sample */
int evtClose( struct pgEvent *evt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	evtLiveOff( evt );

	/* leave the event loop */
	pgExitEventLoop( );

	/* free the class resources */
	if( pPGT )	delete pPGT;

	/* close the opened device */
    DEV_Close( deviceHandle );

	/* free the image buffer */
	IPL_BufFree( imageHandle );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//----------------------------------------------------------------------
int main( int argc, char *argv[] )
{
	/* open a device */
	CHECKSDKCALL( DEV_Open( &deviceHandle, "matrixvision.ddc" ));
    CHECKSDKCALL( DEV_Control( deviceHandle, DEV_SINGLE, 0 ));

	/* grab a image */
	CHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));

	/* init picogui and register application */
	pgInit( argc, argv );
	wPanel = pgRegisterApp( PG_APP_NORMAL, TOOL_PREFIX, 0 );
	pgBind( PGDEFAULT, PG_WE_CLOSE, &evtClose, NULL );

	if(( pPGT = new CPGTools( )) != NULL )
	{
		/* make 2 buttons */
		pghandle wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_INSIDE, wPanel );
		pgSetWidget( PGDEFAULT,
					PG_WP_SIZE, 30,
					0 );

		wLive = pPGT->PGT_mkButton(      wBox, PG_DERIVE_INSIDE, "Live", &evtLiveOn,  PGT_COLOR_BLACK, 10 );
		wStop = pPGT->PGT_mkButton( PGDEFAULT, PG_DERIVE_AFTER, "Stop",  &evtLiveOff, PGT_COLOR_BLACK, 10 );

		/* make a scroll box for the dispay window */
		pghandle wSBox = pgNewWidget( PG_WIDGET_SCROLLBOX, PG_DERIVE_AFTER, wBox );
		/* open a dispay window in the scroll box */
		CHECKSDKCALL( DISP_OpenWindow( imageHandle, &hwnd, wSBox, NULL ));
		/* update the display */
		CHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));

		/* start the event loop */
		pgEventLoop( );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": out of memory!!\n" );

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
