//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>gmmtool<<
// Tue May 20 14:35:47 CEST 2003
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include <picogui.h>
#include <system.h>
#include <pgutils.h>

#include "gmmtool.h"
#include "pgapi.h"

//-------------------------------------------------------------------
//-------------------------------------------------------------------
void widget_delete( pghandle *widget )
{
	if( *widget != 0 )
	{
		pgDelete( *widget );
		*widget = 0;
	}
}

//-------------------------------------------------------------------
void cp_chimg( unsigned char *src, unsigned char *dst, int sx, int sy, int width, int height, int spitch, int dpitch )
{
	dprintf((" ++ " TOOL_PREFIX ": cp_chimg - sx: %d, sy: %d, width: %d, height: %d, spitch: %d, dpitch: %d\n",
		sx, sy, width, height, spitch, dpitch ));

	if( src != NULL )
	{
		int x, y;
		for( y = 0; y < height; y++ )
		{
			for( x = 0; x < width; x++ )
				dst[x + y * dpitch] = src[ x + sx + ( y + sy ) * spitch];
		}
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": cp_chimg - no source image found!!\n");

	dprintf((" -- " TOOL_PREFIX ": cp_chimg\n"));
}

//-------------------------------------------------------------------
void cp_img( unsigned char *src, unsigned char *dst, int width, int height, int spitch, int dpitch )
{
	dprintf((" ++ " TOOL_PREFIX ": cp_img - width: %d, height: %d, spitch: %d, dpitch: %d\n",
				width, height, spitch, dpitch ));

	if( src != NULL )
	{
		int y;
		for( y = 0; y < height; y++ )
			memcpy( dst + y * dpitch, src + y * spitch, width );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": cp_chimg - no source image found!!\n");

	dprintf((" -- " TOOL_PREFIX ": cp_img\n" ));
}

//-------------------------------------------------------------------
void repl_txt_si( STATUS_INFO si )
{
	dprintf((" ++ " TOOL_PREFIX ": repl_txt - >>%s<<\n", si.msg ));

	pgReplaceText( si.wText, si.msg );
	pgSubUpdate( si.wBox );
	pgSubUpdate( si.wText );

	dprintf((" -- " TOOL_PREFIX ": repl_txt\n"));
}

//-------------------------------------------------------------------
void repl_txt( pghandle widget, char *text )
{
	dprintf((" ++ " TOOL_PREFIX ": repl_txt - >>%s<<\n", text ));

	pgReplaceText( widget, text );
	pgSubUpdate( widget );

	dprintf((" -- " TOOL_PREFIX ": repl_txt\n"));
}

//-------------------------------------------------------------------
void draw_aoi( pgcontext gc, int x, int y, int w, int h, pgcolor color )
{
	dprintf((" ++ " TOOL_PREFIX ": draw_aoi - x: %d, y: %d, w: %d, h: %d, color: 0x%x\n",
								x, y, w, h, color ));

	pgSetColor( gc, color );
	pgFrame( gc, x, y, w, h );
	pgContextUpdate( gc );

	dprintf((" -- " TOOL_PREFIX ": draw_aoi\n"));
}

//-------------------------------------------------------------------
void draw_txt( pgcontext gc, int x, int y, char *text, pgcolor color )
{
	dprintf((" ++ " TOOL_PREFIX ": draw_txt - x: %d, y: %d, text: >>%s<<, color: 0x%x\n",
						x, y, text, color ));

	pgSetColor( gc, color );
	pgText( gc, x, y, pgNewString( text ));
	pgContextUpdate( gc );

	dprintf((" -- " TOOL_PREFIX ": draw_txt\n"));
}

//----------------------------------------------------------------------
void draw_line( pgcontext gc, int x1, int y1, int x2, int y2, pgcolor color )
{
	dprintf(( " ++ " TOOL_PREFIX ": draw_line - x/y1: %d/%d, x/y2: %d/%d\n", x1, y1, x2, y2 ));

	pgSetColor( gc, color );
	pgLine( gc, x1, y1, x2, y2 );
	pgContextUpdate( gc );

	dprintf(( " -- " TOOL_PREFIX ": draw_line\n" ));
}

//----------------------------------------------------------------------
void draw_point( pgcontext gc, int x, int y, pgcolor color )
{
	dprintf(( " ++ " TOOL_PREFIX ": draw_point - x/y: %d/%d\n", x, y ));

	pgSetColor( gc, color );
	pgPixel( gc, x, y );
	//pgContextUpdate( gc );

	dprintf(( " -- " TOOL_PREFIX ": draw_point\n" ));
}

//----------------------------------------------------------------------
void draw_circle( pgcontext gc, int x_cent, int y_cent, int r, pgcolor color )
{
	dprintf(( " ++ " TOOL_PREFIX ": draw_circle - x/y: %d/%d, r: %d\n", x_cent, y_cent, r ));
	int x, y, w, h;

	x = x_cent - r;
	y = y_cent - r;
	w = 2 * r;
	h = 2 * r;

	dprintf(( " == " TOOL_PREFIX ": draw_circle - x/y: %d/%d, w/h: %d/%d\n", x, y, w, h ));

	pgSetColor( gc, color );
	pgEllipse( gc, x, y, w, h );
	pgContextUpdate( gc );

	dprintf(( " -- " TOOL_PREFIX ": draw_circle\n"));
}

//-------------------------------------------------------------------
void exit_demo( const char *s, long err_code )
{
	fprintf( stderr, "*** " TOOL_PREFIX ": %s - err: %ld!!\n", s, err_code );
	evtClose( NULL );
	exit( err_code );
}

//-------------------------------------------------------------------
pghandle mk_chb( pghandle parent, char *txt, pgevthandler chbox_cb )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_chb\n"));

	pghandle wChBox = pgNewWidget( PG_WIDGET_CHECKBOX, PG_DERIVE_INSIDE, parent );
	pgSetWidget( wChBox,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_EXTDEVENTS, PG_EXEV_TOGGLE,
				0 );
	pgBind( wChBox, PG_WE_ACTIVATE, chbox_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_chb\n"));

	return wChBox;
}

//-------------------------------------------------------------------
pghandle mk_btn( pghandle parent, char *txt, int pos, int size, pgevthandler btn_cb )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_btn\n"));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_SIZE, size,
				PG_WP_FONT, pgNewFont( NULL, 6, 0 ),
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, btn_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_btn\n"));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_ctrl_btn( pghandle parent, char *txt, int pos, pgevthandler btn_cb, pghandle color )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_ctrl_btn\n"));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_COLOR, color,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, btn_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_ctrl_btn\n"));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_label( pghandle parent, char *txt, int pos )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_label\n"));

	pghandle wLabel = pgNewWidget( PG_WIDGET_LABEL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_ALIGN, PG_A_RIGHT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_SIZE, 8,
				0 );

	dprintf((" -- " TOOL_PREFIX ": mk_label\n"));

	return wLabel;
}

//-------------------------------------------------------------------
pghandle mk_scroller( pghandle parent, int pos, int max_s, int def_s, pgevthandler scroll_cb )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_scroller - max_s: %d, def_s: %d\n", max_s, def_s ));

	pghandle wScroller = pgNewWidget( PG_WIDGET_SCROLL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, max_s,
				PG_WP_VALUE, def_s,
				PG_WP_SCROLL_X, 0,
				PG_WP_SCROLL_Y, def_s,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, scroll_cb, NULL );

	dprintf((" -- " TOOL_PREFIX ": mk_scroller\n" ));

	return wScroller;
}

//-------------------------------------------------------------------
pghandle mk_wBox( pghandle parent, int pat_pos, int side, int size )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_wBox\n" ));

	pghandle wigdet = pgNewWidget( PG_WIDGET_BOX, pat_pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, side,
				PG_WP_SIZE, size,
				0 );

	dprintf((" -- " TOOL_PREFIX ": mk_wBox\n" ));

	return wigdet;
}

//-------------------------------------------------------------------
pghandle mk_scroller_label( pghandle parent, int pos, char *txt )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_scroller_label\n" ));

	int x,y;
	pghandle font = pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT );

	get_str_size( &x, &y, font, txt );

	dprintf((" == " TOOL_PREFIX ": mk_scroller_label - txt: >>%s<< (x/y: %d/%d)\n", txt, x, y ));

	pghandle wScLab = mk_wBox( parent, pos, PG_S_LEFT, x + 5 );

	pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, PGDEFAULT );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, font,
				PG_WP_SIZE, x,
				PG_WP_SIDE, PG_S_LEFT,
				PG_WP_MARGIN, 0,
				0 );

	dprintf((" -- " TOOL_PREFIX ": mk_scroller_label\n" ));

	return wScLab;
}

//-------------------------------------------------------------------
int set_min_max( char *name, int val, int min_val, int max_val )
{
	int new_val = val;
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	pgEnterContext( );
	pghandle wToolbar, wScrlLab, wScroller, wOK, wCancel;
	struct pgEvent *evt;

	sprintf( msg, "Set segment: %s", name );
	pgDialogBox( msg );
	wToolbar = pgNewWidget( PG_WIDGET_TOOLBAR, 0, 0 );
	pgSetWidget( PGDEFAULT,
			PG_WP_SIDE, PG_S_LEFT,
			0 );

	wOK = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_INSIDE, wToolbar );
	pgSetWidget( PGDEFAULT,
			PG_WP_TEXT, pgNewString( "OK" ),
			0 );
	wCancel = pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_INSIDE, wToolbar );
	pgSetWidget( PGDEFAULT,
			PG_WP_TEXT, pgNewString( "Cancel" ),
			0 );

	wScroller = pgNewWidget( PG_WIDGET_SCROLL, PG_DERIVE_AFTER, wToolbar );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, max_val,
				PG_WP_VALUE, val,
				PG_WP_SCROLL_X, 0,
				PG_WP_SCROLL_Y, val,
				0 );

	pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_AFTER, PGDEFAULT );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( name ),
				PG_WP_FONT, pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT ),
				PG_WP_SIDE, PG_S_LEFT,
				0 );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "old value: %d", val );

	pgNewWidget( PG_WIDGET_LABEL, 0,0 );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( msg ),
				PG_WP_FONT, pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT ),
				PG_WP_SIDE, PG_S_LEFT,
				0 );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "new value: %d", val );

	wScrlLab = pgNewWidget( PG_WIDGET_LABEL, 0,0 );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( msg ),
				PG_WP_FONT, pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT ),
				PG_WP_SIDE, PG_S_RIGHT,
				PG_WP_ALIGN, PG_A_RIGHT,
				0 );

	for (;;)
	{
		evt = pgGetEvent();

		if( evt->from == wOK )
		{
			if( new_val < min_val )
				new_val = min_val;

			pgReplaceTextFmt( wScrlLab, "new value: %d", new_val );
			pgSubUpdate( wScrlLab );

			dprintf((" == " TOOL_PREFIX ": set_min_max - new_val: %d\n", new_val ));
			pgLeaveContext( );
			return new_val;
		}
		else if( evt->from == wCancel )
		{
			pgReplaceTextFmt( wScrlLab, "new value: %d", val );
			pgSubUpdate( wScrlLab );

			dprintf((" == " TOOL_PREFIX ": set_min_max - new_val: %d\n", val ));
			pgLeaveContext( );
			return val;
		}
		else if( evt->from == wScroller )
		{
			dprintf((" == " TOOL_PREFIX ": set_min_max - wScroller: evt->e.param: %d\n", evt->e.param ));

			if( evt->e.param != 0 )
				new_val = evt->e.param;
			else
				new_val = min_val;

			pgReplaceTextFmt( wScrlLab, "new value: %d", new_val );
			pgSubUpdate( wScrlLab );
		}
	}

	pgLeaveContext( );

	return -1;
}

//-------------------------------------------------------------------
pghandle mk_statline( pghandle parent, int pos, int size )
{
	pghandle wStatLine = pgNewWidget( PG_WIDGET_TOOLBAR, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, size,
				0 );


	return wStatLine;
}

//-------------------------------------------------------------------
void get_str_size( int *x, int *y, pghandle font, char *str )
{
	pgEnterContext( );
	pgSizeText( x, y, font, pgNewString( str ));
	pgLeaveContext( );
}

//-------------------------------------------------------------------
void mk_stll_si( pghandle parent, int pos, STATUS_INFO *si )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_stll_si\n" ));
	int x,y, offs = 5;

	x = si->msg_len;

	if( x < 0 )
	{
		pghandle font = pgNewFont( NULL, 8, 0 );
		get_str_size( &x, &y, font, si->msg );
	}

	si->wBox = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
	si->wText = mk_label( PGDEFAULT, si->msg, PG_DERIVE_INSIDE );
	dprintf((" -- " TOOL_PREFIX ": mk_stll_si\n" ));
}

//-------------------------------------------------------------------
pghandle mk_stll( pghandle parent, int pos, pghandle *wText, char *str, int size )
{
	int x,y, offs = 5;

	x = size;

	if( size < 0 )
	{
		pghandle font = pgNewFont( NULL, 8, 0 );
		get_str_size( &x, &y, font, str );
	}

	pghandle wB = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
	*wText = mk_label( PGDEFAULT, str, PG_DERIVE_INSIDE );

	return wB;
}

//-------------------------------------------------------------------
void set_w_color( pghandle widget, pghandle color )
{
	pgSetWidget( widget, PG_WP_COLOR, color, 0 );
	pgSubUpdate( widget );

}

//-------------------------------------------------------------------
int check_if_file_exist( char *fname )
{
	FILE *f;
	char s[MSG_SIZE];
	memset( s, 0, MSG_SIZE );

	dprintf(( " ++ " TOOL_PREFIX ": check_if_file_exist\n" ));

	if(( f = fopen( fname, "r" )) == NULL )
	{
		sprintf( s, "No '%s' found!!", fname );
		pgMessageDialog( KB_ERROR, s, PG_MSGBTN_OK | PG_MSGICON_ERROR );
		return -1;
	}

	fclose( f );

	dprintf(( " -- " TOOL_PREFIX ": check_if_file_exist\n" ));

	return 0;
}

//-------------------------------------------------------------------
void get_s_size( char *s, int *x, int *y )
{
	dprintf(( " ++ " TOOL_PREFIX ": get_s_size - s: >>%s<<, x: %d, y: %d\n",s, *x, *y ));
	pghandle font = pgNewFont( NULL, 8, 0 );
	get_str_size( x, y, font, s );
	dprintf(( " -- " TOOL_PREFIX ": get_s_size - x: %d, y: %d\n", *x, *y ));
}

//-------------------------------------------------------------------
int get_x_size( char *s )
{
	dprintf(( " ++ " TOOL_PREFIX ": get_x_size - s: >>%s<<\n", s ));

	int x = 0, y = 0;
	get_s_size( s, &x, &y );

	dprintf(( " -- " TOOL_PREFIX ": get_x_size - x: %d\n", y ));

	return x;
}

//-------------------------------------------------------------------
int get_y_size( char *s )
{
	dprintf(( " ++ " TOOL_PREFIX ": get_x_size - s: >>%s<<\n", s ));

	int x = 0, y = 0;
	get_s_size( s, &x, &y );

	dprintf(( " -- " TOOL_PREFIX ": get_x_size - y: %d\n", y ));
	return y;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
static int bcMenuFromArray( pghandle wMainCanvas, pghandle *items, int numitems )
{
	dprintf((" ++ " TOOL_PREFIX ": bcMenuFromArray (%d)\n", numitems ));
	int i, ret;
	pghandle returnHandle;
	returnHandle = pgNewPopupAt( PG_POPUP_ATEVENT, PG_POPUP_ATEVENT, 0, 0 );

	for( i = 0; i < numitems; i++ )
	{
		pgNewWidget( PG_WIDGET_MENUITEM, 0, 0 );
		pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, items[i],
				0 );

		pgSetPayload( PGDEFAULT, i+1 );
	}

	pghandle evtFrom = pgGetEvent()->from;

	while( evtFrom == wMainCanvas )
	{
		dprintf((" ==** " TOOL_PREFIX ": bcMenuFromArray - event from canvas: %ld\n", evtFrom ));
		evtFrom = pgGetEvent()->from;
	}

	dprintf((" == " TOOL_PREFIX ": bcMenuFromArray - event from: %ld\n", evtFrom ));

	/* Return event */
	ret = pgGetPayload( evtFrom );

	dprintf((" -- " TOOL_PREFIX ": bcMenuFromArray (%d)\n", ret ));

	return ret;
}

//-------------------------------------------------------------------
int mk_popup_menu( pghandle wMainCanvas, char *menu_entrys, unsigned int entry_cnt )
{
	dprintf((" ++ " TOOL_PREFIX ": mk_popup_menu - menu entrys: >%s<, cnt: %d\n",
						menu_entrys, entry_cnt ));

	char *p, menu[MSG_SIZE];
	int ret;

	//pgSetIdle( 0, NULL );

	if( !menu_entrys || !*menu_entrys || entry_cnt <= 0 ) return 0;

	pghandle *items = (pghandle*) malloc( entry_cnt * sizeof( pghandle ));

	for( unsigned int i = 0; i < entry_cnt; i++ )
	{
		if( !( p = strchr( menu_entrys,'|' )))
			p = menu_entrys + strlen( menu_entrys );

		memset( menu, 0, MSG_SIZE );
		strncpy( menu, menu_entrys, strlen( menu_entrys ) - strlen( p ));

		pgFlushRequests( );
		menu_entrys = p + 1;
		items[i] = pgNewString( menu );
	}

	ret = bcMenuFromArray( wMainCanvas, items, entry_cnt );

	if( items )
		free( items );

	dprintf((" -- " TOOL_PREFIX ": mk_popup_menu (%d)\n", ret ));

	return ret;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
