//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
/*! \file mk_msg.cpp
 *  \brief MATRIX VISION GmbH
 *
 * $Id: mk_msg.cpp,v 1.1 2004/10/20 08:18:19 td Exp $
 *
 *
 */

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <syslog.h>

#include "mk_msg.h"

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
char db_args[200];

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
int make_message( int flags, const char *fmt, ... )
{
	/* Guess we need no more than 100 bytes. */
	int n, size = 100;
	char *p;
	va_list ap;

	if( flags )
	{
		if(( p =(char*) malloc( size )) == NULL )
			return -1;

#ifdef DEBUG
		printf(" make_message - alloc %p, %d\n", p, size );
#endif	// DEBUG
		while( 1 )
		{
			/* Try to print in the allocated space. */
			va_start( ap, fmt );
			n = vsnprintf( p, size, fmt, ap );
			va_end( ap );

			/* If that worked, return the string. */
			if( n > -1 && n < size )
			{
				printf("%s", p );
				free( p );
#ifdef DEBUG
				printf(" make_message - free %p\n", p );
#endif	// DEBUG
				return 0;
			}

			/* Else try again with more space. */
			if( n > -1 )			/* glibc 2.1 */
				size = n+1;			/* precisely what is needed */
			else					/* glibc 2.0 */
				size *= 2;			/* twice the old size */
			if(( p = (char*)realloc( p, size )) == NULL )
			{
				if( p )	free( p );
#ifdef DEBUG
				printf(" make_message - free %p\n", p );
#endif	// DEBUG
				return -1;
			}
#ifdef DEBUG
			else
				printf(" make_message - realloc %p, %d\n", p, size );
#endif	// DEBUG
		}
	}

	return 0;
}

//-----------------------------------------------------------------------------------
int make_dbg_msg( const char *fmt, ... )
{
	/* Guess we need no more than 100 bytes. */
	int n, size = 100;
	char *p;
	va_list ap;

	if( fmt == NULL )
	{
		printf( " %s\n", db_args );
		return 0;
	}

	if(( p =(char*) malloc( size )) == NULL )
		return -1;

	while( 1 )
	{
		/* Try to print in the allocated space. */
		va_start( ap, fmt );
		n = vsnprintf( p, size, fmt, ap );
		va_end( ap );

		/* If that worked, return the string. */
		if( n > -1 && n < size )
		{
			printf( " %s - %s", db_args, p );
			free( p );
			return 0;
		}

		/* Else try again with more space. */
		if( n > -1 )			/* glibc 2.1 */
			size = n+1;			/* precisely what is needed */
		else					/* glibc 2.0 */
			size *= 2;			/* twice the old size */
		if(( p = (char*)realloc( p, size )) == NULL )
		{
			if( p )	free( p );
			return -1;
		}
	}

	return 0;
}

//-----------------------------------------------------------------------------------
int make_error_msg( const char *fmt, ... )
{
	/* Guess we need no more than 100 bytes. */
	int n, size = 200;
	char *p;
	va_list ap;

	if(( p =(char*) malloc( size )) == NULL )
		return -1;

	while( 1 )
	{
		/* Try to print in the allocated space. */
		va_start( ap, fmt );
		n = vsnprintf( p, size, fmt, ap );
		va_end( ap );

		/* If that worked, return the string. */
		if( n > -1 && n < size )
		{
			fprintf( stderr, "*** %s - %s", db_args, p );
			free( p );
			return 0;
		}

		/* Else try again with more space. */
		if( n > -1 )			/* glibc 2.1 */
			size = n+1;			/* precisely what is needed */
		else					/* glibc 2.0 */
			size *= 2;			/* twice the old size */
		if(( p = (char*)realloc( p, size )) == NULL )
		{
			if( p )	free( p );
			return -1;
		}
	}

	return 0;
}

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
