//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \file pgtools.cpp
 *  \brief MATRIX VISION GmbH
 *
 * $Id: pgtools.cpp,v 1.1 2004/10/20 08:18:19 td Exp $
 *
 *
 */

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <mk_msg.h>

#include <picogui.h>
#include <system.h>

#include <pgtools.h>

//-------------------------------------------------------------------
//-------------------------------------------------------------------
void CPGTools :: PGT_WDelete( pghandle& widget )
{
	PGT_FCT_START( DBG_START_END );

	if( widget != 0 )
	{
		pgDelete( widget );
		widget = 0;
	}

	PGT_FCT_END( DBG_START_END, 0 );
}

//-------------------------------------------------------------------
int CPGTools :: pgt_bcMenuFromArray( const pghandle wCanvas, const pghandle *pItems, const unsigned int NumItems )
{
	PGT_FCT_START( DBG_START_END );

	int ret = 0;
	pgNewPopupAt( PG_POPUP_ATEVENT, PG_POPUP_ATEVENT, 0, 0 );

	for( unsigned int i = 0; i < NumItems; i++ )
	{
		pgNewWidget( PG_WIDGET_MENUITEM, 0, 0 );
		pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pItems[i],
				0 );

		pgSetPayload( PGDEFAULT, i+1 );
	}

	pghandle evtFrom = pgGetEvent()->from;

	while( evtFrom == wCanvas )
		evtFrom = pgGetEvent()->from;

	dprintf((" == %s - event from: %ld\n", __FUNCTION__, evtFrom ));

	/* Return event */
	ret = pgGetPayload( evtFrom );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_mkPopupMenu( const pghandle wCanvas, const char *pMenuEntrys, const unsigned int EntryCnt )
{
	PGT_FCT_START( DBG_START_END );

	dprintf((" == %s - menu entrys: >%s<, cnt: %d\n", __FUNCTION__, menu_entrys, entry_cnt ));

	char *p, menu[PGT_MSG_SIZE];
	int ret = 0;

	if( pMenuEntrys && *pMenuEntrys && EntryCnt > 0 )
	{

		pghandle *pItems = NULL;

		if(( pItems = new pghandle[ EntryCnt] ) != NULL )
		{
			for( unsigned int i = 0; i < EntryCnt; i++ )
			{
				if( !( p = strchr( pMenuEntrys,'|' )))
					p = (char*)(pMenuEntrys + strlen( pMenuEntrys ));

				memset( menu, 0, PGT_MSG_SIZE );
				strncpy( menu, pMenuEntrys, strlen( pMenuEntrys ) - strlen( p ));

				pgFlushRequests( );
				pMenuEntrys = p + 1;
				pItems[i] = pgNewString( menu );
			}

			ret = pgt_bcMenuFromArray( wCanvas, pItems, EntryCnt );
		}

		if( pItems )	delete pItems;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//---------------------------------------------------------------------------------------------------------
int CPGTools :: PGT_CpyChldImg( const BYTE *pSource, BYTE *pDestination, const unsigned long SrcX, const unsigned long SrcY,
						const unsigned long width, const unsigned long height, const unsigned long SrcPitch,
						const unsigned long DstPitch )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;
	if( pSource != NULL )
	{
		for( unsigned long y = 0; y < height; y++ )
		{
			for( unsigned long x = 0; x < width; x++ )
				pDestination[x + y * DstPitch] = pSource[ x + SrcX + ( y + SrcY ) * SrcPitch];
		}
		ret = 0;
	}
	else
	{
		char *msg = "Empty source image";
		PGT_PERR( msg );
		PGT_GUIErrMsg( msg, ret );
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_CpyImg( const BYTE *pSource, BYTE *pDestination, const unsigned long width,
						const unsigned long height, const unsigned long SrcPitch, const unsigned long DstPitch )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;
	if( pSource != NULL )
	{
		for( unsigned long y = 0; y < height; y++ )
			memcpy( pDestination + y * DstPitch, pSource + y * SrcPitch, width );

		ret = 0;
	}
	else
	{
		char *msg = "Empty source image";
		PGT_PERR( msg );
		PGT_GUIErrMsg( msg, ret );
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_ReplaceSIText( PGT_STATUS_INFO si )
{
	PGT_FCT_START( DBG_START_END );

	int ret = 0;
	dprintf((" == %s - >>%s<<\n", __FUNCTION__, si.pMsg ));

	pghandle wOldText = pgGetWidget( si.wText ,PG_WP_TEXT );

	if( si.MsgLen > 0 )
		pgSetWidget( si.wText,
					PG_WP_TEXT, pgNewString( si.pMsg ),
					PG_WP_COLOR, si.wMsgCol,
					PG_WP_SIZE, si.MsgLen,
					0 );
	else
		pgSetWidget( si.wText,
					PG_WP_TEXT, pgNewString( si.pMsg ),
					PG_WP_COLOR, si.wMsgCol,
					0 );

	pgSubUpdate( si.wBox );
	pgSubUpdate( si.wText );

	PGT_WDelete( wOldText );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_ReplaceWText( pghandle widget, const char *pText )
{
	PGT_FCT_START( DBG_START_END );

	int ret = -1;
	dprintf((" == %s - >>%s<<\n", __FUNCTION__, text ));

	if( widget > 0 )
	{
		pgReplaceText( widget, pText );
		pgSubUpdate( widget );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_DrawROI( const pgcontext gc, const long x, const long y, const long w, const long h, const pgcolor wColor )
{
	PGT_FCT_START( DBG_START_END );

	int ret = -1;
	dprintf((" == %s - x: %ld, y: %ld, w: %ld, h: %ld, color: 0x%x\n", __FUNCTION__, x, y, w, h, wColor ));

	if( gc )
	{
		pgSetColor( gc, wColor );
		pgFrame( gc, x, y, w, h );
		//pgContextUpdate( gc );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_DrawLine( const pgcontext gc, const long x0, const long y0, const long x1, const long y1, const pgcolor wColor )
{
	PGT_FCT_START( DBG_START_END );

	int ret = -1;
	dprintf((" == %s - x0: %ld, y0: %ld, x1: %ld, y1: %ld, color: 0x%x\n", __FUNCTION__, x0, y0, x1, y1, wColor ));

	if( gc )
	{
		pgSetColor( gc, wColor );
		pgLine( gc, x0, y0, x1, y1 );
		//pgContextUpdate( gc );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_DrawPoint( const pgcontext gc, const long x, const long y, const pgcolor wColor )
{
	PGT_FCT_START( DBG_START_END );

	int ret = -1;
	dprintf((" == %s - x: %ld, y: %ld, color: 0x%x\n", __FUNCTION__, x, y, wColor ));

	if( gc )
	{
		pgSetColor( gc, wColor );
		pgPixel( gc, x, y );
		//pgContextUpdate( gc );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_DrawRect( const pgcontext gc, const long x, const long y, const long w, const long h, const pgcolor wColor )
{
	PGT_FCT_START( DBG_START_END );

	int ret = -1;
	dprintf((" == %s - x: %ld, y: %ld, w: %ld, h: %ld, color: 0x%x\n", __FUNCTION__, x, y, w, h, wColor ));

	if( gc )
	{
		pgSetColor( gc, wColor );
		pgRect( gc, x, y, w, h );
		//pgContextUpdate( gc );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_DrawText( const pgcontext gc, const long x, const long y, const char *pText, const pgcolor wColor )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	dprintf((" == %s - x: %ld, y: %ld, text: >>%s<<, color: 0x%x\n", __FUNCTION__, x, y, pText, wColor ));

	if( gc )
	{
		pgSetColor( gc, wColor );
		pgText( gc, x, y, pgNewString( pText ));
		//pgContextUpdate( gc );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GUINoteOrErrMsg( const char *pMsgType, const char *pS, const long ErrCode )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pS )
	{
#if 1
		ret = PGT_GUINoteOrErrMsg( pMsgType, pS, NULL, ErrCode );
#else
		pgt_CleanMsgBuf( );

		if( ErrCode == 0 )
			sprintf( m_pgt_msg, "%s !!\n", pS );
		else
			sprintf( m_pgt_msg, "%s -> err: %ld !!\n", pS, ErrCode );

		if( pMsgType )
			pgMessageDialog( pMsgType, m_pgt_msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
		else
			pgMessageDialog( "Message", m_pgt_msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

		ret = 0;
#endif
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GUINoteOrErrMsg( const char *pMsgType, const char *pS,  const char *pErrStr, const long ErrCode )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pS )
	{
		pgt_CleanMsgBuf( );

		if( pErrStr == NULL )
		{
			if( ErrCode == 0 )
				sprintf( m_pgt_msg, "%s !!\n", pS );
			else
				sprintf( m_pgt_msg, "%s -> err: %ld !!\n", pS, ErrCode );
		}
		else
		{
			if( ErrCode == 0 )
				sprintf( m_pgt_msg, "%s !!\n%s\n", pS, pErrStr );
			else
				sprintf( m_pgt_msg, "%s -> %s (code: %ld) !!\n", pS, pErrStr, ErrCode );
		}

		if( pMsgType )
			pgMessageDialog( pMsgType, m_pgt_msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
		else
			pgMessageDialog( "Message", m_pgt_msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GUIErrMsg( const char *pS, const long ErrCode )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pS )
	{
#if 1
		ret = PGT_GUIErrMsg( pS, NULL, ErrCode );
#else
		pgt_CleanMsgBuf( );
		sprintf( m_pgt_msg, "*** An error hase occurred! ***\n"
						"Error message: %s\ncode: %ld!!\n", pS, ErrCode );
		pgMessageDialog( "ERROR!!!!", m_pgt_msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
		ret = 0;
#endif
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GUIErrMsg( const char *pS, const char *pErrStr, const long ErrCode )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pS )
	{
		pgt_CleanMsgBuf( );

		if( pErrStr == NULL )
			sprintf( m_pgt_msg, "*** An error hase occurred! ***\n"
							"Error message: %s\ncode: %ld!!\n", pS, ErrCode );
		else
			sprintf( m_pgt_msg, "*** An error hase occurred! ***\n"
							"Error message: %s\n %s\n", pS, pErrStr );

		pgMessageDialog( "ERROR!!!!", m_pgt_msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GUIExitApplication( const char *pS, const long ErrCode )
{
	PGT_FCT_START( DBG_START_END );
	int ret = 0;

	if( pS ) PGT_GUIErrMsg( pS, ErrCode );

	PGT_CloseAll( );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GUIExitApplication( const char *pS, const char *pErrStr, const long ErrCode )
{
	PGT_FCT_START( DBG_START_END );
	int ret = 0;

	if( pS ) PGT_GUIErrMsg( pS, pErrStr, ErrCode );

	PGT_CloseAll( );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkCheckBox( const pghandle wParent, const int position,
						const char *pText, pgevthandler evtFctHandle, int status )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if(( wRet = pgNewWidget( PG_WIDGET_CHECKBOX, position, wParent )) > 0 )
	{
		pgSetWidget( wRet,
					PG_WP_TEXT, pgNewString( pText ),
					PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
					PG_WP_EXTDEVENTS, PG_EXEV_TOGGLE,
					PG_WP_ON, status,
					0 );

		if( evtFctHandle != NULL )
			pgBind( wRet, PG_WE_ACTIVATE, evtFctHandle, NULL );
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkButton( const pghandle wParent, const int position, const char *pText,
						const int size, pgevthandler evtFctHandle )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if(( wRet = pgNewWidget( PG_WIDGET_BUTTON, position, wParent )) > 0 )
	{
		pgSetWidget( wRet,
					PG_WP_TEXT, pgNewString( pText ),
					PG_WP_SIZE, size,
					PG_WP_FONT, pgNewFont( NULL, 6, 0 ),
					0 );

		if( evtFctHandle != NULL )
			pgBind( PGDEFAULT, PG_WE_ACTIVATE, evtFctHandle, NULL );
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkButton( const pghandle wParent, const int position, const char *pText, pgevthandler evtFctHandle,
						const pghandle wColor, const int FontSize )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if(( wRet = pgNewWidget( PG_WIDGET_BUTTON, position, wParent )) > 0 )
	{
		pgSetWidget( wRet,
					PG_WP_TEXT, pgNewString( pText ),
					PG_WP_FONT, pgNewFont( NULL, FontSize, 0 ),
					PG_WP_COLOR, wColor,
					0 );

		if( evtFctHandle != NULL )
			pgBind( PGDEFAULT, PG_WE_ACTIVATE, evtFctHandle, NULL );
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkLabel( const pghandle wParent, const int position, const char *pText, pghandle wColor )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if(( wRet = pgNewWidget( PG_WIDGET_LABEL, position, wParent )) > 0 )
	{
		pgSetWidget( wRet,
					PG_WP_SIDE, PG_S_TOP,
					PG_WP_ALIGN, PG_A_RIGHT,
					PG_WP_TEXT, pgNewString( pText ),
					PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
					PG_WP_COLOR, wColor,
					PG_WP_SIZE, 8,
					0 );
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkLabel( const pghandle wParent, const int position, const int FontSize, const char *pText )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if( pText )
	{
		long x = 0, y = 0;
		pghandle wFont = pgNewFont( NULL, FontSize, PG_FSTYLE_DEFAULT );
		if( PGT_GetStrSize( wFont, pText, x, y ) >= 0 )
		{
			wRet = pgNewWidget( PG_WIDGET_LABEL, position, wParent );
			pgSetWidget( wRet,
						PG_WP_SIZE, x,
						PG_WP_SIDE, PG_S_LEFT,
						PG_WP_FONT, wFont,
						PG_WP_TEXT, pgNewString( pText ),
						0 );
		}
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}


//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkScroller( const pghandle wParent, const int position, const long MaxSize,
						const long DefaultValue, pgevthandler evtFctHandle )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	dprintf((" == %s - MaxSize: %ld, DefaultValue: %ld\n", __FUNCTION__, MaxSize, DefaultValue ));

	if(( wRet = pgNewWidget( PG_WIDGET_SCROLL, position, wParent )) > 0 )
	{
		pgSetWidget( wRet,
					PG_WP_SIDE, PG_S_TOP,
					PG_WP_SIZE, MaxSize,
					PG_WP_VALUE, DefaultValue,
					PG_WP_SCROLL_X, 0,
					PG_WP_SCROLL_Y, DefaultValue,
					0 );

		if( evtFctHandle != NULL )
			pgBind( PGDEFAULT, PG_WE_ACTIVATE, evtFctHandle, NULL );
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkWBox( const pghandle wParent, const int position, const int side, const int size )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if(( wRet = pgNewWidget( PG_WIDGET_BOX, position, wParent )) > 0 )
		pgSetWidget( wRet,
					PG_WP_SIDE, side,
					PG_WP_SIZE, size,
					0 );

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkScrollerLabel( const pghandle wParent, int position, const char *pText )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	long x = 0, y = 0;
	pghandle wFont = pgNewFont( NULL, 6, PG_FSTYLE_DEFAULT );
	if( PGT_GetStrSize( wFont, pText, x, y ) >= 0 )
	{
		dprintf((" == %s - text: >>%s<< (x/y: %d/%d)\n", __FUNCTION__, pText, x, y ));

		if(( wRet = PGT_mkWBox( wParent, position, PG_S_LEFT, x + 5 )) > 0 )
		{
			pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, wRet );
			pgSetWidget( PGDEFAULT,
						PG_WP_TEXT, pgNewString( pText ),
						PG_WP_FONT, wFont,
						PG_WP_SIZE, x,
						PG_WP_SIDE, PG_S_LEFT,
						0 );
		}
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkStatusBar( const pghandle wParent, const int position, const int size )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if(( wRet = pgNewWidget( PG_WIDGET_TOOLBAR, position, wParent )) > 0 )
		pgSetWidget( wRet,
					PG_WP_SIDE, PG_S_TOP,
					PG_WP_SIZE, size,
					0 );

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkCanvas( const pghandle wParent, const int position, pgevthandler evtFctHandle, const bool bUseTrigg )
{
	PGT_FCT_START( DBG_START_END );

	pghandle wRet = pgNewWidget( PG_WIDGET_CANVAS, position, wParent );

	if( bUseTrigg )
		pgSetWidget( wRet,
				PG_WP_TRIGGERMASK, pgGetWidget( wRet, PG_WP_TRIGGERMASK ) | PG_TRIGGER_MOVE,
				0 );

	if( evtFctHandle )
		pgBind( wRet, PGBIND_ANY, evtFctHandle, NULL );

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GetStrSize( const pghandle wFont, const char *pStr, long& x, long& y )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pStr && wFont > 0 )
	{
		pgEnterContext( );
		pgSizeText( (int*)&x, (int*)&y, wFont, pgNewString( pStr ));
		pgLeaveContext( );

		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GetStrSize( const char *pStr, long& x, long& y )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pStr )
	{
		pghandle wFont = pgNewFont( NULL, 8, 0 );
		ret = PGT_GetStrSize( wFont, pStr, x, y );
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GetStrXSize( const char *pStr )
{
	PGT_FCT_START( DBG_START_END );
	long ret = -1;

	if( pStr )
	{
		long y = 0;
		if( PGT_GetStrSize( pStr, ret, y ) < 0 )
			ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GetStrYSize( const char *pStr )
{
	PGT_FCT_START( DBG_START_END );
	long ret = -1;

	if( pStr )
	{
		long x = 0;
		if( PGT_GetStrSize( pStr, x, ret ) < 0 )
			ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_mkStatBarLab( const pghandle wParent, int position, PGT_STATUS_INFO& si )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	long x = 0, y = 0, offs = 5;

	if( si.pMsg != NULL )
	{
		ret = 0;
		if(( x = si.MsgLen ) < 0 )
		{
			pghandle wFont = pgNewFont( NULL, 8, 0 );
			ret = PGT_GetStrSize( wFont, si.pMsg, x, y );
		}

		if( ret >= 0 )
		{
			ret = -1;
			if(( si.wBox = PGT_mkWBox( wParent, position, PG_S_LEFT, x + offs )) > 0 )
				if(( si.wText = PGT_mkLabel( PGDEFAULT, PG_DERIVE_INSIDE, si.pMsg, si.wMsgCol )) > 0 )
					ret = 0;
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_mkStatBarLab( const pghandle wParent, const int position, pghandle& wText,
						const char *pStr, int size )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet = 0;

	if( pStr )
	{
		int ret = 0;
		long x = 0, y = 0, offs = 5;

		if(( x = size ) < 0 )
		{
			pghandle wFont = pgNewFont( NULL, 8, 0 );
			ret = PGT_GetStrSize( wFont, pStr, x, y );
		}

		if( ret >= 0 )
		{
			if(( wRet = PGT_mkWBox( wParent, position, PG_S_LEFT, x + offs )) > 0 )
				if(( wText = PGT_mkLabel( PGDEFAULT, PG_DERIVE_INSIDE, pStr, PGT_COLOR_BLACK )) <= 0 )
					wRet = 0;
		}
	}

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_SetWColor( const pghandle widget, const pghandle wColor )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	dprintf((" == %s - widget: %ld, wColor: %ld\n", __FUNCTION__, widget, wColor ));

	if( widget > 0 )
	{
		pgSetWidget( widget, PG_WP_COLOR, wColor, 0 );
		pgSubUpdate( widget );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_EnableWButton( const pghandle wButton, const bool OnOff )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( wButton > 0 )
	{
		pgSetWidget( wButton, PG_WP_DISABLED, !OnOff, 0 );
		pgSubUpdate( wButton );
		ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
bool CPGTools :: PGT_CheckIfFileExist( const char *pFileName )
{
	PGT_FCT_START( DBG_START_END );
	bool ret = false;

	FILE *f;

	if( pFileName )
	{
		if(( f = fopen( pFileName, "r" )) != NULL )
		{
			fclose( f );
			ret = true;
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_GUINote4Limit( const char *pType, const long value )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pType )
	{
		pgt_CleanMsgBuf( );
		sprintf( m_pgt_msg, "The entered value is %ld!! This is an illegal value!\n"
						"The %s will be set to the default value!\n", value, pType );
		pgMessageDialog( "Note...", m_pgt_msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
pghandle CPGTools :: PGT_GetStatusColor( const long status )
{
	PGT_FCT_START( DBG_START_END );
	pghandle wRet;

	if( status == 0 )
		wRet = PGT_COLOR_GREY;
	else if( status < 0 )
		wRet = PGT_COLOR_RED;
	else
		wRet = PGT_COLOR_BLACK;

	PGT_FCT_END( DBG_START_END, wRet );
	return wRet;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_LSIUpdate( PGT_STATUS_INFO& si, const char *pMsgPrefix, const char *pMsgSufix,
							const long value, const int ColorMode, const int MsgLen )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	char msg[PGT_MSG_SIZE];
	memset( msg, 0, PGT_MSG_SIZE );

	if( pMsgPrefix != NULL )
		sprintf( msg, "%s: %ld", pMsgPrefix, value );
	if( pMsgSufix != NULL )
		sprintf( msg, "%ld %s", value, pMsgSufix );
	if( pMsgPrefix != NULL && pMsgSufix != NULL )
	{
		memset( msg, 0, PGT_MSG_SIZE );
		sprintf( msg, "%s: %ld %s", pMsgPrefix, value, pMsgSufix );
	}

	si.pMsg = msg;

	if( MsgLen == -1 )
		si.MsgLen = MsgLen;
	else
		si.MsgLen = PGT_GetStrXSize( si.pMsg );

	si.wMsgCol = PGT_GetStatusColor( ColorMode );

	ret = PGT_ReplaceSIText( si );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_DSIUpdate( PGT_STATUS_INFO& si, const char *pMsgPrefix, const char *pMsgSufix,
							const double value, const int ColorMode, const int MsgLen )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	char msg[PGT_MSG_SIZE];
	memset( msg, 0, PGT_MSG_SIZE );

	if( pMsgPrefix != NULL )
		sprintf( msg, "%s: %.3f", pMsgPrefix, value );
	if( pMsgSufix != NULL )
		sprintf( msg, "%.3f %s", value, pMsgSufix );
	if( pMsgPrefix != NULL && pMsgSufix != NULL )
	{
		memset( msg, 0, PGT_MSG_SIZE );
		sprintf( msg, "%s: %.3f %s", pMsgPrefix, value, pMsgSufix );
	}

	si.pMsg = msg;

	if( MsgLen == -1 )
		si.MsgLen = MsgLen;
	else
		si.MsgLen = PGT_GetStrXSize( si.pMsg );

	si.wMsgCol = PGT_GetStatusColor( ColorMode );

	ret = PGT_ReplaceSIText( si );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_CSIUpdate( PGT_STATUS_INFO& si, const char *pMsgPrefix, const char *pMsgSufix, const char *pData,
							const int ColorMode, const int MsgLen )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	char msg[PGT_MSG_SIZE];
	memset( msg, 0, PGT_MSG_SIZE );

	if( pData != NULL )
	{
		if( pMsgPrefix != NULL && ( strncmp( PGT_PREFIX_4_MSG_DATA, pMsgPrefix, strlen( PGT_PREFIX_4_MSG_DATA )) == 0 ))
		{
			sprintf( msg, "%s", pData );
		}
		else
		{
			if( pMsgPrefix != NULL && pMsgSufix != NULL )
				sprintf( msg, "%s: >>%s<< %s", pMsgPrefix, pData, pMsgSufix );
			else if( pMsgPrefix == NULL && pMsgSufix == NULL )
				sprintf( msg, ">>%s<<", pData );
			else if( pMsgPrefix != NULL )
				sprintf( msg, "%s: >>%s<<", pMsgPrefix, pData );
			else if( pMsgSufix != NULL )
				sprintf( msg, ">>%s<< %s", pData, pMsgSufix );
		}

		si.pMsg = msg;

		if( MsgLen == -1 )
			si.MsgLen = MsgLen;
		else
			si.MsgLen = PGT_GetStrXSize( si.pMsg );

		si.wMsgCol = PGT_GetStatusColor( ColorMode );

		ret = PGT_ReplaceSIText( si );
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_SavePGMFile( const BYTE *pDataBuf, const FILE *pF, const long FileSize )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	char str[PGT_MSG_SIZE];

	if( pDataBuf != NULL && pF != NULL && FileSize > 0 )
	{
		if(( ret = fwrite( pDataBuf, 1, FileSize, (FILE*)pF )) != FileSize )
		{
			memset( str, 0, PGT_MSG_SIZE );
			sprintf( str, "%d BYTES written from %ld", ret, FileSize );
			PGT_GUINoteOrErrMsg( "Save error", str, -1 );
			ret = -1;
		}
		else
			ret = 0;
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_SaveFile( const BYTE *pDataBuf, const char *pFileName, const PGT_FL_TYPES FileType, const long FileSize )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pDataBuf != NULL && pFileName != NULL && FileSize > 0 )
	{
		const char *pFile = NULL;
		char msg[PGT_MAX_CHAR_SIZE];
		FILE *pF = NULL;

		memset( msg, 0, PGT_MAX_CHAR_SIZE );

		if(( pFile = pgFilePicker( NULL, NULL, pFileName, PG_FILESAVE, "Save the file" )))
		{
			//i = pgMessageDialog("Saving the file...", file, PG_MSGBTN_YES | PG_MSGBTN_NO );

			//if( i == PG_MSGBTN_YES )
			{
				switch( FileType )
				{
				case PGT_FT_INI:
					if(( pF = fopen( pFile, "w" )) != NULL )
						ret = PGT_SaveINIFile( pF );
					break;
				case PGT_FT_DDC:
					if(( pF = fopen( pFile, "w" )) != NULL )
						ret = PGT_SaveINIFile( pF );
					break;
				case PGT_FT_PGM:
					if(( pF = fopen( pFile, "wb" )) != NULL )
						ret = PGT_SavePGMFile( pDataBuf, pF, FileSize );
					break;
				default:
					PGT_GUIErrMsg( "Wrong file type specified", ret );
					break;
				}

				if( pF != NULL )
				{
					fclose( pF );
					pF = NULL;
				}
			}
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
int CPGTools :: PGT_SaveFile( const BYTE *pDataBuf, const char *pFileName, const char *pFileType, const long FileSize )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pDataBuf != NULL && pFileName != NULL && FileSize > 0 )
	{
		PGT_FL_TYPES FileType = pgt_GetFlType( pFileType );

		switch( FileType )
		{
		case PGT_FT_INI:
		case PGT_FT_DDC:
		case PGT_FT_PGM:
			ret = PGT_SaveFile( pDataBuf, pFileName, FileType, FileSize );
			break;
		default:
			break;
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
PGT_FL_TYPES CPGTools :: pgt_GetFlType( const char *pFileType )
{
	PGT_FCT_START( DBG_START_END );
	PGT_FL_TYPES ret = PGT_FT_LAST;

	if( pFileType )
	{
		for( unsigned i = PGT_FT_FIRST + 1; i < PGT_FT_LAST; i++ )
		{
			if( strncmp( m_pgt_ft[i].sufix, pFileType, 4 ) == 0 )
			{
				ret = (PGT_FL_TYPES)i;
				break;
			}
		}
	}

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
int CPGTools :: PGT_mkPGMFile( const BYTE *pSource, BYTE **pDestination, const unsigned long width,
					unsigned long height, unsigned long pitch, unsigned long bpp )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	char pgmh[32];
	int hlen;
	unsigned int image_file_size = width * height * bpp;
	unsigned int fsize;

	if( pSource != NULL )
	{
		PGT_PGMHeader( pgmh, width, height );
		hlen = strlen( pgmh );

		fsize = image_file_size + hlen;

		if( *pDestination != NULL )
		{
			delete *pDestination;
			*pDestination = NULL;
		}

		if(( *pDestination = new BYTE[fsize]) != NULL )
		{
			memcpy( *pDestination + 0, pgmh, hlen );
			for( unsigned int y = 1; y < height; y++ )
				memcpy( *pDestination + hlen + y * width, pSource + y * pitch, width );

			ret = fsize;
		}
		else
			PGT_PERR( "out of memory" );
	}
	else
		PGT_PERR( "empty source buffer" );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//----------------------------------------------------------------------
void CPGTools :: PGT_DrawROI( const pghandle wCanvas, const long x, const long y, const long w, const long h, const pghandle wColor )
{
	PGT_FCT_START( DBG_START_END );
	if( wCanvas )
	{
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, wColor );
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 5, PG_GROP_FRAME, x, y, w, h );
	}

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
void CPGTools :: PGT_DrawRect( const pghandle wCanvas, const long x, const long y, const long w, const long h, const pghandle wColor )
{
	PGT_FCT_START( DBG_START_END );
	if( wCanvas )
	{
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, wColor );
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 5, PG_GROP_RECT, x, y, w, h );
	}

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
void CPGTools :: PGT_DrawLine( const pghandle wCanvas, const long x0, const long y0, const long x1, const long y1, const pghandle wColor )
{
	PGT_FCT_START( DBG_START_END );
	if( wCanvas )
	{
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, wColor );
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 5, PG_GROP_LINE, x0, y0, x1-x0, y1-y0 );
	}

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
void CPGTools :: PGT_DrawPoint( const pghandle wCanvas, const long x, const long y, const pghandle wColor )
{
	PGT_FCT_START( DBG_START_END );
	if( wCanvas )
	{
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 2, PG_GROP_SETCOLOR, wColor );
		pgWriteCmd( wCanvas, PGCANVAS_GROP, 5, PG_GROP_PIXEL, x, y, 1, 1 );
	}

	PGT_FCT_END( DBG_START_END, 0 );
}

//----------------------------------------------------------------------
int CPGTools :: PGT_LoadFile( char *pFileName )
{
	PGT_FCT_START( DBG_START_END );
	int ret = -1;

	if( pFileName )
	{
		const char *pName = NULL;
		if(( pName = pgFilePicker( NULL, NULL, "dummy.pgt", PG_FILEOPEN, "Load a file...")))
		{
			dprintf((" == %s (%s: %d) - %s\n", __FUNCTION__, __FILE__, __LINE__, pName ));
			sprintf( pFileName, "%s", pName );
			ret = 0;
		}
	}
	else
		PGT_GUIErrMsg( "Wrong input buffer (NULL)", ret );

	PGT_FCT_END( DBG_START_END, ret );
	return ret;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
