//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - demo
// Mon Feb 14 11:50:05 CET 2005
//----------------------------------------------------------------------
#ifndef __display_h__
#define __display_h__

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <vl_base.h>
#include <vl_display.h>

#include "ocrtool.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
class CDmoDisplay
{
public:
	/*----------------------------------------------------------------*/
	CDmoDisplay( )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		m_wSBox = 0;
		m_hOoisHandle = DISP_DONT_CARE;
		m_hAoiHandle = NULL;
		m_hPoiHandle = NULL;
		m_hTextHandle = IPL_DONT_CARE;
		m_hWnd = 0;
		m_Count = 0;

		m_hRoiHandle = IPL_DONT_CARE;

		m_roi.left   = 10;
		m_roi.top    = 10;
		m_roi.right  = 100;
		m_roi.bottom = 100;
		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	virtual ~CDmoDisplay( )
	{
		dprintf((" ++ ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		widget_delete( &m_wSBox );
		delOOIS( );
		delROI( );

		if( m_hOoisHandle != DISP_DONT_CARE )
			DISP_OoiSetFree( m_hOoisHandle );

		delDisplay( );
		dprintf((" -- ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void delDisplay( )	{ DISP_CloseWindow( m_hWnd ); }

	/*----------------------------------------------------------------*/
	/* make the display */
	int mkDisplay( const pghandle wParent, const int position, const IPL_BUFHANDLE imageHandle )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		DMO_CHECK_ERR( wParent > 0, -1 );

		/* rebuild the display scroll box */
		widget_delete( &m_wSBox );
		m_wSBox = pgNewWidget( PG_WIDGET_SCROLLBOX, position, wParent );
		pgSetWidget( PGDEFAULT, PG_WP_SIDE, PG_S_ALL, 0 );

		/* open a dispay window in the scroll box */
		DMO_CHECK_ERR( DISP_OpenWindow( imageHandle, &m_hWnd, m_wSBox, NULL ), IPL_GetErrorCode2( ));

		/* display the image*/
		displayImage( );

		DMO_CHECK_ERR( DISP_OoiSetAlloc( &m_hOoisHandle ), IPL_GetErrorCode2( ));

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return IPL_TRUE;
	}

	/*----------------------------------------------------------------*/
	void displayImage( )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		/* */
		DISP_ControlWindow( DISP_OOISET, m_hWnd, &m_hOoisHandle );

		/* update the display */
		DISP_UpdateWindow( m_hWnd, DISP_INVALIDATE );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void drawAOI( const int index, const IPL_RECT rect, const pgcolor wColor, const int xOffset = 0, const int yOffset = 0 )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( index >= 0 )
			drawAOI( m_hAoiHandle[index], rect, wColor, xOffset, yOffset );
		else
			drawAOI( m_hRoiHandle, rect, wColor, xOffset, yOffset );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void drawText( const int index, const int x, const int y, const char* pText, const pgcolor wColor )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( index >= 0 )
			drawText( m_hPoiHandle[index], x, y, pText, wColor );
		else
			drawText( m_hTextHandle, x, y, pText, wColor );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void delOOI( DISP_OOIHANDLE*& ooiHandle )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		if( ooiHandle )
		{
			for( int i = 0;  i < m_Count; i++ )
			{
				if( ooiHandle[i] != DISP_DONT_CARE )
				{
					DISP_OoiFree( m_hOoisHandle, ooiHandle[i] );
					ooiHandle[i] = DISP_DONT_CARE;
				}
			}

			delete [] ooiHandle;
			ooiHandle = NULL;
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void delOOIS( )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		delOOI( m_hAoiHandle );
		delOOI( m_hPoiHandle );
		m_Count = 0;

		if( m_hTextHandle != DISP_DONT_CARE )
			DISP_OoiFree( m_hOoisHandle, m_hTextHandle );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	int mkOOIs( const int count )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret = IPL_FALSE;

		delOOIS( );
		m_Count = count;
		if( m_Count > 0 )
		{
			DMO_CHECK_ERR(( m_hAoiHandle = new DISP_OOIHANDLE[m_Count] ) != NULL, -1 );
			memset( m_hAoiHandle, IPL_DONT_CARE, sizeof( DISP_OOIHANDLE ) * m_Count );

			DMO_CHECK_ERR(( m_hPoiHandle = new DISP_OOIHANDLE[m_Count] ) != NULL, -1 );
			memset( m_hPoiHandle, IPL_DONT_CARE, sizeof( DISP_OOIHANDLE ) * m_Count );

			ret = IPL_TRUE;
		}

		dprintf((" -- %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	IPL_RECT getROI( ){ getROIPos( ); return m_roi; }
	void setROI( const IPL_RECT roi ){ m_roi = roi; }

	/*----------------------------------------------------------------*/
	void drawROI( const pghandle wColor = MVCOLOR_LBLUE )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		drawAOI( -1, m_roi, wColor );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void delROI( )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		getROIPos( );

		if( m_hRoiHandle != DISP_DONT_CARE )
		{
			DISP_OoiFree( m_hOoisHandle, m_hRoiHandle );
			m_hRoiHandle = DISP_DONT_CARE;
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	pghandle m_wSBox;

	/*----------------------------------------------------------------*/
private:
	/*----------------------------------------------------------------*/
	DISP_OOISETHANDLE m_hOoisHandle;
	DISP_OOIHANDLE* m_hAoiHandle;
	DISP_OOIHANDLE* m_hPoiHandle;
	DISP_OOIHANDLE m_hTextHandle;
	IPL_HWND m_hWnd;									/* mvIMPACT windows handle */

	int m_Count;

	DISP_OOIHANDLE m_hRoiHandle;
	IPL_RECT m_roi;

	/*----------------------------------------------------------------*/
	void drawText( DISP_OOIHANDLE& ooiHandle, const int x, const int y, const char* pText, const pgcolor wColor )
	{
		dprintf((" ++ %s (%s: %d) - text: >>%s<<\n", __FUNCTION__, __FILE__, __LINE__, pText ));

		IPL_POINT boxPoints = {0,0};
		boxPoints.xpos = x;
		boxPoints.ypos = y;

		DISP_OoiAlloc( m_hOoisHandle, &ooiHandle, DISP_POI_TOOL, &boxPoints, 2, NULL );

		int vis = 0;
		DISP_ControlOoi( m_hOoisHandle, ooiHandle, DISP_OOI_LINES_VISIBLE, &vis );

		if( pText )
		{
			DISP_ControlOoi( m_hOoisHandle, ooiHandle, DISP_OOI_TEXT, pText );
			DISP_ControlOoi( m_hOoisHandle, ooiHandle, DISP_OOI_TEXT_COLOR, &wColor );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void drawAOI( DISP_OOIHANDLE& ooiHandle, const IPL_RECT rect, const pgcolor wColor, const int xOffset = 0, const int yOffset = 0 )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		IPL_POINT boxPoints[5];
		memset( boxPoints, 0, 5 * sizeof( IPL_POINT ));

		dprintf((" == %s - Rect: %ld %ld %ld %ld Offset %d %d\n",
				__FUNCTION__, rect.left, rect.top, rect.right, rect.bottom, xOffset, yOffset ));

		boxPoints[0].xpos = rect.left + xOffset;
		boxPoints[0].ypos = rect.top  + yOffset;

		boxPoints[1].xpos = rect.right + xOffset;
		boxPoints[1].ypos = rect.top   + yOffset;

		boxPoints[2].xpos = rect.right  + xOffset;
		boxPoints[2].ypos = rect.bottom + yOffset;

		boxPoints[3].xpos = rect.left   + xOffset;
		boxPoints[3].ypos = rect.bottom + yOffset;

		boxPoints[4].xpos = rect.left + xOffset;
		boxPoints[4].ypos = rect.top  + yOffset;

		DISP_OoiAlloc( m_hOoisHandle, &ooiHandle, DISP_AOI_TOOL, boxPoints, sizeof( boxPoints )/sizeof( boxPoints[0] ), NULL );
		DISP_ControlOoi( m_hOoisHandle, ooiHandle, DISP_OOI_COLOR, &wColor );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	void getROIPos( )
	{
		dprintf((" ++ %s (%s: %d) - m_hRoiHandle: %ld\n", __FUNCTION__, __FILE__, __LINE__, m_hRoiHandle ));

		if( m_hRoiHandle != IPL_DONT_CARE )
		{
			m_roi.left = 0;
			DISP_InquireOoi( m_hOoisHandle, m_hRoiHandle, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+0), &m_roi.left );
			m_roi.right = 0;
			DISP_InquireOoi( m_hOoisHandle, m_hRoiHandle, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+1), &m_roi.right );
			m_roi.top = 0;
			DISP_InquireOoi( m_hOoisHandle, m_hRoiHandle, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+2), &m_roi.top );
			m_roi.bottom = 0;
			DISP_InquireOoi( m_hOoisHandle, m_hRoiHandle, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+3), &m_roi.bottom );

			dprintf((" == %s - %ld %ld %ld %ld\n", __FUNCTION__, m_roi.left, m_roi.top, m_roi.right, m_roi.bottom ));
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#endif	// __display_h__

