//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>ocrtool<<
// Tue May 20 14:35:47 CEST 2003
//----------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>
#include <vl_ocr.h>
#include <vl_file.h>
#include <vl_ipl.h>

#include <stopwatch.h>
#include <samples.h>
#include <mvslib.h>

#include <pgutils.h>
#include "pgtools.h"

#include "ocrtool.h"
#include "display.h"
#include "ocr.h"
#include "preproc.h"
#include "rwfile.h"
#include "senddata.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define KB_WAIT_TIME		100000	/* usec */
#define BMP_IMG_SUFIX		".bmp"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define WI2FL( a, b )							\
		DMO_CHECK_ERR(( ret = g_pRWF->writeNewIEntry( INIT_STR_##a, b )), -1 );

#define WL2FL( a, b )							\
		DMO_CHECK_ERR(( ret = g_pRWF->writeNewLEntry( INIT_STR_##a, b )), -1 );

/*----------------------------------------------------------------*/
#define GIV( istr, p_min, p_max, p_def )						\
		g_pRWF->readIValue( INIT_STR_##istr, p_min, p_max, p_def );

#define GLV( istr, p_min, p_max, p_def )						\
		g_pRWF->readLValue( INIT_STR_##istr, p_min, p_max, p_def );

#define GBV( istr, p_def )						\
		g_pRWF->readBoolValue( INIT_STR_##istr, p_def );

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*! \brief Application start mode (start process) */
typedef enum START_MODE
{
	START_ONLY = 0,			/*!< \brief Start the application with no processing and no live image. */
	START_LIVE,				/*!< \brief Start the application in the live snap mode. */
	START_READ,				/*!< \brief Start the application in the live snap and with the processing. */
};

//-------------------------------------------------------------------
/*! \brief Size type (segment size or character size) */
typedef enum SIZETYPE
{
	ST_SEGMENT = 0,			/*!< \brief Segment size. */
	ST_CHARACTER,			/*!< \brief Character size. */
};

//-------------------------------------------------------------------
//-------------------------------------------------------------------
typedef struct SSCROLLER
{
	pghandle wScrollBox;					/*!< \brief Scroll box handle */
	pghandle wScroller;						/*!< \brief Scroller handle */
	pghandle wScrollLab;					/*!< \brief Scroller label */
};

//-------------------------------------------------------------------
//-------------------------------------------------------------------
static STATUS_INFO siRunMode    = { 0, 0, NULL, -1 };	/*!< \brief \b Run \b Mode status info. */
static STATUS_INFO siOCRMode    = { 0, 0, NULL, -1 };	/*!< \brief \b OCR \b Mode status info. */
static STATUS_INFO siThreshold  = { 0, 0, NULL, -1 };	/*!< \brief Threshold value status info. */
static STATUS_INFO siSegMinW    = { 0, 0, NULL, -1 };	/*!< \brief Minimal segment width status info. */
static STATUS_INFO siSegMaxW    = { 0, 0, NULL, -1 };	/*!< \brief Maximal segment width status info. */
static STATUS_INFO siSegMinH    = { 0, 0, NULL, -1 };	/*!< \brief Minimal segment height status info. */
static STATUS_INFO siSegMaxH    = { 0, 0, NULL, -1 };	/*!< \brief Maximal segment height status info. */
static STATUS_INFO siPolarity   = { 0, 0, NULL, -1 };	/*!< \brief Polarity status info. */
static STATUS_INFO siSegCnt     = { 0, 0, NULL, -1 };	/*!< \brief Segmen count status info. */
static STATUS_INFO siMinQuality = { 0, 0, NULL, -1 };	/*!< \brief Minimal allowed quality status info. */
static STATUS_INFO siFPS        = { 0, 0, NULL, -1 };	/*!< \brief Frames per second status info. */
static STATUS_INFO siProc       = { 0, 0, NULL, -1 };	/*!< \brief Processing time status info. */
static STATUS_INFO siOPLen      = { 0, 0, NULL, -1 };	/*!< \brief Operator length (dyn. th.) status info. */
static STATUS_INFO siMinDist    = { 0, 0, NULL, -1 };	/*!< \brief Minimal distance (dyn. th.) status info. */
static STATUS_INFO siCharMinW   = { 0, 0, NULL, -1 };	/*!< \brief Minimal character width status info. */
static STATUS_INFO siCharMaxW   = { 0, 0, NULL, -1 };	/*!< \brief Maximal character width status info. */
static STATUS_INFO siCharMinH   = { 0, 0, NULL, -1 };	/*!< \brief Minimal character height status info. */
static STATUS_INFO siCharMaxH   = { 0, 0, NULL, -1 };	/*!< \brief Maximal character height status info. */

static STATUS_INFO siThMode    = { 0, 0, NULL, -1 };	/*!< \brief Threshold mode status info. */
static STATUS_INFO siRankFilt  = { 0, 0, NULL, -1 };	/*!< \brief Rank filter status info. */
static STATUS_INFO siMorph = { 0, 0, NULL, -1 };	/*!< \brief Closing status info. */
static STATUS_INFO siSegType   = { 0, 0, NULL, -1 };	/*!< \brief Segmentation type status info. */

/*-------------------------------------------------------------*/
static int ocr_mode = -1; //READ_MODE;					/*!< \brief OCR mode */

static bool g_bLoadImage        = false;				/*!< \brief Load image flag. */
static bool g_bShowPreprocImage = false;				/*!< \brief Display preprocessed image flag. */
static bool g_bShowBinImage     = false;				/*!< \brief Display the binary image flag. */
static bool g_bTriggerModeSet   = false;				/*!< \brief Trigger mode flag. */
static bool g_bQuality          = false;				/*!< \brief Display Quality flag. */
static bool g_bROI              = false;				/*!< \brief Use ROI flag. */
static bool g_bRunLive          = false;				/*!< \brief Run loop flag. */
static bool g_bCpOne2One        = true;					/*!< \brief Copy image with same size flag. */

/*-------------------------------------------------------------*/
static pghandle wApp       = 0;					/*!< \brief Application widget handle. */
static pghandle wTrigMode  = 0;					/*!< \brief Trigger mode check box widget handle. */
static pghandle wLive      = 0;					/*!< \brief \b Run button widget handle. */
static pghandle wStop      = 0;					/*!< \brief \b Stop button widget handle. */
static pghandle wRead      = 0;					/*!< \brief \b Read button widget handle. */
static pghandle wTeach     = 0;					/*!< \brief \b Teach button widget handle. */
static pghandle wDelete    = 0;					/*!< \brief \b Delete button widget handle. */
static pghandle wChboxROI  = 0;					/*!< \brief \b ROI check box widget handle. */
static pghandle wSettings  = 0;					/*!< \brief \b Settings pop up button widget handle. */
static pghandle wPreproc   = 0;					/*!< \brief Preprocessing pop up button widget handle. */
static pghandle wImage     = 0;					/*!< \brief \b Image pop up button widget handle. */
static pghandle wDisplay   = 0;					/*!< \brief \b Display pop up button widget handle. */
static pghandle wROIBox    = 0;					/*!< \brief Widget handle of the box needed for the ROI check box. */
static pghandle wStatLine  = 0;					/*!< \brief Top status line widget handle. */
static pghandle wStatLine2 = 0;					/*!< \brief Bottom status line widget handle. */

/*-------------------------------------------------------------*/
static SSCROLLER g_scrThreshold = {0,0,0};		/*!< \brief Threshold scroller. */
static SSCROLLER g_scrOPLen     = {0,0,0};		/*!< \brief Operator length scroller. */
static SSCROLLER g_scrMinDist   = {0,0,0};		/*!< \brief Minimal distance scroller. */

/*-------------------------------------------------------------*/
static DEV_HANDLE    g_deviceHandle    = IPL_DONT_CARE;		/*!< \brief Device handle. */
static IPL_BUFHANDLE g_imageHandle     = IPL_DONT_CARE;		/*!< \brief Capture image handle. */
static IPL_BUFHANDLE g_chldImageHandle = IPL_DONT_CARE;		/*!< \brief Childe image handle. */
static IPL_BUFHANDLE g_loadImageHandle = IPL_DONT_CARE;		/*!< \brief Loaded image handle. */

/*-------------------------------------------------------------*/
static unsigned char* g_pLDBuffer = NULL;		/*!< \brief Pointer to the loaded image data. */
static long g_lLDWidth  = 0;					/*!< \brief Width of the loaded image. */
static long g_lLDHeight = 0;					/*!< \brief Height of the loaded image. */
static long g_lLDPitch  = 0;					/*!< \brief Pitch of the loaded image. */

static unsigned char* pImage = NULL;			/*!< \brief Pointer to the captured image data. */
static int g_ImgWidth  = 0;						/*!< \brief Width of the captured image. */
static int g_ImgHeight = 0;						/*!< \brief Height of the captured image. */
static int g_ImgPitch  = 0;						/*!< \brief Pitch of the captured image. */

/*-------------------------------------------------------------*/
static long g_SegCount = 0;					/*!< \brief Number of found segments. */
static int g_TriggerMode = -1;					/*!< \brief Trigger mode. */
static double frame_ps = 0.;					/*!< \brief Frames per seconds. */

static IPL_RECT g_roi = {10,100,10,100};		/*!< \brief ROI rect. */

static START_MODE g_ProcessType = START_ONLY;	/*!< \brief Start application process type. */

static THTESHOLD_TYPE g_OldThType = (THTESHOLD_TYPE)-1;	/*!< \brief Remember the last set threshold mode. */

/*-------------------------------------------------------------*/
static bool g_bSendData = false;				/*!< \brief Send data flag. */
static int g_Baudrate   = 115200;				/*!< \brief Baudrate to send the data. */
static int g_DevNo      = 1;					/*!< \brief Serial device number. */

/*-------------------------------------------------------------*/
static CDmoDisplay*    g_pDisp = NULL;				/*!< \brief Display class instance. */
static CDmoOCR*        g_pOCR  = NULL;				/*!< \brief OCR class instance. */
static CDmoPreproc*    g_pPp   = NULL;				/*!< \brief Preprocessing class instance. */
static CDmoRWFile*     g_pRWF  = NULL;				/*!< \brief Read/Write file class instance. */
static CSendDatattySX* g_pSend = NULL;				/*!< \brief Send data class instance. */

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*! \brief Arrow bitmap: 11x6 1bpp PBM file */
static unsigned char g_pArrowBits[] = {
0x50, 0x34, 0x0A, 0x0A, 0x31, 0x31, 0x20, 0x36, 0x0A,
0xFF, 0xE0,
0x7F, 0xC0,
0x3F, 0x80,
0x1F, 0x00,
0x0E, 0x00,
0x04, 0x00,
};
#define ARROW_LEN		21

static pghandle g_wArrowMask = 0;		/*!< \brief Arrow bitmap mask widget handle. */
static pghandle g_wArrow = 0;			/*!< \brief Arrow bitmap widget handle. */

//-------------------------------------------------------------------
/*! \brief Exit the application after a mvIMPACT error
 *  \param pMsg error message
 *  \param errCode error code. */
static void iplExitDemo( const char* pMsg, const long errCode )
{
	char msg[512];
	memset( msg, 0, 512 );

	sprintf( msg, "*** " TOOL_PREFIX ": %s - %s! err: %ld!!\n", pMsg, IPL_GetErrorString( errCode ), errCode );

	fprintf( stderr, msg );
	pgMessageDialog( "Fatal Error!!", msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	evtClose( NULL );
	exit( errCode );
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*! \brief Make a arrow bitmap.
 *  \param wArrowMask arrow bitmap mask
 *  \param wArrow arrow bitmap */
static void mkArrow( pghandle& wArrowMask, pghandle& wArrow )
{
	dprintf((" ++ %s\n", __FUNCTION__ ));

	wArrowMask = pgNewBitmap( pgFromMemory( g_pArrowBits, ARROW_LEN ));
	wArrow = pgCreateBitmap( 11, 6 );
	pgRender( wArrow, PG_GROP_SETCOLOR, MVCOLOR_BLACK );
	pgRender( wArrow, PG_GROP_RECT, 0, 0, 15, 8 );

	dprintf((" -- %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Set the arrow bitmap to a button widget.
 *  \param wButton handle of the button widget */
static void setArrow( const pghandle wButton )
{
	if( wButton > 0 )
		pgSetWidget( wButton,
					PG_WP_EXTDEVENTS, PG_EXEV_PNTR_DOWN,
					PG_WP_BITMAP, g_wArrow,
					PG_WP_BITMASK, g_wArrowMask,
					PG_WP_BITMAPSIDE, PG_S_RIGHT,
					0 );
}

//-------------------------------------------------------------------
/*! \brief Delete a scroller.
 *  \param scr the scroller */
static void deleteScroller( SSCROLLER& scr )
{
	widget_delete( &scr.wScrollLab);
	widget_delete( &scr.wScroller );
	widget_delete( &scr.wScrollBox );
}

//-------------------------------------------------------------------
/*! \brief Update a scroller.
 *  \param scr the scroller */
static void updateScroller( SSCROLLER scr )
{
	if( scr.wScrollLab )
		pgSubUpdate( scr.wScrollLab );
	if( scr.wScroller )
		pgSubUpdate( scr.wScroller );
	if( scr.wScrollBox )
		pgSubUpdate( scr.wScrollBox );
}

//-------------------------------------------------------------------
/*! \brief Make a new scroller.
 *  \param scr the scroller
 *  \param wParent the parent handle of the new scroller box
 *  \param position the position relative to the parent widget
 *  \param size the size of the scroller
 *  \param value the default value of the scroller
 *  \param pName the name of the scroller (label)
 *  \param evtCb the event callback funktion.
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
static int mkScroller( SSCROLLER& scr, const pghandle wParent, const int position,
			const int size, const int value, char* pName, pgevthandler evtCb )
{
	int ret = IPL_TRUE;

	DMO_CHECK_ERR( wParent > 0, IPL_HANDLE_ERROR );
	DMO_CHECK_ERR( size > 0 , -1 );
	DMO_CHECK_ERR( value > 0 && value < size, -1 );

	deleteScroller( scr );

	scr.wScrollBox = mk_wBox( wParent, position, PG_S_LEFT, 80 );
	scr.wScroller = mk_scroller( PGDEFAULT, PG_DERIVE_INSIDE, size, value, evtCb );

	if( pName )
		scr.wScrollLab = mk_scroller_lable( scr.wScrollBox, PG_DERIVE_AFTER, pName );

	return ret;
}

//-------------------------------------------------------------------
/*! \brief Make the manual threshold scroller.
 *  \param wParent the parent handle of the new scroller box
 *  \param position the position relative to the parent widget
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
static int mkManualThScroller( const pghandle wParent, const int position )
{
	int ret = IPL_TRUE;

	DMO_CHECK_ERR( wParent > 0, IPL_HANDLE_ERROR );
	DMO_CHECK_ERR( mkScroller( g_scrThreshold, wParent, position,
				255, g_pPp->m_threshold, "Threshold", evtThresholdScroller ), -1 );

	updateScroller( g_scrThreshold );
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Make the dynamic line threshold scroller.
 *  \param wParent the parent handle of the new scroller box
 *  \param position the position relative to the parent widget
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
static int mkDynThScroller( const pghandle wParent, const int position )
{
	int ret = IPL_TRUE;

	DMO_CHECK_ERR( wParent > 0, IPL_HANDLE_ERROR );
	DMO_CHECK_ERR( mkScroller( g_scrOPLen, wParent, position, g_ImgWidth-1,
									g_pPp->m_opLen, "OPLen", evtOPLen ), -1);
	DMO_CHECK_ERR( mkScroller( g_scrMinDist, g_scrOPLen.wScrollLab, PG_DERIVE_AFTER, 255,
									g_pPp->m_minDist, "MinDist", evtMinDist ), -1 );

	updateScroller( g_scrOPLen );
	updateScroller( g_scrMinDist );
	return ret;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*! \brief Write the settings to the \c INI file.
 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
static int write2file( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret;

	/* start the writing */
	DMO_CHECK_ERR(( ret = g_pRWF->startWriteToFile( OCRTOOL_INIT )), -1 );

	/* write globale settings */
	DMO_CHECK_ERR(( ret = g_pRWF->writeSection( "GLOBALS" )), -1 );
	WI2FL( PROC, g_ProcessType );
	DMO_CHECK_ERR(( ret = g_pRWF->writeNewLine( )), -1 );

	/* write OCR processing settings */
	DMO_CHECK_ERR(( ret = g_pRWF->writeCommentLine( )), -1 );
	DMO_CHECK_ERR(( ret = g_pRWF->writeSection( "OCRSETTINGS" )), -1 );
	WL2FL( WMIN,    g_pOCR->m_WMin );
	WL2FL( WMAX,    g_pOCR->m_WMax );
	WL2FL( HMIN,    g_pOCR->m_HMin );
	WL2FL( HMAX,    g_pOCR->m_HMax );
	WI2FL( POL,     g_pOCR->m_bgrd );
	WI2FL( DISPQ,   g_bQuality );
	WI2FL( QMIN,    g_pOCR->m_MinQuality );
	WI2FL( SEGTYPE, g_pOCR->m_SegType );
	WI2FL( CLSM,    g_pOCR->m_ClsM );
	WL2FL( WMINC,   g_pOCR->m_WMinC );
	WL2FL( WMAXC,   g_pOCR->m_WMaxC );
	WL2FL( HMINC,   g_pOCR->m_HMinC );
	WL2FL( HMAXC,   g_pOCR->m_HMaxC );
	DMO_CHECK_ERR(( ret = g_pRWF->writeNewLine( )), -1 );

	/* write preprocessing settings */
	DMO_CHECK_ERR(( ret = g_pRWF->writeCommentLine( )), -1 );
	DMO_CHECK_ERR(( ret = g_pRWF->writeSection( "PREPPROCESSING" )), -1 );
	WI2FL( THTYPE,  g_pPp->m_ThType );
	WL2FL( TH,      g_pPp->m_threshold );
	WI2FL( RANK,    g_pPp->m_RankType );
	WI2FL( MORPH,   g_pPp->m_MorphType );
	WI2FL( OPLEN,   g_pPp->m_opLen );
	WI2FL( MINDIST, g_pPp->m_minDist );
	DMO_CHECK_ERR(( ret = g_pRWF->writeNewLine( )), -1 );

	/* write ROI settings */
	DMO_CHECK_ERR(( ret = g_pRWF->writeCommentLine( )), -1 );
	DMO_CHECK_ERR(( ret = g_pRWF->writeSection( "ROI" )), -1 );
	WI2FL( ROI,        g_bROI );
	WL2FL( ROI_LEFT,   g_roi.left );
	WL2FL( ROI_TOP,    g_roi.top );
	WL2FL( ROI_RIGHT,  g_roi.right );
	WL2FL( ROI_BOTTOM, g_roi.bottom );
	DMO_CHECK_ERR(( ret = g_pRWF->writeNewLine( )), -1 );

	/* write send data settings */
	DMO_CHECK_ERR(( ret = g_pRWF->writeCommentLine( )), -1 );
	DMO_CHECK_ERR(( ret = g_pRWF->writeSection( "SENDDATA" )), -1 );
	WI2FL( SENDDATA, g_bSendData );
	WI2FL( BAUDRATE, g_Baudrate );
	WI2FL( DEVNO,    g_DevNo );

	/* stop writing */
	DMO_CHECK_ERR(( ret = g_pRWF->stopWriteToFile( )), -1 );

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Read the settings from the \c INI file if found.
 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
static int initFromFile( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int ret = IPL_TRUE;
	/* */
	if( !g_pRWF->startReadFile( OCRTOOL_INIT ))
		return ret;

	/* get int values */
	GIV( PROC,     (int)START_ONLY, (int)START_READ,      (int&)g_ProcessType );
	GIV( POL,      BGRD_WHITE,      BGRD_BLACK,           g_pOCR->m_bgrd );
	GIV( QMIN,     0,               100,                  g_pOCR->m_MinQuality );
	GIV( THTYPE,   (int)TH_MANUAL,  (int)TH_DYN_LINE,     (int&)g_pPp->m_ThType );
	GIV( RANK,     (int)RANK_NONE,  (int)RANK_HORIZONTAL, (int&)g_pPp->m_RankType );
	GIV( MORPH,    (int)MORPH_NONE, (int)MORPH_CLOSE,     (int&)g_pPp->m_MorphType );
	GIV( SEGTYPE,  (int)SEG_NONE,   (int)SEG_MERGE_SPLIT, (int&)g_pOCR->m_SegType );
	GIV( CLSM,     (int)CLSM_MATCH, (int)CLSM_FOURIER,    (int&)g_pOCR->m_ClsM );
	GIV( OPLEN,    1,               g_ImgWidth,            g_pPp->m_opLen );
	GIV( MINDIST,  1,               255,                   g_pPp->m_minDist );
	GIV( BAUDRATE, 0,               230400,                g_Baudrate );
	GIV( DEVNO,    0,               1,                     g_DevNo );

	/* get long values */
	GLV( WMIN,       MIN_W, g_ImgWidth,  g_pOCR->m_WMin );
	GLV( WMAX,       MIN_W, g_ImgWidth,  g_pOCR->m_WMax );
	GLV( HMIN,       MIN_H, g_ImgHeight, g_pOCR->m_HMin  );
	GLV( HMAX,       MIN_H, g_ImgHeight, g_pOCR->m_HMax );
	GLV( ROI_LEFT,   0,     g_ImgWidth,  g_roi.left );
	GLV( ROI_TOP,    0,     g_ImgHeight, g_roi.top );
	GLV( ROI_RIGHT,  1,     g_ImgWidth,  g_roi.right );
	GLV( ROI_BOTTOM, 1,     g_ImgHeight, g_roi.bottom );
	GLV( TH,         0,     255,         g_pPp->m_threshold );
	GLV( WMINC,      MIN_W, g_ImgWidth,  g_pOCR->m_WMinC );
	GLV( WMAXC,      MIN_W, g_ImgWidth,  g_pOCR->m_WMaxC );
	GLV( HMINC,      MIN_H, g_ImgHeight, g_pOCR->m_HMinC );
	GLV( HMAXC,      MIN_H, g_ImgHeight, g_pOCR->m_HMaxC );

	/* get bool values */
	GBV( DISPQ,    g_bQuality );
	GBV( ROI,      g_bROI );
	GBV( SENDDATA, g_bSendData );

	/* */
	DMO_CHECK_ERR(( ret = g_pRWF->stopReadFile( )), -1 );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return ret;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*! \brief Clean a status info
 *  \param si the status info */
static void cleanSIWidget( STATUS_INFO& si )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	widget_delete( &si.wText );
	widget_delete( &si.wBox );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Free the loaded image buffer */
static void freeLoadedImage( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( g_pLDBuffer != NULL )
	{
		IPL_ReleasePtr( g_loadImageHandle, IPL_LOCK_READ, 0, 0, 0, g_pLDBuffer );
		g_pLDBuffer = NULL;
	}

	if( g_loadImageHandle != IPL_DONT_CARE )
	{
		IPL_BufFree( g_loadImageHandle );
		g_loadImageHandle = IPL_DONT_CARE;
	}
	g_lLDWidth  = 0;
	g_lLDHeight = 0;
	g_lLDPitch  = 0;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Close and free everything. */
static void close_all( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( pImage != NULL )
		IPL_ReleasePtr( g_imageHandle, IPL_LOCK_READ, 0, 0, 0, pImage );

	if( g_deviceHandle != IPL_DONT_CARE )
    	DEV_Close( g_deviceHandle );

	if( g_imageHandle != IPL_DONT_CARE )
		IPL_BufFree( g_imageHandle );

	freeLoadedImage( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Replace threshold status info message. */
static void ReplThSI( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Threshold: %3ld", g_pPp->m_threshold );
	siThreshold.msg = msg;
	repl_txt_si( siThreshold );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Replace operator lenght status info message. */
 static void ReplOpLenSI( )
 {
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "OPLen: %4d", g_pPp->m_opLen );
	siOPLen.msg = msg;
	repl_txt_si( siOPLen );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Replace preprocessing status info messages. */
 static void ReplPreprocSI( )
 {
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];

	/* */
	memset( msg, 0, MSG_SIZE );
	switch( g_pPp->m_ThType )
	{
	case TH_MANUAL:		sprintf( msg, "Th: " MANUAL_STR );		break;
	case TH_STAT_BEST:	sprintf( msg, "Th: " STATBEST_TH_STR );	break;
	case TH_DYN_LINE:	sprintf( msg, "Th: " DYNLINE_TH_STR );	break;
	}

	siThMode.msg = msg;
	repl_txt_si( siThMode );

	/* */
	memset( msg, 0, MSG_SIZE );
	switch( g_pPp->m_RankType )
	{
	case RANK_NONE:			sprintf( msg, "Rank: " NONE_STR );		break;
	case RANK_HV:			sprintf( msg, "Rank: " VH_RANK_STR );	break;
	case RANK_VERTICAL:		sprintf( msg, "Rank: " VERT_RANK_STR );	break;
	case RANK_HORIZONTAL:	sprintf( msg, "Rank: " HOR_RANK_STR );	break;
	}

	siRankFilt.msg = msg;
	repl_txt_si( siRankFilt );

	/* */
	memset( msg, 0, MSG_SIZE );
	switch( g_pPp->m_MorphType )
	{
	case MORPH_NONE:	sprintf( msg, "Morph: " NONE_STR );			break;
	case MORPH_OPEN:	sprintf( msg, "Morph: " OPEN_MORPH_STR );	break;
	case MORPH_CLOSE:	sprintf( msg, "Morph: " CLOSE_MORPH_STR );	break;
	}
	siMorph.msg = msg;
	repl_txt_si( siMorph );

	/* */
	memset( msg, 0, MSG_SIZE );
	switch( g_pOCR->m_SegType )
	{
	case SEG_NONE:			sprintf( msg, "Seg.Type: " NONE_STR );		break;
	case SEG_SPLIT:			sprintf( msg, "Seg.Type: " SPLIT_SEG_STR );	break;
	case SEG_MERGE:			sprintf( msg, "Seg.Type: " MERGE_SEG_STR );	break;
	case SEG_SPLIT_MERGE:	sprintf( msg, "Seg.Type: " SM_SEG_STR );	break;
	case SEG_MERGE_SPLIT:	sprintf( msg, "Seg.Type: " MS_SEG_STR );	break;
	}
	siSegType.msg = msg;
	repl_txt_si( siSegType );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Use last loaded image message.
 *  \return \c PG_MSGBTN_YES or \c PG_MSGBTN_NO will be returned. */
static int useLastLoadImage( )
{
	return pgMessageDialog( "Load", "\nReload last loaded image?\n", PG_MSGBTN_YES | PG_MSGBTN_NO /*| PG_MSGBTN_CANCEL*/ );
}

//-------------------------------------------------------------------
/*! \brief Copy the loaded image to the display.
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
static int cp2Display( )
{
	int ret = IPL_FALSE;

	if( g_bCpOne2One )
	{
		DMO_CHECK_ERR(( ret = IPL_PtBufCopy( g_loadImageHandle, &g_imageHandle, 0xffff, 0xffff, 0 )), IPL_GetErrorCode2( ));
	}
	else
	{
		if( !g_pLDBuffer )
		{
			DMO_CHECK_ERR(( ret = IPL_GetPtr( g_loadImageHandle, IPL_LOCK_READ, 0, 0, 0, &g_lLDPitch, &g_pLDBuffer )), IPL_GetErrorCode2( ));
		}

		memset( pImage, 0, g_ImgHeight * g_ImgPitch );
		cp_chimg( g_pLDBuffer, pImage, 0, 0, g_lLDWidth, g_lLDHeight, g_lLDPitch, g_ImgPitch );
	}

	return ret;
}

//-------------------------------------------------------------------
/*! \brief Copy the binary image
 *  \return \c 0 will be returned. */
static int cpBinImage( )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int ret = IPL_TRUE;

	if( g_bShowBinImage )
	{
		DMO_CHECK_ERR(( ret = IPL_PtBufCopy( g_pPp->m_binImageHandle, g_chldImageHandle != IPL_DONT_CARE ? &g_chldImageHandle : &g_imageHandle, 0xffff, 0xffff, 0 )),
							IPL_GetErrorCode2( ));
	}
	else
	{
		if( g_pPp->m_ppImageHandle != IPL_DONT_CARE )
			DMO_CHECK_ERR(( ret = IPL_PtBufCopy( g_pPp->m_ppImageHandle, g_chldImageHandle != IPL_DONT_CARE ? &g_chldImageHandle : &g_imageHandle, 0xffff, 0xffff, 0 )),
								IPL_GetErrorCode2( ));
	}

	dprintf(( " -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return ret;
}

//-------------------------------------------------------------------
/*! \brief Save the snapped image as a file.
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
int saveImage( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	int ret = IPL_TRUE;

	DMO_CHECK_ERR( g_imageHandle != IPL_DONT_CARE, -1 );

	char* pName = NULL;
	const char* pFile = NULL;

	DMO_CHECK_ERR(( pFile = pgFilePicker( NULL, NULL, TOOL_PREFIX BMP_IMG_SUFIX, PG_FILESAVE, "Save the file" )) != NULL, -1 );

	int sufixLen = 0;
	if( strstr( pFile, BMP_IMG_SUFIX ) == NULL && strstr( pFile, ".BMP" ) == NULL )
		sufixLen = strlen( BMP_IMG_SUFIX );

	int nameSize = sufixLen + strlen(pFile) + 1;
	DMO_CHECK_ERR(( pName = new char[nameSize]) != NULL, -1 );
	memset( pName, 0, nameSize );
	if( sufixLen )
		sprintf( pName, "%s" BMP_IMG_SUFIX, pFile );
	else
		sprintf( pName, "%s", pFile );

	ret = IPL_FlExport( g_imageHandle, IPL_DONT_CARE, IPL_BMP, pName );

	if( pName ) delete [] pName;

	dprintf((" -- " TOOL_PREFIX ": %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief OCR teach
 *  \param imageHandle the handle of the used image
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
static int teach_ocr( const IPL_BUFHANDLE imageHandle )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - g_SegCount: %ld\n", __FUNCTION__, g_SegCount ));

	int w, h;
	int c = -1, ret;
	unsigned char *chimg_buffer = NULL;
	pgcolor color = MVCOLOR_RED;

	pgEnterContext( );

	g_pDisp->mkOOIs( g_SegCount );

	int xOffset = g_bROI ? g_roi.left : 0;
	int yOffset = g_bROI ? g_roi.top  : 0;

	for( int i = 0; i < g_SegCount; i++ )
	{
		dprintf((" == " TOOL_PREFIX ": %s - txtpos: %d, l/t: %ld/%ld, r/b: %ld/%ld\n", __FUNCTION__,
						i, g_pOCR->m_Rect[i].left, g_pOCR->m_Rect[i].top, g_pOCR->m_Rect[i].right, g_pOCR->m_Rect[i].bottom ));

		/* */
		g_pDisp->drawAOI( i, g_pOCR->m_Rect[i], color, xOffset, yOffset );

		/* */
		w = g_pOCR->m_Rect[i].right - g_pOCR->m_Rect[i].left;
		h = g_pOCR->m_Rect[i].bottom - g_pOCR->m_Rect[i].top;

		if(( chimg_buffer = (unsigned char*) malloc( w * h )) != NULL )
		{
			long imgPitch = 0;
			unsigned char* pData = NULL;
			DMO_CHECK_ERR(( ret = IPL_GetPtr( imageHandle, IPL_LOCK_READ, 0, 0, 0, &imgPitch, &pData )), IPL_GetErrorCode2( ));

			cp_chimg( pData, chimg_buffer, g_pOCR->m_Rect[i].left, g_pOCR->m_Rect[i].top, w, h, g_ImgPitch, w );
// 			cp_chimg( pData, chimg_buffer, xOffset+g_pOCR->m_Rect[i].left, yOffset+g_pOCR->m_Rect[i].top, w, h, g_ImgPitch, w );

			DMO_CHECK_ERR(( ret = IPL_ReleasePtr( imageHandle, IPL_LOCK_READ, 0, 0, 0, pData )), IPL_GetErrorCode2( ));


			if( !g_pOCR->readChar( imageHandle, i ))
// 			if( !g_pOCR->readChar( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle, i ))
			{
				char emsg[256];
				memset( emsg, 0, 256 );
				sprintf( emsg, "Read failed at char: %d >>%c<< - err: %ld!!\n", i, g_pOCR->m_ReadText[i], IPL_GetErrorCode2( ));
				dprintf(( "*** " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, emsg ));
// 				pgMessageDialog( "Error", emsg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
			}

			ret = get_chars( &c, chimg_buffer, w, h, g_pOCR->m_Result );
			free( chimg_buffer );

			if( ret == RET_CLOSE )
				break;
			else if( ret != RET_SKIPP && ret != RET_ERROR )
			{
				dprintf((" == " TOOL_PREFIX ": %s - i: %d, txt: %c (%d)\n", __FUNCTION__, i, c, c ));

				sprintf( g_pOCR->m_ReadText, "%c", c );

				g_pDisp->drawText( i, g_pOCR->m_Rect[i].left  + w/2 + xOffset,
									g_pOCR->m_Rect[i].bottom + 7 + yOffset, g_pOCR->m_ReadText, color );

				if( !g_pOCR->teachChar( imageHandle, i, c ))
				{
					char emsg[256];
					memset( emsg, 0, 256 );
					sprintf( emsg, "Teach failed - err: %ld!!\n", IPL_GetErrorCode2( ));
					dprintf(( "*** " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, emsg ));
					pgMessageDialog( "Error", emsg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
				}
			}
		}
	}
	pgLeaveContext( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return IPL_TRUE;
}

//-------------------------------------------------------------------
/*! \brief Send data on the serial port.
 *  \param pData data to be send */
static void sendData( const char* pData )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

	if( g_pSend && pData )
	{
		int inLen = strlen( pData );
		if( inLen > 0 )
		{
			char* pPrefix = "Data:>>";
			char* pSufix  = "<<\n";
			int newSize = inLen + strlen(pPrefix) + strlen(pSufix) + 1;

			char* pTmp = new char[newSize];
			memset( pTmp, 0, newSize );
			sprintf( pTmp, "%s%s%s", pPrefix, pData, pSufix );

			g_pSend->Writen( pTmp, strlen( pTmp ));

			if( pTmp ) delete [] pTmp;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
}

//-------------------------------------------------------------------
/*! \brief Draw the sements on the GUI */
static void draw_segments( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%s: %d) - g_SegCount: %ld\n", __FUNCTION__, __FILE__, __LINE__, g_SegCount ));
	int disp_txt_flag = 0;
	char str[MSG_SIZE];
	pgcolor color = MVCOLOR_RED;

	if( g_pOCR->m_bgrd == BGRD_WHITE )
		color = MVCOLOR_BLUE;
	else if( g_pOCR->m_bgrd == BGRD_BLACK )
		color = MVCOLOR_GREEN;

	if( g_SegCount > 0 )
	{
		memset( str, 0, MSG_SIZE );

		pgEnterContext( );
		g_pDisp->mkOOIs( g_SegCount );

		int xOffset = g_bROI ? g_roi.left : 0;
		int yOffset = g_bROI ? g_roi.top  : 0;
		pgcolor drawColor;

		dprintf((" == " TOOL_PREFIX ": %s - xOffset: %d yOffset: %d\n", __FUNCTION__, xOffset, yOffset ));
		for( int i = 0; i < g_SegCount; i++ )
		{
			if( g_pOCR->m_Quality[i] >= g_pOCR->m_MinQuality )
			{
				drawColor = color;
			}
			else
			{
				drawColor = MVCOLOR_RED;
			}

			dprintf((" == " TOOL_PREFIX ": %s - txtpos: %d, l/t: %ld/%ld, r/b: %ld/%ld\n", __FUNCTION__,
							i, g_pOCR->m_Rect[i].left, g_pOCR->m_Rect[i].top, g_pOCR->m_Rect[i].right, g_pOCR->m_Rect[i].bottom ));
			g_pDisp->drawAOI( i, g_pOCR->m_Rect[i], drawColor, xOffset, yOffset );

			if( g_bQuality )
				sprintf( str,"%c (%d%%)", g_pOCR->m_ReadText[i], g_pOCR->m_Quality[i] );
			else
				sprintf( str,"%c", g_pOCR->m_ReadText[i] );

			g_pDisp->drawText( i, g_pOCR->m_Rect[i].left + xOffset,
								g_pOCR->m_Rect[i].bottom + 7 + yOffset, str, drawColor );
			disp_txt_flag = 1;
		}

		if( disp_txt_flag )
		{
			g_pDisp->drawText( -1, g_ImgWidth/2, 20, g_pOCR->m_DisplayText, color );

			sendData( g_pOCR->m_DisplayText );
		}

		pgLeaveContext( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
}

//-------------------------------------------------------------------
/*! \brief Get the sements from the OCR
 *  \param imageHandle the handle of the used image */
static void get_segment( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	ReplThSI( );
	g_pOCR->setThershold( g_pPp->m_threshold );
	dprintf((" == " TOOL_PREFIX ": %s - threshold: %3ld\n", __FUNCTION__, g_pPp->m_threshold ));

	g_SegCount = g_pOCR->getSegments( g_pPp->m_binImageHandle );

	dprintf((" == " TOOL_PREFIX ": %s (%s: %d) - SegCount: %ld\n", __FUNCTION__, __FILE__, __LINE__,  g_SegCount ));

	if( g_pOCR->m_SegType != SEG_MERGE_SPLIT )
	{
		/* Split large segments */
		g_SegCount = g_pOCR->splitSeg( g_pPp->m_ppImageHandle != IPL_DONT_CARE ? g_pPp->m_ppImageHandle :
						( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle ));

		dprintf((" == " TOOL_PREFIX ": %s (%s: %d) - SegCount: %ld\n", __FUNCTION__, __FILE__, __LINE__,  g_SegCount ));

		/* Merge small segments */
		g_SegCount = g_pOCR->mergeSeg( g_pPp->m_ppImageHandle != IPL_DONT_CARE ? g_pPp->m_ppImageHandle :
						( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle ));
	}
	else if( g_pOCR->m_SegType == SEG_MERGE_SPLIT )
	{
		/* Merge small segments */
		g_SegCount = g_pOCR->mergeSeg( g_pPp->m_ppImageHandle != IPL_DONT_CARE ? g_pPp->m_ppImageHandle :
						( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle ));

		dprintf((" == " TOOL_PREFIX ": %s (%s: %d) - SegCount: %ld\n", __FUNCTION__, __FILE__, __LINE__,  g_SegCount ));

		/* Split large segments */
		g_SegCount = g_pOCR->splitSeg( g_pPp->m_ppImageHandle != IPL_DONT_CARE ? g_pPp->m_ppImageHandle :
						( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle ));
	}

	dprintf((" == " TOOL_PREFIX ": %s - SegCount: %ld, (MAX_OBJ_CNT: %d)\n",
			__FUNCTION__, g_SegCount, MAX_OBJ_CNT ));

	if( g_SegCount > MAX_OBJ_CNT )
	{
		char emsg[256];
		memset( emsg, 0, 256 );
		sprintf( emsg, "To many segments found - max: %ld!!\n", g_SegCount );
		dprintf(( "*** " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, emsg ));
		pgMessageDialog( "Error", emsg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

		g_SegCount = 0;
	}
	else
	{
		char msg[MSG_SIZE];
		memset( msg, 0, MSG_SIZE );
		sprintf( msg, "Seg:  %3ld", g_SegCount );
		siSegCnt.msg = msg;
		repl_txt_si( siSegCnt );
	}

	dprintf((" -- " TOOL_PREFIX ": %s (g_SegCount: %ld)\n", __FUNCTION__, g_SegCount ));
}

//-------------------------------------------------------------------
/*! \brief Delete the childe image */
static void delChldImage( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( g_chldImageHandle != IPL_DONT_CARE )
	{
		IPL_BufFree( g_chldImageHandle );
		g_chldImageHandle = IPL_DONT_CARE;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Check the ROI position in the image.
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
static int checkROIPos( )
{
	int ret = IPL_TRUE;
#if 1
	if( g_roi.left < 0 || g_roi.left >= g_ImgWidth )
		g_roi.left = 0;

	if( g_roi.top < 0 || g_roi.top >= g_ImgHeight )
		g_roi.top = 0;

	if( g_roi.right < 0 || g_roi.right >= g_ImgWidth )
		g_roi.right = g_ImgWidth;

	if( g_roi.bottom < 0 || g_roi.bottom >= g_ImgHeight )
		g_roi.bottom = g_ImgHeight;

#else
	if( g_roi.left < 0 )
		ret = IPL_FALSE;
	else if( g_roi.left >= g_ImgWidth )
		ret = IPL_FALSE;

	if( g_roi.top < 0 )
		ret = IPL_FALSE;
	else if( g_roi.top >= g_ImgHeight )
		ret = IPL_FALSE;

	if( g_roi.right < 0 )
		ret = IPL_FALSE;
	else if( g_roi.right >= g_ImgWidth )
		ret = IPL_FALSE;

	if( g_roi.bottom < 0 )
		ret = IPL_FALSE;
	else if( g_roi.bottom >= g_ImgHeight )
		ret = IPL_FALSE;
#endif
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Make a child image depending on the ROI size
 *  \return If succeeds \c IPL_TRUE will be returned else \c IPL_FALSE . */
static int mkChldImage( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = IPL_TRUE;

	g_roi = g_pDisp->getROI( );

	dprintf((" == " TOOL_PREFIX ": %s - %ld %ld %ld %ld\n",
				__FUNCTION__, g_roi.left, g_roi.top, g_roi.right, g_roi.bottom ));

	if(( ret = checkROIPos( )))
	{
		delChldImage( );


		long lXSize = abs( g_roi.right - g_roi.left );
		long lYSize = abs( g_roi.bottom - g_roi.top );
		if( lXSize && lYSize )
		{
			long lWidth, lHeight;
			IPL_BufInquire( g_imageHandle, IPL_XSIZE, &lWidth );
			IPL_BufInquire( g_imageHandle, IPL_YSIZE, &lHeight );
			dprintf((" == " TOOL_PREFIX ": %s - xs: %ld ys: %ld w: %ld h: %ld\n",
						__FUNCTION__, lXSize, lYSize, lWidth, lHeight ));

			long lXPos = g_roi.right  > g_roi.left ? g_roi.left : g_roi.right;
			long lYPos = g_roi.bottom > g_roi.top  ? g_roi.top  : g_roi.bottom;

			if( lXPos + lXSize >= lWidth )
				lXSize = lWidth - lXPos - 1;

			if( lYPos + lYSize >= lHeight )
				lYSize = lHeight - lYSize - 1;

			dprintf((" == " TOOL_PREFIX ": %s - xp: %ld yp: %ld xs: %ld ys: %ld w: %ld h: %ld\n",
						__FUNCTION__, lXPos, lYPos, lXSize, lYSize, lWidth, lHeight ));

			DMO_CHECK_ERR( IPL_BufAllocChild2D( &g_chldImageHandle, g_imageHandle,
							lXPos, lYPos, 0, 0, 0, lXSize, lYSize ), IPL_GetErrorCode2());
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Transform the timeval struct data into milliseconds
 *  \return The time in milliseconds will be returned. */
double getTime_ms( struct timeval tv )
{
	return (double)(tv.tv_sec * 1000 + tv.tv_usec/1000. );
}

//-------------------------------------------------------------------
/*! \brief The main operating loop. */
static void snap_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	static struct timeval start_tv, diff_tv;
	static struct timeval procStart_tv, procDiff_tv;
	char msg[MSG_SIZE];
	int ret = IPL_TRUE;

	while( g_bRunLive )
	{
		pgEventPoll( );

		StartTimer( &start_tv );

		g_SegCount = 0;
		if( !g_bLoadImage && g_bRunLive )
		{
			if( !DEV_Grab( g_deviceHandle, &g_imageHandle ))
				iplExitDemo( "grab image failed", IPL_GetErrorCode2( ));
		}
		else if( g_bRunLive )
		{
			cp2Display( );
		}

		if( g_bRunLive && g_bROI )
		{
			ret = mkChldImage( );
		}
		else
		{
			ret = IPL_TRUE;
		}

		StartTimer( &procStart_tv );
		if( g_bRunLive && ret )
		{
			g_pPp->Preprocess( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle, g_pOCR->m_bgrd );
		}

		switch( ocr_mode )
		{
		case READ_MODE:
			if( g_bRunLive )
			{
				if( ret )
				{
					get_segment( );
					if( g_SegCount )
						g_pOCR->readOCR( g_pPp->m_ppImageHandle != IPL_DONT_CARE ? g_pPp->m_ppImageHandle :
										( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle ));
				}
			}
			break;
		}

		ElapsedTime( &procStart_tv, &procDiff_tv, 30 );

		if(( g_bShowPreprocImage || g_bShowBinImage ) && g_bRunLive )
		{
			cpBinImage( );
		}

		draw_segments( );
		g_pDisp->displayImage( );

		ElapsedTime( &start_tv, &diff_tv, 30 );

		/* */
		frame_ps = 1000.0/getTime_ms( diff_tv );
		dprintf((" == " TOOL_PREFIX ": frame_ps: %.2f\n", frame_ps ));
		memset( msg, 0, MSG_SIZE );
		sprintf( msg, "Frames:  %.2f", frame_ps );
		siFPS.msg = msg;
		repl_txt_si( siFPS );

		/* */
		memset( msg, 0, MSG_SIZE );
		sprintf( msg, "Proc:     %12.2f ms", getTime_ms( procDiff_tv ));
		siProc.msg = msg;
		repl_txt_si( siProc );

		g_pDisp->delOOIS( );

		if( ret < 0 )
			g_bRunLive = false;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Close the application event callback
 *  \return \c 0 will be returned. */
int evtClose( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	write2file( );
	pgExitEventLoop( );
	close_all( );

	if( g_pDisp )  delete g_pDisp;
	if( g_pOCR )   delete g_pOCR;
	if( g_pPp )    delete g_pPp;
	if( g_pRWF )   delete g_pRWF;
	if( g_pSend )  delete g_pSend;

	printf(" == " TOOL_PREFIX ": %s - exit...   CU\n", __FUNCTION__ );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//------------------------------- ------------------------------------
/*! \brief Stop the operating mode event callback
 *  \return \c 0 will be returned. */
int evtLiveOff(  struct  pgEvent* /*pEvt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );
	set_w_color( wRead, MVCOLOR_BLACK );

	g_bRunLive = false;

	ocr_mode = -1;
	siOCRMode.msg = NO_MODE_STR;
	repl_txt_si( siOCRMode );

	siRunMode.msg = STOP_LIVE_STR;
	repl_txt_si( siRunMode );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Start the operating mode event callback
 *  \return \c 0 will be returned. */
int evtLiveOn( struct pgEvent* /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	set_w_color( wLive, MVCOLOR_GREY );
	set_w_color( wStop, MVCOLOR_BLACK );

	g_bRunLive = true;
	siRunMode.msg = START_LIVE_STR;
	repl_txt_si( siRunMode );
	pgUpdate( );
	snap_img( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Teach event callback
 *  \return \c 0 will be returned. */
int evtTeach( struct pgEvent* /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int kb_pid = open_keybrd( );
	usleep(KB_WAIT_TIME);

	set_w_color( wRead, MVCOLOR_BLACK );
	set_w_color( wTeach, MVCOLOR_GREY );
	set_w_color( wDelete, MVCOLOR_BLACK );

	g_bRunLive = false;
	ocr_mode = TEACH_MODE;
	siOCRMode.msg = TEACH_MODE_STR;
	repl_txt_si( siOCRMode );

	g_pOCR->resetOCR( );

	if( g_imageHandle == IPL_DONT_CARE )
	{
		if( !g_bLoadImage )
		{
			if( !DEV_Grab( g_deviceHandle, &g_imageHandle ))
				iplExitDemo( "grab image failed", IPL_GetErrorCode2( ));
		}
	}

	g_pPp->Preprocess( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle, g_pOCR->m_bgrd );
	get_segment( );

	if( g_SegCount > 0 )
		teach_ocr( g_pPp->/*m_binImageHandle*/m_ppImageHandle != IPL_DONT_CARE ? g_pPp->m_ppImageHandle :
					( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle ));
	else
		pgMessageDialog( "Teach error...", "No characters found!\n\nPlease check the settings:\n  * Threshold\n  * white on black\n  * width(min/max)\n  * height(min/max)\n\nYou can try to read in\nlive mode and change the\nsettings!\n", PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	g_pOCR->closeOCR( );

	siOCRMode.msg = NO_MODE_STR;
	repl_txt_si( siOCRMode );
	set_w_color( wTeach, MVCOLOR_BLACK );
	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );

	close_keybrd( kb_pid );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
int chlst4eq( int *list, int cnt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int n_cnt = 0, found_flag = 0;
	int *n_list = NULL;

	DMO_CHECK_ERR( cnt > 0, -1 );
	DMO_CHECK_ERR( list != NULL, -1 );

	DMO_CHECK_ERR(( n_list = new int [cnt] ) != NULL, -1 );
	memset( n_list, 0, cnt * sizeof( int ));

	n_list[n_cnt++] = list[0];

	for( int i = 1; i < cnt; i++ )
	{
		for( int j = 0; j < n_cnt; j++ )
		{
			if( n_list[j] == list[i] )
			{
				found_flag = 1;
				break;
			}
			else
				found_flag = 0;
		}

		if( !found_flag )
			n_list[n_cnt++] = list[i];
	}

	memset( list, 0, cnt );
	dprintf((" == " TOOL_PREFIX ": %s - n_cnt: %d, cnt: %d\n", __FUNCTION__, n_cnt, cnt ));

	for( int i = 0; i < n_cnt; i++ )
	{
		list[i] = n_list[i];
		dprintf((" == " TOOL_PREFIX ": %s - n_list[%d]: %c (%d)\n", __FUNCTION__, i, list[i], list[i] ));
	}

	if( n_list != NULL )	delete [] n_list;

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, n_cnt ));

	return n_cnt;
}

//-------------------------------------------------------------------
/*! \brief Delete characters event callback
 *  \return \c 0 will be returned. */
int evtDelete( struct pgEvent* /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int kb_pid = open_keybrd( );

	sleep( 1 );
	//usleep(KB_WAIT_TIME);

	OCR_PATTERN pat;
	int new_cnt = 0, old_cnt = 0, c_cnt = 0, tmp_cnt = 0;
	int c_list[MAX_CHAR_SIZE], *pCntList = NULL;
	char msg[2*MAX_CHAR_SIZE], s[MSG_SIZE];
	memset( c_list, 0, MAX_CHAR_SIZE );
	memset( msg, 0, 2*MAX_CHAR_SIZE );
	memset( s, 0, MSG_SIZE );

	set_w_color( wRead, MVCOLOR_BLACK );
	set_w_color( wTeach, MVCOLOR_BLACK );
	set_w_color( wLive, MVCOLOR_BLACK );
	set_w_color( wStop, MVCOLOR_GREY );
	set_w_color( wDelete, MVCOLOR_GREY );

	g_bRunLive = false;
	ocr_mode = DELETE_MODE;
	siOCRMode.msg = DELETE_MODE_STR;
	repl_txt_si( siOCRMode );

	g_pOCR->resetOCR( );

	c_cnt = get2delchar( c_list );

	dprintf((" == " TOOL_PREFIX ": %s - c_cnt: %d\n", __FUNCTION__, c_cnt ));
	if( c_cnt != RET_CLOSE && c_cnt != RET_ERROR )
	{
		c_cnt = chlst4eq( c_list, c_cnt );
		dprintf((" == " TOOL_PREFIX ": %s - new c_cnt: %d\n", __FUNCTION__, c_cnt ));

		DMO_CHECK_ERR(( pCntList = new int[c_cnt] ) != NULL, -1 );
		memset( pCntList, 0, c_cnt * sizeof( int ));

		old_cnt = g_pOCR->getPatternNo( );

		dprintf((" == " TOOL_PREFIX ": %s - old_cnt: %d\n", __FUNCTION__, old_cnt ));
		tmp_cnt = old_cnt;

		for( int c = 0; c < c_cnt; c++ )
		{
			pCntList[c] = 0;
			dprintf((" == " TOOL_PREFIX ": %s - delete all '%c'\n", __FUNCTION__, c_list[c] ));

			int i = 0;
			while ( i < tmp_cnt )
			{
				g_pOCR->getPattern( i, pat );
				dprintf(( " == " TOOL_PREFIX ": %s - del:%3d '%c'(%p) list: '%c'\n", __FUNCTION__, i, pat.ascii, pat.data, c_list[c] ));

				if ( pat.ascii != c_list[c] )
					i++;
				else
				{
					dprintf(( " == " TOOL_PREFIX ": %s - deleteOCR :%3d (0x%p)\n", __FUNCTION__, i, pat.data ));

					if( !g_pOCR->deleteOCR( i ))
					{
						char emsg[256];
						memset( emsg, 0, 256 );
						sprintf( emsg, "Delete failed - err: %ld!!\n", IPL_GetErrorCode2( ));
						dprintf(( "*** " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, emsg ));
						pgMessageDialog( "Error", emsg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
					}

					tmp_cnt = g_pOCR->getPatternNo( );
					i = 0;
					pCntList[c]++;
				}
			}
		}

		new_cnt = g_pOCR->getPatternNo( );
		dprintf((" == " TOOL_PREFIX ": %s - new_cnt: %d\n", __FUNCTION__, new_cnt ));

		sprintf( s, "%d characters deleted!\n", old_cnt - new_cnt );
		strncat( msg, s, strlen( s ));

		for( int c = 0; c < c_cnt; c++ )
		{
			memset( s, 0, MSG_SIZE );
			sprintf( s, "'%c': %5d\n", c_list[c], pCntList[c] );
			strncat( msg, s, strlen( s ) );
		}

		pgMessageDialog("Delete", msg , PG_MSGBTN_OK );

		if( pCntList )
		{
			delete [] pCntList;
			pCntList = NULL;
		}
	}

	set_w_color( wDelete, MVCOLOR_BLACK );
	siOCRMode.msg = NO_MODE_STR;
	repl_txt_si( siOCRMode );

	g_pOCR->closeOCR( );

	close_keybrd( kb_pid );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Check if the OCR file exists */
static void ch4ocrfl( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	FILE *f = NULL;

	bool bHaveSegm = false;

	if(( f = fopen( DEFAULT_OCR_FILE, "r" )) != NULL )
	{
		fseek( f, 0, SEEK_END );
		long f_size = ftell( f );
		fseek( f, 0, SEEK_SET );
		fclose( f );

		if( f_size > 0 )
			bHaveSegm = true;
	}

	if( !bHaveSegm )
		pgMessageDialog( "NOTE", "\nAdjust the threshold until you get the\nsegments and click then to the\n'TEACH' button!!\n", PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief GUI segmentation type pop up menue
 *  \return \c 0 will be returned. */
static int guiSegType( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cSplit segments|%cMerge segments|%cSplit & merge segments|%cMerge & Split segments| Reset",
			getStatusC( g_pOCR->m_SegType == SEG_SPLIT ),
			getStatusC( g_pOCR->m_SegType == SEG_MERGE ),
			getStatusC( g_pOCR->m_SegType == SEG_SPLIT_MERGE ),
			getStatusC( g_pOCR->m_SegType == SEG_MERGE_SPLIT ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 5 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:
		/* click outside*/
		break;
	case 1:
	case 2:
	case 3:
	case 4:
		g_pOCR->m_SegType = (SEG_TYPE)ret;
		break;
	case 5:
		g_pOCR->m_SegType = SEG_NONE;
		g_pPp->ResetPPHandle( );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Depending on the threshold moder the new scroller will be draw. */
static void drawNewThScroller( )
{
	dprintf(( " ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	if( g_OldThType != g_pPp->m_ThType )
	{
		switch( g_pPp->m_ThType )
		{
		case TH_DYN_LINE:
			deleteScroller( g_scrThreshold );
			mkDynThScroller( wROIBox, PG_DERIVE_AFTER );
			break;

		case TH_MANUAL:
			deleteScroller( g_scrOPLen );
			deleteScroller( g_scrMinDist );
			mkManualThScroller( wROIBox, PG_DERIVE_AFTER );
			break;

		default:
			deleteScroller( g_scrThreshold );
			deleteScroller( g_scrOPLen );
			deleteScroller( g_scrMinDist );
			break;
		}

		g_OldThType = g_pPp->m_ThType;
	}

	dprintf(( " == %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
}
//-------------------------------------------------------------------
/*! \brief GUI threshold type pop up menue
 *  \return \c 0 will be returned. */
static int guiThresholdType( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cStat. best threshold|%cDyn. line threshold|%cManual threshold| Reset",
			getStatusC( g_pPp->m_ThType == TH_STAT_BEST ),
			getStatusC( g_pPp->m_ThType == TH_DYN_LINE ),
			getStatusC( g_pPp->m_ThType == TH_MANUAL ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 4 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:			/* case click outside*/			break;
	case 1:		g_pPp->m_ThType = TH_STAT_BEST;		break;
	case 2:		g_pPp->m_ThType = TH_DYN_LINE;		break;
	case 3:		g_pPp->m_ThType = TH_MANUAL;		break;
	case 4:		g_pPp->m_ThType = TH_STAT_BEST;		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief GUI rank filter type pop up menue
 *  \return \c 0 will be returned. */
static int guiRankType( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cVert. & Horiz.|%cVertical|%cHorizontal| Reset",
			getStatusC( g_pPp->m_RankType == RANK_HV ),
			getStatusC( g_pPp->m_RankType == RANK_VERTICAL ),
			getStatusC( g_pPp->m_RankType == RANK_HORIZONTAL ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 4 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:
		/* click outside*/
		break;
	case 1:
	case 2:
	case 3:
		g_pPp->m_RankType = (RANK_TYPE)ret;
		break;
	case 4:
		g_pPp->m_RankType = RANK_NONE;
		g_pPp->ResetPPHandle( );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief GUI morphological filter type pop up menue
 *  \return \c 0 will be returned. */
static int guiMorphType( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cOpen|%cClose| Reset",
			getStatusC( g_pPp->m_MorphType == MORPH_OPEN ),
			getStatusC( g_pPp->m_MorphType == MORPH_CLOSE ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 3 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:
		/* click outside*/
		break;
	case 1:
	case 2:
		g_pPp->m_MorphType = (MORPH_TYPE)ret;
		break;
	case 3:
		g_pPp->m_MorphType = MORPH_NONE;
		g_pPp->ResetPPHandle( );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief GUI classification methode pop up menue
 *  \return \c 0 will be returned. */
static int guiClassificationMethod( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cMatch|%cFourier descriptors",
			getStatusC( g_pOCR->m_ClsM == CLSM_MATCH ),
			getStatusC( g_pOCR->m_ClsM == CLSM_FOURIER ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 2 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:
		/* click outside*/
		break;
	case 1:
		g_pOCR->m_ClsM = CLSM_MATCH;
		break;
	case 2:
		g_pOCR->m_ClsM = CLSM_FOURIER;
		break;
	}

	if( ret > 0 )
	{
		g_pOCR->resetOCR( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Replace the polarity status info message. */
static void setPolaritySI( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%s: %d) - m_bgrd: %d BGRD_BLACK: %d BGRD_WHITE: %d\n",
			__FUNCTION__, __FILE__, __LINE__, g_pOCR->m_bgrd, BGRD_BLACK, BGRD_WHITE ));

	if( g_pOCR->m_bgrd == BGRD_BLACK )
	{
		siPolarity.msg = W2B_STR;
		repl_txt_si( siPolarity );
	}
	else if( g_pOCR->m_bgrd == BGRD_WHITE )
	{
		siPolarity.msg = B2W_STR;
		repl_txt_si( siPolarity );
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
}

//-------------------------------------------------------------------
/*! \brief GUI polarity pop up menue
 *  \return \c 0 will be returned. */
static int guiPolarity( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cBlack on white|%cWhite on black",
			getStatusC( !g_pOCR->m_bgrd ),
			getStatusC( g_pOCR->m_bgrd ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 2 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:		/* click outside*/
		break;
	case 1:
		g_pOCR->m_bgrd = BGRD_WHITE;
		break;
	case 2:
		g_pOCR->m_bgrd = BGRD_BLACK;
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set minimal width
 *  \param type size type which will be changed
 *  \return \c 0 will be returned. */
static int setWMin( const SIZETYPE type )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	long value;

	switch( type )
	{
	case ST_SEGMENT:
		value = set_min_max( " min seg. width", g_pOCR->m_WMin, MIN_W, g_ImgWidth );
		dprintf((" == " TOOL_PREFIX ": %s - seg. wmin: %ld\n", __FUNCTION__, value ));
		g_pOCR->setWMin( value );
		break;
	case ST_CHARACTER:
		value = set_min_max( " min char width", g_pOCR->m_WMinC, MIN_W, g_ImgWidth );
		dprintf((" == " TOOL_PREFIX ": %s - char wmin: %ld\n", __FUNCTION__, value ));
		g_pOCR->setMinCharW( value );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set maximal segment width
 *  \param type size type which will be changed
 *  \return \c 0 will be returned. */
int setWMax( const SIZETYPE type )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	long value;

	switch( type )
	{
	case ST_SEGMENT:
		value = set_min_max( " max seg. width", g_pOCR->m_WMax, MIN_W, g_ImgWidth );
		dprintf((" == " TOOL_PREFIX ": %s - seg. wmax: %ld\n", __FUNCTION__, value ));
		g_pOCR->setWMax( value );
		break;
	case ST_CHARACTER:
		value = set_min_max( " max char width", g_pOCR->m_WMaxC, MIN_W, g_ImgWidth );
		dprintf((" == " TOOL_PREFIX ": %s - char wmax: %ld\n", __FUNCTION__, value ));
		g_pOCR->setMaxCharW( value );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set minimal segment height
 *  \param type size type which will be changed
 *  \return \c 0 will be returned. */
static int setHMin( const SIZETYPE type )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	long value;

	switch( type )
	{
	case ST_SEGMENT:
		value = set_min_max( " min seg. height", g_pOCR->m_HMin, MIN_H, g_ImgHeight );
		dprintf((" == " TOOL_PREFIX ": %s - seg. hmin: %ld\n", __FUNCTION__, value ));
		g_pOCR->setHMin( value );
		break;
	case ST_CHARACTER:
		value = set_min_max( " min char height", g_pOCR->m_HMinC, MIN_H, g_ImgHeight );
		dprintf((" == " TOOL_PREFIX ": %s - char hmin: %ld\n", __FUNCTION__, value ));
		g_pOCR->setMinCharH( value );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Set maximal segment height
 *  \param type size type which will be changed
 *  \return \c 0 will be returned. */
static int setHMax( const SIZETYPE type )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	long value;

	switch( type )
	{
	case ST_SEGMENT:
		value = set_min_max( " max seg. height", g_pOCR->m_HMax, MIN_H, g_ImgHeight );
		dprintf((" == " TOOL_PREFIX ": %s - seg. hmax: %ld\n", __FUNCTION__, value ));
		g_pOCR->setHMax( value );
		break;
	case ST_CHARACTER:
		value = set_min_max( " max char height", g_pOCR->m_HMaxC, MIN_H, g_ImgHeight );
		dprintf((" == " TOOL_PREFIX ": %s - char hmax: %ld\n", __FUNCTION__, value ));
		g_pOCR->setMaxCharH( value );
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Update the segment size status informations
 *  \return \c 0 will be returned. */
static int setSegmentSI( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "W(min): %3ld", g_pOCR->m_WMin );
	siSegMinW.msg = msg;
	repl_txt_si( siSegMinW );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "W(max): %3ld", g_pOCR->m_WMax );
	siSegMaxW.msg = msg;
	repl_txt_si( siSegMaxW );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "H(min): %3ld", g_pOCR->m_HMin );
	siSegMinH.msg = msg;
	repl_txt_si( siSegMinH );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "H(max): %3ld", g_pOCR->m_HMax );
	siSegMaxH.msg = msg;
	repl_txt_si( siSegMaxH );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Update the character size status informations
 *  \return \c 0 will be returned. */
static int setCharacterSI( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	dprintf((" == " TOOL_PREFIX ": %s - %ld %ld %ld %ld\n", __FUNCTION__,
			g_pOCR->m_WMinC, g_pOCR->m_WMaxC, g_pOCR->m_HMinC, g_pOCR->m_HMaxC ));

	char msg[MSG_SIZE];

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C minW: %3ld", g_pOCR->m_WMinC );
	siCharMinW.msg = msg;
	repl_txt_si( siCharMinW );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C maxW: %3ld", g_pOCR->m_WMaxC );
	siCharMaxW.msg = msg;
	repl_txt_si( siCharMaxW );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C minH: %3ld", g_pOCR->m_HMinC );
	siCharMinH.msg = msg;
	repl_txt_si( siCharMinH );

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C maxH: %3ld", g_pOCR->m_HMaxC );
	siCharMaxH.msg = msg;
	repl_txt_si( siCharMaxH );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Make the GUI to change the segment or character size
 *  \param type size type which will be changed
 *  \return \c 0 will be returned. */
static int guiSCSize( const SIZETYPE type )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "min width|max width|min height|max height" );

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 4 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:	/* click outside*/	break;
	case 1: setWMin( type );	break;
	case 2: setWMax( type );	break;
	case 3: setHMin( type );	break;
	case 4: setHMax( type );	break;
	}
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Make the GUI to change the segment size
 *  \return \c 0 will be returned. */
static int guiSegSize( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cAuto|%cManual->",
			getStatusC( g_pOCR->m_bAutoSegmentation ), getStatusC( !g_pOCR->m_bAutoSegmentation ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 2 );

	switch( ret )
	{
	case 0:	/* click outside*/
		pgLeaveContext( );
		break;
	case 1:
		g_pOCR->m_bAutoSegmentation = true;
		pgLeaveContext( );
/*++++ rm after implemented ++++*/
		NotImplemented();
		g_pOCR->m_bAutoSegmentation = false;
/*---- rm after implemented ----*/
		break;
	case 2:
		guiSCSize( ST_SEGMENT );
		pgLeaveContext( );
		g_pOCR->m_bAutoSegmentation = false;
		break;
	}
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*! \brief Read event callback
 *  \return \c 0 will be returned. */
int evtRead( struct pgEvent *evt )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	set_w_color( wRead, MVCOLOR_GREY );
	set_w_color( wTeach, MVCOLOR_BLACK );
	set_w_color( wDelete, MVCOLOR_BLACK );

	ocr_mode = READ_MODE;
	g_bRunLive = true;
	siOCRMode.msg = READ_MODE_STR;
	repl_txt_si( siOCRMode );

	ch4ocrfl( );
	g_pOCR->resetOCR( );

	evtLiveOn( evt );

	dprintf(( " -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Threshold scroller event callback
 *  \return \c 0 will be returned. */
int evtThresholdScroller( struct pgEvent* pEvt )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	g_pPp->m_threshold = pEvt->e.param;

	ReplThSI( );

	dprintf(( " == " TOOL_PREFIX ": %s - threshold: %ld\n", __FUNCTION__, g_pPp->m_threshold ));

	if( g_bShowPreprocImage && g_bLoadImage )
	{
		if( g_pPp->mkBinary( g_chldImageHandle != IPL_DONT_CARE ? g_chldImageHandle : g_imageHandle, g_pOCR->m_bgrd ))
		{
			cpBinImage( );
			g_pDisp->displayImage( );
		}
	}

	dprintf(( " -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Operator length scroller event callback
 *  \return \c 0 will be returned. */
int evtOPLen( struct pgEvent* pEvt )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	g_pPp->m_opLen = pEvt->e.param;

	if( g_pPp->m_opLen == 0 )
		g_pPp->m_opLen = 1;

	if( g_chldImageHandle != IPL_DONT_CARE )
	{
		long imgWidth;
		IPL_BufInquire( g_chldImageHandle, IPL_XSIZE, &imgWidth );

		if( g_pPp->m_opLen >= imgWidth )
			g_pPp->m_opLen = imgWidth - 1;
	}
	else
	{
		if( g_pPp->m_opLen >= g_ImgWidth )
			g_pPp->m_opLen = g_ImgWidth - 1;
	}

	/* */
	ReplOpLenSI( );

	dprintf(( " -- " TOOL_PREFIX ": %s (opLen: %d)\n", __FUNCTION__, g_pPp->m_opLen ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Minimal distance scroller event callback
 *  \return \c 0 will be returned. */
int evtMinDist( struct pgEvent* pEvt )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	g_pPp->m_minDist = pEvt->e.param;

	if( g_pPp->m_minDist == 0 )
		g_pPp->m_minDist = 1;

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "MinDist: %5d", g_pPp->m_minDist );
	siMinDist.msg = msg;
	repl_txt_si( siMinDist );

	dprintf(( " -- " TOOL_PREFIX ": %s (minDist: %d)\n", __FUNCTION__, g_pPp->m_minDist ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Enable trigger mode checkbox event callback
 *  \return \c 0 will be returned. */
int chTrigModeOnOff( struct pgEvent * /*evt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	g_bTriggerModeSet = pgGetWidget( wTrigMode, PG_WP_ON );

	if( g_bTriggerModeSet )
	{
		if( !DEV_ControlCaps( g_deviceHandle, DEV_TRIGGER, (IPL_VOID*)&g_TriggerMode ))
		{
			char emsg[256];
			memset( emsg, 0, 256 );
			sprintf( emsg, "Set the trigger mode failed (%ld)!!\n", IPL_GetErrorCode2( ));
			dprintf(( "*** " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, emsg ));
			pgMessageDialog( "Error", emsg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
		}
	}
	else
	{
		int tm = TRIG_SW;
		if( !DEV_ControlCaps( g_deviceHandle, DEV_TRIGGER, (IPL_VOID*)&tm ))
		{
			char emsg[256];
			memset( emsg, 0, 256 );
			sprintf( emsg, "Set the SW trigger mode failed (%ld)!!\n", IPL_GetErrorCode2( ));
			dprintf(( "*** " TOOL_PREFIX ": %s - %s\n", __FUNCTION__, emsg ));
			pgMessageDialog( "Error", emsg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Load an image event callback
 *  \return \c 0 will be returned. */
int evtLoad( struct pgEvent* pEvt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	evtLiveOff( pEvt );

	siOCRMode.msg = NO_MODE_STR;
	repl_txt_si( siOCRMode );

	set_w_color( wRead, MVCOLOR_BLACK );
	set_w_color( wTeach, MVCOLOR_BLACK );
	set_w_color( wDelete, MVCOLOR_BLACK );

	if( g_loadImageHandle != IPL_DONT_CARE )
	{
		if( useLastLoadImage( ) == PG_MSGBTN_YES )
		{
			g_bLoadImage = true;
			g_pDisp->displayImage( );
			pgUpdate();
			return 0;
		}
		else
			freeLoadedImage( );
	}

	const char* pFile = NULL;
	DMO_CHECK_ERR(( pFile = pgFilePicker( NULL, NULL, DUMMY_BMP_FILE, PG_FILE_FIELD, "Load a BMP file...")) != NULL, -1 );

	if( strstr( pFile, ".bmp") == NULL && strstr( pFile, ".BMP") == NULL )
	{
		pgMessageDialog("Load error", "Wrong file selected!!\n Please enter a *.bmp file!!\n", PG_MSGBTN_OK );
		return 0;
	}

	DMO_CHECK_ERR( IPL_FlImport( &g_loadImageHandle, NULL, FL_DONT_CARE, pFile ), IPL_GetErrorCode2( ));
	DMO_CHECK_ERR( IPL_BufInquire( g_loadImageHandle, IPL_XSIZE, &g_lLDWidth ), IPL_GetErrorCode2( ));
	DMO_CHECK_ERR( IPL_BufInquire( g_loadImageHandle, IPL_YSIZE, &g_lLDHeight ), IPL_GetErrorCode2( ));

	if( g_lLDWidth == g_ImgWidth && g_lLDHeight == g_ImgHeight )
	{
		g_bCpOne2One = true;
		DMO_CHECK_ERR( cp2Display( ), -1 );
		g_bLoadImage = true;
	}
	else if( g_lLDWidth <= g_ImgWidth && g_lLDHeight <= g_ImgHeight )
	{
		g_bCpOne2One = false;
		DMO_CHECK_ERR( cp2Display( ), -1 );
		g_bLoadImage = true;
	}
	else
	{
		char msg[256];
		memset( msg, 0, 256 );
		sprintf( msg, "Wrong image size!\nThe image will not be loaded!\nmax: %d x %d\n", g_ImgWidth, g_ImgHeight );
		pgMessageDialog( "LOAD error...", msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	}

	if( g_bLoadImage )
		g_pDisp->displayImage( );

	if( g_pPp->m_ppImageHandle != IPL_DONT_CARE )
	{
		IPL_BufFree( g_pPp->m_ppImageHandle );
		g_pPp->m_ppImageHandle = IPL_DONT_CARE;
	}

	dprintf((" == %s - g_bLoadImage: %d\n", __FUNCTION__, g_bLoadImage ));

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Enable ROI check box event callback
 *  \return \c 0 will be returned. */
int evtChboxROI( struct pgEvent* /*pEvt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

	if(( g_bROI = pgGetWidget( wChboxROI, PG_WP_ON )))
	{
		g_pDisp->drawROI( );
	}
	else
	{
		g_pDisp->delROI( );
		delChldImage( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Settings pop up button event callback
 *  \return \c 0 will be returned. */
int evtSettings( struct pgEvent* /*pEvt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "Polarity->|Segment size->|Character size->|Classification type->" );

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 4 );

	switch( ret )
	{
	case 0:	pgLeaveContext( ); /* click outside*/						break;
	case 1:	guiPolarity( ); pgLeaveContext( ); setPolaritySI( );		break;
	case 2:	guiSegSize( );  pgLeaveContext( ); setSegmentSI( );			break;
// 	case 2:	guiSCSize( ST_SEGMENT );   pgLeaveContext( ); setSegmentSI( );		break;
	case 3:	guiSCSize( ST_CHARACTER ); pgLeaveContext( ); setCharacterSI( );	break;
	case 4:	guiClassificationMethod(); pgLeaveContext( );				break;
	}

	if( ret > 0 && ocr_mode == READ_MODE )
		g_pOCR->resetOCR( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Preprocessing pop up button event callback
 *  \return \c 0 will be returned. */
int evtPreproc( struct pgEvent* /*pEvt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, " Threshold type->| Rank filter->| Morphological filter->| Segmantation type->| Reset" );
// 	sprintf( msg, " Threshold type->| Rank filter->|%cClosing| Segmantation type->| Reset",
// 			getStatusC( g_pPp->m_bClosing ));

	pgEnterContext( );

	switch( mkPopupMenu( 0, msg, 5 ))
	{
	case 0:	/* click outside*/									break;
	case 1:	guiThresholdType( );								break;
	case 2:	guiRankType( );										break;
// 	case 3:	g_pPp->m_bClosing = negate( g_pPp->m_bClosing );	break;
	case 3:	guiMorphType( );	break;
	case 4:	guiSegType( );										break;
	case 5:	g_pPp->ResetPP( );
			g_pOCR->m_SegType = SEG_NONE;
			g_OldThType = (THTESHOLD_TYPE)(g_pPp->m_ThType+1);
			break;
	}

	pgLeaveContext( );

	drawNewThScroller( );
	ReplPreprocSI( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Image pop up button event callback
 *  \return \c 0 will be returned. */
int evtImage( struct pgEvent* pEvt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cLive|%cLoad| Save",
				getStatusC( !g_bLoadImage ), getStatusC( g_bLoadImage ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 3 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:	/* click outside*/		break;
	case 1:	g_bLoadImage = false;	break;
	case 2:	evtLoad( pEvt );		break;
	case 3:	saveImage( );			break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Display pop up button event callback
 *  \return \c 0 will be returned. */
int evtDisplay( struct pgEvent* /*pEvt*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "%cBinary image|%cPreprocessed image|%cRead quality| Reset",
				getStatusC( g_bShowBinImage ), getStatusC( g_bShowPreprocImage ), getStatusC( g_bQuality ));

	pgEnterContext( );
	int ret = mkPopupMenu( 0, msg, 4 );
	pgLeaveContext( );

	switch( ret )
	{
	case 0:	/* click outside*/										break;
	case 1:	g_bShowBinImage = true; g_bShowPreprocImage = false;	break;
	case 2:	g_bShowBinImage = false; g_bShowPreprocImage = true;	break;
	case 3:	g_bQuality      = negate( g_bQuality );					break;
	case 4: g_bShowBinImage = false; g_bShowPreprocImage = false; g_bQuality = false; break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
/*! \brief Get the size of the strings. */
static void get_s_len(  )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int y;
	pghandle font = pgNewFont( NULL, 8, 0 );

	siRunMode.msg = STOP_LIVE_STR;
	get_str_size( &siRunMode.msg_len, &y, font, siRunMode.msg );

	siOCRMode.msg = NO_MODE_STR;
	get_str_size( &siOCRMode.msg_len, &y, font, siOCRMode.msg );

	siPolarity.msg = B2W_STR;
	get_str_size( &siPolarity.msg_len, &y, font, siPolarity.msg );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make the top status line entries.
 *  \param wParent the handle of the status line. */
static void mkStatLineLabel( const pghandle wParent )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	/* minimal allowed quality status info */
	cleanSIWidget( siMinQuality );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Q(min):  %3d%%", g_pOCR->m_MinQuality );
	siMinQuality.msg = msg;
	mk_stll_si( wParent, PG_DERIVE_INSIDE, &siMinQuality );

	/* */
	cleanSIWidget( siSegMinW );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "W(min):  %3ld", g_pOCR->m_WMin );
	siSegMinW.msg = msg;
	mk_stll_si( siMinQuality.wBox, PG_DERIVE_AFTER, &siSegMinW );

	/* */
	cleanSIWidget( siSegMaxW );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "W(max):  %3ld", g_pOCR->m_WMax );
	siSegMaxW.msg = msg;
	mk_stll_si( siSegMinW.wBox, PG_DERIVE_AFTER, &siSegMaxW );

	/* */
	cleanSIWidget( siSegMinH );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "H(min):  %3ld", g_pOCR->m_HMin );
	siSegMinH.msg = msg;
	mk_stll_si( siSegMaxW.wBox, PG_DERIVE_AFTER, &siSegMinH );

	/* */
	cleanSIWidget( siSegMaxH );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "H(max):  %3ld", g_pOCR->m_HMax );
	siSegMaxH.msg = msg;
	mk_stll_si( siSegMinH.wBox, PG_DERIVE_AFTER, &siSegMaxH );

	/* */
	cleanSIWidget( siPolarity );
	mk_stll_si( siSegMaxH.wBox, PG_DERIVE_AFTER, &siPolarity );

	/* */
	cleanSIWidget( siThreshold );
	sprintf( msg, "Threshold: %3ld", g_pPp->m_threshold );
	siThreshold.msg = msg;
	mk_stll_si( siPolarity.wBox, PG_DERIVE_AFTER, &siThreshold );

	/* */
	cleanSIWidget( siOPLen );
	sprintf( msg, "OPLen: %4d", g_pPp->m_opLen );
	siOPLen.msg = msg;
	mk_stll_si( siThreshold.wBox, PG_DERIVE_AFTER, &siOPLen );

	/* */
	cleanSIWidget( siMinDist );
	sprintf( msg, "MinDist: %5d", g_pPp->m_minDist );
	siMinDist.msg = msg;
	mk_stll_si( siOPLen.wBox, PG_DERIVE_AFTER, &siMinDist );

	/* */
	cleanSIWidget( siCharMinW );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C minW:  %3ld", g_pOCR->m_WMinC );
	siCharMinW.msg = msg;
	mk_stll_si( siMinDist.wBox, PG_DERIVE_AFTER, &siCharMinW );

	/* */
	cleanSIWidget( siCharMaxW );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C maxW:  %3ld", g_pOCR->m_WMaxC );
	siCharMaxW.msg = msg;
	mk_stll_si( siCharMinW.wBox, PG_DERIVE_AFTER, &siCharMaxW );

	/* */
	cleanSIWidget( siCharMinH );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C minH:  %3ld", g_pOCR->m_HMinC );
	siCharMinH.msg = msg;
	mk_stll_si( siCharMaxW.wBox, PG_DERIVE_AFTER, &siCharMinH );

	/* */
	cleanSIWidget( siCharMaxH );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "C maxH:  %3ld", g_pOCR->m_HMaxC );
	siCharMaxH.msg = msg;
	mk_stll_si( siCharMinH.wBox, PG_DERIVE_AFTER, &siCharMaxH );

	/* */
	setPolaritySI( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make the bottom status line entries.
 *  \param wParent the handle of the status line. */
void mkStatLine2Label( const pghandle wParent )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	cleanSIWidget( siRunMode );
	mk_stll_si( wParent, PG_DERIVE_INSIDE, &siRunMode );

	cleanSIWidget( siOCRMode );
	mk_stll_si( siRunMode.wBox, PG_DERIVE_AFTER, &siOCRMode );

	cleanSIWidget( siSegCnt );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Seg:  %3ld", g_SegCount );
	siSegCnt.msg = msg;
	mk_stll_si( siOCRMode.wBox, PG_DERIVE_AFTER, &siSegCnt );

	cleanSIWidget( siFPS );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Frames:   %.2f", frame_ps );
	siFPS.msg = msg;
	mk_stll_si( siSegCnt.wBox, PG_DERIVE_AFTER, &siFPS );

	cleanSIWidget( siProc );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Proc:     %12.2f ms", 0. );
	siProc.msg = msg;
	mk_stll_si( siFPS.wBox, PG_DERIVE_AFTER, &siProc );

	cleanSIWidget( siThMode );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Th: " STATBEST_TH_STR );
	siThMode.msg = msg;
	mk_stll_si( siProc.wBox, PG_DERIVE_AFTER, &siThMode );

	cleanSIWidget( siRankFilt );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Rank: " VH_RANK_STR );
	siRankFilt.msg = msg;
	mk_stll_si( siThMode.wBox, PG_DERIVE_AFTER, &siRankFilt );

	cleanSIWidget( siMorph );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Morph: " CLOSE_MORPH_STR );
	siMorph.msg = msg;
	mk_stll_si( siRankFilt.wBox, PG_DERIVE_AFTER, &siMorph );

	cleanSIWidget( siSegType );
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "Seg.Type: " SM_SEG_STR );
	siSegType.msg = msg;
	mk_stll_si( siMorph.wBox, PG_DERIVE_AFTER, &siSegType );

	ReplPreprocSI( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make the GUI buttons, check boxes and scroller.
 *  \param wParent handle for the parent widget
 *  \param position position relative to the parent widget.
 *  \return The handle of the widget box in which the controlles are made will be returned. */
pghandle mkControls( const pghandle wParent, const int position )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	/* make the arrow for the popup buttons */
	mkArrow( g_wArrowMask, g_wArrow );

	/* make a box for the buttons & controlls */
	pghandle wBox = pgNewWidget( PG_WIDGET_BOX, position, wParent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIZE, 30,
				0 );

	/* make buttons */
	wLive     = mk_ctrl_btn(      wBox, "Run",      PG_DERIVE_INSIDE, &evtLiveOn,   MVCOLOR_BLACK );
	wStop     = mk_ctrl_btn( PGDEFAULT, "Stop",     PG_DERIVE_AFTER,  &evtLiveOff,  MVCOLOR_BLACK );
	wTeach    = mk_ctrl_btn( PGDEFAULT, "Teach",    PG_DERIVE_AFTER,  &evtTeach,    MVCOLOR_BLACK );
	wRead     = mk_ctrl_btn( PGDEFAULT, "Read",     PG_DERIVE_AFTER,  &evtRead,     MVCOLOR_BLACK );
	wDelete   = mk_ctrl_btn( PGDEFAULT, "Delete",   PG_DERIVE_AFTER,  &evtDelete,   MVCOLOR_BLACK );
	wImage    = mk_ctrl_btn( PGDEFAULT, "Image",    PG_DERIVE_AFTER,  &evtImage,    MVCOLOR_BLACK );
	wPreproc  = mk_ctrl_btn( PGDEFAULT, "Preproc.", PG_DERIVE_AFTER,  &evtPreproc,  MVCOLOR_BLACK );
	wSettings = mk_ctrl_btn( PGDEFAULT, "Settings", PG_DERIVE_AFTER,  &evtSettings, MVCOLOR_BLACK );
	wDisplay  = mk_ctrl_btn( PGDEFAULT, "Display",  PG_DERIVE_AFTER,  &evtDisplay,  MVCOLOR_BLACK );

	/* make the arrow for the popup buttons */
	setArrow( wImage );
	setArrow( wPreproc );
	setArrow( wSettings );
	setArrow( wDisplay );

	/* make a check box for the ROI on/off */
	wROIBox = mk_wBox( PGDEFAULT, PG_DERIVE_AFTER, PG_S_LEFT, 40 );
	wChboxROI = mk_chb( wROIBox, "ROI", &evtChboxROI );
	pgSetWidget( wChboxROI, PG_WP_ON, g_bROI, 0 );

	/* make scroller if needed */
	pghandle wLastBox = 0;
	switch( g_pPp->m_ThType )
	{
	case TH_MANUAL:
		mkManualThScroller( wROIBox, PG_DERIVE_AFTER );
		wLastBox = g_scrThreshold.wScrollLab;
		break;
	case TH_DYN_LINE:
		mkDynThScroller( wROIBox, PG_DERIVE_AFTER );
		wLastBox = g_scrMinDist.wScrollLab;
		break;
	default:
		wLastBox = wROIBox;
		break;
	}

	/* */
	if( g_TriggerMode != TRIG_SW && g_TriggerMode != TRIG_AUTO )
	{
		mk_wBox( wLastBox, PG_DERIVE_AFTER, PG_S_LEFT, 40 );
		wTrigMode = mk_chb( PGDEFAULT, "Trig.", &chTrigModeOnOff );
		pgSetWidget( wTrigMode, PG_WP_ON, 1, 0 );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return wBox;
}

//-------------------------------------------------------------------
/*! \brief Device initialization */
static void initDev( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( !DEV_Open( &g_deviceHandle, "matrixvision.ddc" ))
		iplExitDemo( "open device failed", IPL_GetErrorCode2( ));

	if( !DEV_Control( g_deviceHandle, DEV_SINGLE, 0 ))
		iplExitDemo( "set control single failed", IPL_GetErrorCode2( ));

	if( !DEV_InquireCaps( g_deviceHandle, DEV_TRIGGER, &g_TriggerMode ))
		iplExitDemo( "get trigger mode failed", IPL_GetErrorCode2( ));

	if( g_TriggerMode != TRIG_SW && g_TriggerMode != TRIG_AUTO )
	{
		int sw_trigmode = TRIG_SW;
		if( !DEV_ControlCaps( g_deviceHandle, DEV_TRIGGER, &sw_trigmode ))
			iplExitDemo( "set SW trigger mode failed", IPL_GetErrorCode2( ));
	}

	if( !DEV_Grab( g_deviceHandle, &g_imageHandle ))
		iplExitDemo( "grab image failed", IPL_GetErrorCode2( ));

	if( g_TriggerMode != TRIG_SW && g_TriggerMode != TRIG_AUTO )
	{
		if( !DEV_ControlCaps( g_deviceHandle, DEV_TRIGGER, &g_TriggerMode ))
			iplExitDemo( "reset the trigger mode failed", IPL_GetErrorCode2( ));
	}

	if( !IPL_BufInquire( g_imageHandle, IPL_XSIZE, &g_ImgWidth ))
		iplExitDemo( "inquier image width failed", IPL_GetErrorCode2( ));
	if( !IPL_BufInquire( g_imageHandle, IPL_YSIZE, &g_ImgHeight ))
		iplExitDemo( "inquier image height failed", IPL_GetErrorCode2( ));

	if( !IPL_GetPtr( g_imageHandle, IPL_LOCK_READ, 0, 0, 0, (long*)&g_ImgPitch, &pImage ))
		iplExitDemo( "get buffer pointer failed", IPL_GetErrorCode2( ));

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Converts a version string as delivered by the mvIMPACT modules to a struct
   that can be used to compare the version numbers */
SVersionNumber GetVersionStructFromString( char *strVersion)
{
        //strtok changes the entire string, so we work on a copy here
        char *copy = (char*)malloc(strlen(strVersion));
        strcpy(copy,strVersion);
	SVersionNumber tVersion( -1, -1, -1, -1 );
	char *sub = strtok(copy,".");
	if( sub != NULL )
	{
		tVersion.iMajor = atoi( sub );
		sub = strtok(NULL,".");
		if( sub != NULL )
		{
			tVersion.iMinor = atoi( sub );
		}

		sub = strtok(NULL,".");
		if( sub != NULL )
		{
			tVersion.iBuild = atoi( sub );
		}
		sub = strtok(NULL,".");
		if( sub != NULL )
		{

			tVersion.iBuild2 = atoi( sub );
		}

	}
	free(copy);
	return tVersion;
}

//-------------------------------------------------------------------
/*! \brief Compares two version strings as delivered by the mvIMPACT modules */
int CheckVersion(char* actVersion, char* requiredVersion)
{
	int bResult = false;
	SVersionNumber tactVerison = GetVersionStructFromString(actVersion);
	SVersionNumber trequiredVerison = GetVersionStructFromString(requiredVersion);

	if( tactVerison.iMajor > trequiredVerison.iMajor )
		bResult = true;
	else if ( tactVerison.iMajor == trequiredVerison.iMajor )
	{
		if ( tactVerison.iMinor > trequiredVerison.iMinor )
			bResult = true;
		else if ( tactVerison.iMinor == trequiredVerison.iMinor )
		{
			if( tactVerison.iBuild > trequiredVerison.iBuild )
				bResult = true;
			else if ( tactVerison.iBuild == trequiredVerison.iBuild )
			{
				if ( tactVerison.iBuild2 >= trequiredVerison.iBuild2 )
					bResult = true;
			}
			else
				bResult = false;
		}
		else
			bResult = false;
	}
	else
		bResult = false;
	return bResult;
}

//-------------------------------------------------------------------
int main( int argc, char *argv[] )
{
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "mvBlueLYNX OCR TOOL - " __DATE__ " - Ver. " APP_VERSION );

	/* init the GUI and register the application to the pgserver */
	pgInit( argc, argv );
	wApp = pgRegisterApp( PG_APP_NORMAL, msg, 0 );
	pgBind( PGDEFAULT, PG_WE_CLOSE, &evtClose, NULL );


	/* check version of OCR library */
	bool ocr_check = false;
	char ocrlibversion[20] = { 0 };
	if ( OCR_GetVersion(&ocrlibversion[0],sizeof(ocrlibversion)) )
	{
	  if ( CheckVersion(ocrlibversion,MIN_OCR_VERSION) == true )
	    {
	      ocr_check = true;
	    }

	}
	if (!ocr_check) {
	  char msg[256] = {0};
	  sprintf(msg,"At least version %s of mvIMPACT-OCR is required to run ocrtool",MIN_OCR_VERSION);
	  pgMessageDialog( "Version Conflict!!", msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	  exit( 0 );

	}




	/* init the device */
	initDev( );

	/* init the preprocessong */
	DMO_CHECK_ERR(( g_pPp = new CDmoPreproc( )) != NULL, -1 );

	/* init the processong (OCR) */
	DMO_CHECK_ERR(( g_pOCR = new CDmoOCR( )) != NULL, -1 );

	/* init the read/write file */
	DMO_CHECK_ERR(( g_pRWF = new CDmoRWFile( )) != NULL, -1 );

	/* read the init file if exists */
	initFromFile( );

	/* init the send data instance */
	if( g_bSendData )
	{
		DMO_CHECK_ERR(( g_pSend = new CSendDatattySX( )) != NULL, -1 );

		switch( g_DevNo )
		{
		case 0:
			g_pSend->InitSerial( TTYS0_DEV, g_Baudrate );
			break;
		case 1:
			g_pSend->InitSerial( TTYS1_DEV, g_Baudrate );
			break;
		}
	}

	/* make the buttons & check boxes & scroller */
	pghandle wCtrlBox = mkControls( wApp, PG_DERIVE_INSIDE );
	/* make the status line */
	wStatLine = mk_statline( wCtrlBox, PG_DERIVE_AFTER, 20 );

	/* make the status line labels */
	siOCRMode.msg = NO_MODE_STR;
	siRunMode.msg = STOP_LIVE_STR;
	siPolarity.msg = B2W_STR;
	get_s_len( );
	mkStatLineLabel( wStatLine );

	/* make the display */
	g_pDisp = new CDmoDisplay( );
	g_pDisp->setROI( g_roi );
	g_pDisp->mkDisplay( wStatLine, PG_DERIVE_AFTER, g_imageHandle );

	/* bottom status line */
	wStatLine2 = pgNewWidget( PG_WIDGET_TOOLBAR, PG_DERIVE_AFTER, wStatLine );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_BOTTOM,
				PG_WP_SIZE, 20,
				0 );
	mkStatLine2Label( wStatLine2 );

	/* check the processing type */
	if( g_ProcessType == START_LIVE )
		evtLiveOn( NULL );
	else if( g_ProcessType == START_READ )
		evtRead( NULL );

	/* needed for the ROI track */
	DISP_KeepRunning( NULL );

	/* enter the event loop */
	pgEventLoop( );

	return 0;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------


