//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - demo
// Mon Feb 14 11:50:05 CET 2005
//----------------------------------------------------------------------
#ifndef __rwfile_h__
#define __rwfile_h__

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <vl_base.h>
#include <vl_ocr.h>

#include "ocrtool.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \class CDmoRWFile
 *  \brief
 *  \author MATRIX VISION GmbH (td)
 *  \date 18.02.2005 */
class CDmoRWFile
{
public:
	/*----------------------------------------------------------------*/
	/*! \brief The constructor.
	 *  \param pFName the name of the file which should be written. */
	CDmoRWFile( const char* pFName = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		m_pF           = NULL;
		m_pFlPath      = NULL;
		m_pTmpFileName = NULL;
		m_bReadFlag    = false;

		if( pFName )
		{
			saveFileName( pFName );
		}
		else
		{
			m_pFileName = NULL;
		}
		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief The destructor. */
	virtual ~CDmoRWFile( )
	{
		dprintf((" ++ ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		freeCBuf( m_pFileName );
		freeCBuf( m_pTmpFileName );
		freeCBuf( m_pFlPath );

		closeFile( );
		dprintf((" -- ~%s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Close the file if it is opened. */
	void closeFile( )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		if( m_pF )
		{
			fclose( m_pF );
			m_pF = NULL;
		}
		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Opens a file for write and writes some comments into it.
	 *  \param pFName new file name
	 *  \param bBinary flag to write binary
 	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int startWriteToFile( const char* pFName = NULL, const bool bBinary = false )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		freeCBuf( m_pTmpFileName );
		freeCBuf( m_pFlPath );

		if( pFName )
		{
			freeCBuf( m_pFileName );
			DMO_CHECK_ERR(( ret = saveFileName( pFName )), -1 );
		}

		DMO_CHECK_ERR( m_pFileName != NULL, -1 );
		DMO_CHECK_ERR(( m_pFlPath = getFilePath( m_pFileName )) != NULL, -1 );

		dprintf((" == %s - m_pFlPath: >>%s<<\n", __FUNCTION__, m_pFlPath ));
		int tmpLen = strlen( m_pFlPath );

		char* pTmpFl = "/x.tmp";
		int tmpFlSize = tmpLen + strlen( pTmpFl ) + 1;
		DMO_CHECK_ERR(( m_pTmpFileName = new char[ tmpFlSize ] ) != NULL,  -1 );
		memset( m_pTmpFileName, 0, tmpFlSize );
		sprintf( m_pTmpFileName, "%s%s", m_pFlPath, pTmpFl );
		dprintf((" == %s - >>%s<< \n", __FUNCTION__, m_pTmpFileName ));

		char* wType;
		if( bBinary )
			wType = "wb";
		else
			wType = "w";

		DMO_CHECK_ERR(( m_pF = fopen( m_pTmpFileName, wType )) != NULL , -1 );

		writeCommentLine( );
		writeCommentLine( );
		fprintf( m_pF, "## MATRIX VISION GmbH - mvIMPACT " TOOL_PREFIX"\n" );
		fprintf( m_pF, "## " __DATE__ " " __TIME__ "\n" );
		fprintf( m_pF, "## Tool version: " APP_VERSION "\n" );
		writeCommentLine( );
		writeCommentLine( );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Closes the file after it was written.
 	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int stopWriteToFile( )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		DMO_CHECK_ERR( m_pF != NULL, -1 );

		writeNewLine( );
		writeCommentLine( );
		writeNewLine( );
		closeFile( );

		DMO_CHECK_ERR( m_pFileName != NULL, -1 );
		DMO_CHECK_ERR( m_pTmpFileName != NULL, -1 );

		dprintf((" == %s - m_pTmpFileName: >>%s<< m_pFileName: >>%s<<\n", __FUNCTION__, m_pTmpFileName, m_pFileName ));
		DMO_CHECK_ERR( rename( m_pTmpFileName, m_pFileName ) == 0, -1 );

		freeCBuf( m_pTmpFileName );
		freeCBuf( m_pFlPath );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Write a new comment line into the file.
	 *  \param pF other file descriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int writeCommentLine( FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		if( pF )
			fprintf( pF, "##----------------------------------------------\n" );
		else
		{
			DMO_CHECK_ERR( m_pF != NULL, -1 );
			fprintf( m_pF, "##----------------------------------------------\n" );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Write a new (empty) line into the file.
	 *  \param pF other file descriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int writeNewLine( FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		if( pF )
			fprintf( pF, "\n" );
		else
		{
			DMO_CHECK_ERR( m_pF != NULL, -1 );
			fprintf( m_pF, "\n" );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Write a new section into the file.
	 *  \param pName section name
	 *  \param pF other file descriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int writeSection( const char* pName, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		DMO_CHECK_ERR( pName != NULL, -1 );

		if( pF )
			fprintf( pF, "[%s]\n", pName );
		else
		{
			DMO_CHECK_ERR( m_pF != NULL, -1 );
			fprintf( m_pF, "[%s]\n", pName );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Write a new \b INT entry into the file.
	 *  \param pKeyName Entry name
	 *  \param value the new value
	 *  \param pF other file descriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int writeNewIEntry( const char* pKeyName, const int value, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );

		if( pF )
			fprintf( pF, "%s=%d\n", pKeyName, value );
		else
		{
			DMO_CHECK_ERR( m_pF != NULL, -1 );
			fprintf( m_pF, "%s=%d\n", pKeyName, value );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Write a new \b LONG entry into the file.
	 *  \param pKeyName Entry name
	 *  \param value the new value
	 *  \param pF other file descriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int writeNewLEntry( const char* pKeyName, const long value, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );

		if( pF )
			fprintf( pF, "%s=%ld\n", pKeyName, value );
		else
		{
			DMO_CHECK_ERR( m_pF != NULL, -1 );
			fprintf( m_pF, "%s=%ld\n", pKeyName, value );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Write a new \b DOUBLE entry into the file.
	 *  \param pKeyName Entry name
	 *  \param value the new value
	 *  \param pF other file descriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int writeNewDEntry( const char* pKeyName, const double value, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );

		if( pF )
			fprintf( pF, "%s=%f\n", pKeyName, value );
		else
		{
			DMO_CHECK_ERR( m_pF != NULL, -1 );
			fprintf( m_pF, "%s=%f\n", pKeyName, value );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Write a new \b STRING (CHAR) entry into the file.
	 *  \param pKeyName Entry name
	 *  \param value the new pValue
	 *  \param pF other file descriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int writeNewSEntry( const char* pKeyName, const char* pValue, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );
		DMO_CHECK_ERR( pValue != NULL, -1 );

		if( pF )
			fprintf( pF, "%s=%s\n", pKeyName, pValue );
		else
		{
			DMO_CHECK_ERR( m_pF != NULL, -1 );
			fprintf( m_pF, "%s=%s\n", pKeyName, pValue );
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Opens a file to read it.
	 *  \param pFName new file name
	 *  \param bBinary flag to write binary
 	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int startReadFile( const char* pFName = NULL, const bool bBinary = false )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_FALSE;

		if( pFName )
		{
			freeCBuf( m_pFileName );
			DMO_CHECK_ERR( saveFileName( pFName ), -1 );
		}

		char* rType;
		if( bBinary )
			rType = "rb";
		else
			rType = "r";

		if(( m_pF = fopen( m_pFileName, rType )) != NULL )
		{
			ret = IPL_TRUE;
			m_bReadFlag = true;
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Closes the file after reading it.
 	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int stopReadFile( )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		closeFile( );
		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read \c INT data from the file
	 *  \param pKeyName key name of the entry
	 *  \param min minimal range value
	 *  \param max maximal range value
	 *  \param defaultval defaukt value. This value will be replaced if the entry in the file is OK.
	 *  \param pF file dercriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int readIValue( const char* pKeyName, const int min, const int max, int& defaultval, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );

		int value;
		if( pF )
		{
			ret = readIData( pKeyName, pF, value );
		}
		else
		{
			ret = readIData( pKeyName, m_pF, value );
		}

		if( value >= min && value <= max )
		{
			defaultval = value;
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read \c LONG data from the file
	 *  \param pKeyName key name of the entry
	 *  \param min minimal range value
	 *  \param max maximal range value
	 *  \param defaultval defaukt value. This value will be replaced if the entry in the file is OK.
	 *  \param pF file dercriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int readLValue( const char* pKeyName, const long min, const long max, long& defaultval, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );

		dprintf((" =+ %s - pKeyName: %s min: %ld max: %ld defaultval: %ld\n", __FUNCTION__, pKeyName, min, max, defaultval ));
		long value;
		if( pF )
		{
			ret = readLData( pKeyName, pF, value );
		}
		else
		{
			ret = readLData( pKeyName, m_pF, value );
		}

		dprintf((" == %s - val: %ld\n", __FUNCTION__, value ));

		if( value >= min && value <= max )
		{
			defaultval = value;
		}

		dprintf((" =- %s - pKeyName: %s min: %ld max: %ld defaultval: %ld\n", __FUNCTION__, pKeyName, min, max, defaultval ));
		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read \c DOUBLE data from the file
	 *  \param pKeyName key name of the entry
	 *  \param min minimal range value
	 *  \param max maximal range value
	 *  \param defaultval defaukt value. This value will be replaced if the entry in the file is OK.
	 *  \param pF file dercriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int readDValue( const char* pKeyName, const double min, const double max, double& defaultval, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );

		double value;
		if( pF )
		{
			ret = readDData( pKeyName, pF, value );
		}
		else
		{
			ret = readDData( pKeyName, m_pF, value );
		}

		if( value >= min && value <= max )
		{
			defaultval = value;
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read \c BOOL data from the file
	 *  \param pKeyName key name of the entry
	 *  \param defaultval defaukt value. This value will be replaced if the entry in the file is OK.
	 *  \param pF file dercriptor
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int readBoolValue( const char* pKeyName, bool& defaultval, FILE* pF = NULL )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );

		int value;
		if( pF )
		{
			ret = readIData( pKeyName, pF, value );
		}
		else
		{
			ret = readIData( pKeyName, m_pF, value );
		}

		if( value >= false && value <= true )
		{
			defaultval = (bool)value;
		}

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
private:
	/*----------------------------------------------------------------*/
	FILE* m_pF;
	char* m_pFileName;
	char* m_pTmpFileName;
	char* m_pFlPath;
	bool m_bReadFlag;

	/*----------------------------------------------------------------*/
	/*! \brief Free a character buffer
	 *  \param pBuf the buffer */
	void freeCBuf( char*& pBuf )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		if( pBuf )
		{
			delete [] pBuf;
			pBuf = NULL;
		}
		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	}

	/*----------------------------------------------------------------*/
	/*! \brief Get file path
	 *  \param pFileName the file name.
	 *  \return Returns the path of the file name. */
	char* getFilePath( const char* pFileName )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		char* pRet = NULL;
		unsigned sLen;

		if(( sLen = strlen( pFileName )))
		{
			int pos = -1;
			/* check if a path exists */
			for( unsigned i = 0; i < sLen; i++ )
			{
				if( pFileName[i] == '/' )
				{
					pos = i;
				}
			}

			if( pos > 0 )
			{
				/* read the path */
				if(( pRet = new char[pos+1] ) != NULL )
				{
					memset( pRet, 0, pos+1 );
					strncpy( pRet, pFileName, pos );
				}
			}
			else
			{
				/* if no path found get the current folder name */
				int size = strlen( get_current_dir_name( )) + 1;
				if(( pRet = new char[size] ) != NULL )
				{
					memset( pRet, 0, size );
					sprintf( pRet, "%s", get_current_dir_name( ));
				}
			}
		}

		dprintf((" -- %s (%s: %d) (%p)\n", __FUNCTION__, __FILE__, __LINE__, pRet ));
		return pRet;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Save the file name to the class member.
	 *  \param pFName file name
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int saveFileName( const char* pFName )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		int ret = IPL_TRUE;
		DMO_CHECK_ERR( pFName != NULL, -1 );

		int len = strlen( pFName ) + 1;
		DMO_CHECK_ERR(( m_pFileName = new char[ len ] ) != NULL, -1 );

		memset( m_pFileName, 0, len );
		sprintf( m_pFileName, "%s", pFName );

		dprintf((" -- %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read \c INT data from the file
	 *  \param pKeyName key name of the entry
	 *  \param pF file dercriptor
	 *  \param retValue return value
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int readIData( const char* pKeyName, FILE* pF, int& retValue )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret = IPL_FALSE;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );
		DMO_CHECK_ERR( pF != NULL, -1 );

		char* pTmp = getFlEntry( pKeyName, pF );
		if( pTmp )
		{
			retValue = atoi( pTmp );
			ret = IPL_TRUE;
			delete [] pTmp;
		}

		dprintf((" -- %s (%s: %d) (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__, pKeyName, retValue ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read \c LONG data from the file
	 *  \param pKeyName key name of the entry
	 *  \param pF file dercriptor
	 *  \param retValue return value
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int readLData( const char* pKeyName, FILE* pF, long& retValue )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret = IPL_FALSE;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );
		DMO_CHECK_ERR( pF != NULL, -1 );

		char* pTmp = getFlEntry( pKeyName, pF );
		if( pTmp )
		{
			retValue = atol( pTmp );
			ret = IPL_TRUE;
			delete [] pTmp;
		}

		dprintf((" -- %s (%s: %d) (%s: %ld)\n", __FUNCTION__, __FILE__, __LINE__, pKeyName, retValue ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read \c DOUBLE data from the file
	 *  \param pKeyName key name of the entry
	 *  \param pF file dercriptor
	 *  \param retValue return value
	 *  \return If succeeds \c IPL_TRUE will be returned, if failed \c IPL_FALSE will be returned. */
	int readDData( const char* pKeyName, FILE* pF, double& retValue )
	{
		dprintf((" ++ %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));

		int ret = IPL_FALSE;

		DMO_CHECK_ERR( pKeyName != NULL, -1 );
		DMO_CHECK_ERR( pF != NULL, -1 );

		char* pTmp = getFlEntry( pKeyName, pF );
		if( pTmp )
		{
			retValue = atof( pTmp );
			ret = IPL_TRUE;
			delete [] pTmp;
		}

		dprintf((" -- %s (%s: %d) (%s: %f)\n", __FUNCTION__, __FILE__, __LINE__, pKeyName, retValue ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	/*! \brief Read the file entry
	 *  \param pKeyName key name of the entry
	 *  \param pF file dercriptor
	 *  \return The charcater read from the file will be returned. */
	char* getFlEntry( const char* pKeyName, FILE* pF )
	{
		char* pRet = NULL;

		if( pKeyName != NULL && pF != NULL )
		{
			char buf[MAX_CHAR_SIZE];
			memset( buf, 0, MAX_CHAR_SIZE );

			dprintf((" == %s - pKeyName >>%s<<\n", __FUNCTION__, pKeyName ));

			int keySize = strlen ( pKeyName );

			rewind( pF );
			for(  ; fgets( buf, MAX_CHAR_SIZE, pF ) != NULL;  )
			{
				dprintf((" == %s - buf: >>%s", __FUNCTION__, buf ));
				if( strncmp( buf, pKeyName, keySize ) == 0 && buf[keySize] == '=' )
				{
					buf[strlen(buf)-1] = 0;
					int size = strlen(buf) - keySize;
					pRet = new char[ size ];
					memset( pRet, 0, size );
					sprintf( pRet, "%s", buf + keySize+1 );

					dprintf((" = %s - pRet: >>%s<<\n", __FUNCTION__, pRet ));
					break;
				}
			}
		}

		dprintf((" -- %s (%s: %d) (%p)\n", __FUNCTION__, __FILE__, __LINE__, pRet ));
		return pRet;
	}

	/*----------------------------------------------------------------*/
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#endif	// __rwfile_h__
