#! /bin/make
############################################################################
# Section intended to be changed by the user
############################################################################
TARGET:=$(shell basename `pwd`)
OSTYPE := $(shell uname | cut -d _ -f 1 | tr [:upper:] [:lower:])

SAMPLE_PATH=../..

#----------------------------------------------------
# list of include search paths needed by the wrapper header
# every item must be preceded by "-I"
USERLIBINCLUDEPATH=			\
	-I$(SAMPLE_PATH)/userlib/include

#----------------------------------------------------
# list of library search paths needed by the wrapper
# every item must be preceded by "-L"
USERLIBLIBPATH=				\
	-L$(SAMPLE_PATH)/userlib/lib/$(LIB_OUT)

#----------------------------------------------------
# list of user libraries found in $(USERLIBLIBPATH), use the -l option
USERLIBLIBS=-luserlib

#----------------------------------------------------
# name of the created Python module
PYMODULENAME=userlib_mvIMPACT

#----------------------------------------------------
# name of the CPP wrapper header file
CPPHEADERNAME=$(PYMODULENAME).h

#----------------------------------------------------
# location of the CPP wrapper header file
CPPHEADERINCLUDEPATH=../include

#----------------------------------------------------
# impact libraries the wrapper needs, for simple buffer pointer access as in userlib this
# is usually only vl_base
IMPACT_LIBS =				\
		-lvl_base

############################################################################
# Section intended to be changed only by the experienced user
############################################################################

mvIMPACT=../../../..
WORK=$(mvIMPACT)/..

#----------------------------------------------------
include $(mvIMPACT)/Makefile.all.inc

#----------------------------------------------------
# cause of SWIG no SPEED_PREFIX possibile ( => delete it)
SPEED_PREFIX=


#----------------------------------------------------
ifeq ($(TARGETARCH),ppc)

	PYTHON_INCLUDE=$(mvIMPACT)/include/python2.3

	ifeq ($(OSTYPE),cygwin)
		LIB_OUT=$(OSTYPE)
	else
		LIB_OUT=$(TARGET)
	endif

else
	LIB_OUT=i386
	PYTHON_INCLUDE=/usr/include/python$(shell python -V 2>&1 |cut -c 8-10)
endif

#----------------------------------------------------
SWIG_INCLUDES=							\
	-I$(CPPHEADERINCLUDEPATH)

CXX_INCLUDES=							\
	-I $(mvIMPACT)/include				\
	-I $(mvIMPACT)/include/mvIMPACT_CPP	\
	-I $(mvIMPACT)/include/common		\
	-I $(mvIMPACT)/include/linux		\
	-I $(CPPHEADERINCLUDEPATH)			\
	-I $(PYTHON_INCLUDE)				\
	$(USERLIBINCLUDEPATH)				\
	$(GLOBINCFILES)						\
	-I ..

#----------------------------------------------------
LIB_DIRS=$(USERLIBLIBPATH) -L$(mvIMPACT)/lib/$(TARGET)

#----------------------------------------------------
LIBS=									\
	$(IMPACT_LIBS)						\
	$(USERLIBLIBS)

#----------------------------------------------------
CPPFLAGS=								\
	$(CXX_INCLUDES)						\
	$(CPPFLAGS_2)

#----------------------------------------------------
#----------------------------------------------------
BUILDDIR=../lib/$(LIB_OUT)

BUILD_PYNAME=../$(TARGET)/$(PYMODULENAME)

WRAP_FILE=$(PYMODULENAME)_wrap.cpp

PYEXTMODULENAME=_$(PYMODULENAME).so
BUILD_PYEXTNAME=$(BUILDDIR)/$(PYEXTMODULENAME)

#----------------------------------------------------
vpath %.cpp ..

#----------------------------------------------------
OBJECTS=$(PYMODULENAME)_wrap.o

#----------------------------------------------------
#----------------------------------------------------
.PHONY:	all build new install clean_mod clean strip $(WRAP_FILE) $(PYMODULENAME).py $(BUILD_PYEXTNAME)

#----------------------------------------------------
all: $(PYMODULENAME).py $(BUILD_PYEXTNAME)

#----------------------------------------------------
install: all
	@echo "***"
	@echo "*** Please copy manually the .so and .py files in lib/$(BUILDDIR)"
	@echo "*** to /usr/lib/python2.3/lib/site-packages"
	@echo "*** on your mvBlueLYNX or create a symlink appropriately"
	@echo "***"

#----------------------------------------------------
clean_mod clean:
	@rm -rf $(BUILDDIR)

#----------------------------------------------------
strip: all
	ls -lh $(BUILD_PYEXTNAME)
	$(TOOLS)strip $(BUILD_PYEXTNAME)
	ls -lh $(BUILD_PYEXTNAME)


#----------------------------------------------------
$(BUILDDIR):
	@mkdir -p $(BUILDDIR)

#----------------------------------------------------
$(WRAP_FILE): $(BUILDDIR) ../$(PYMODULENAME).i $(CPPHEADERINCLUDEPATH)/$(CPPHEADERNAME)
	swig -includeall -Wall -c++ -fvirtual -fastdispatch -noproxydel -fcompact -python -modern $(SWIG_INCLUDES) -o $@ -outdir $(BUILDDIR) ../$(PYMODULENAME).i

#----------------------------------------------------
$(PYMODULENAME).py: $(WRAP_FILE)

#----------------------------------------------------
$(BUILD_PYEXTNAME) : $(OBJECTS)
	$(CXX) -shared  $< -o $@ $(LIB_DIRS) $(LIBS) $(LDOPT)

#----------------------------------------------------
#----------------------------------------------------
